@@ -32,7 +32,6 @@
 #include "KoShapeStrokeModel.h"
 #include "KoShapeGroup.h"
 #include "KoToolProxy.h"
-#include "KoShapeManagerPaintingStrategy.h"
 #include "KoShapeShadow.h"
 #include "KoShapeLayer.h"
 #include "KoFilterEffect.h"
@@ -41,8 +40,12 @@
 #include "KoShapeBackground.h"
 #include <KoRTree.h>
 #include "KoClipPath.h"
+#include "KoClipMaskPainter.h"
 #include "KoShapePaintingContext.h"
 #include "KoViewConverter.h"
+#include "KisQPainterStateSaver.h"
+#include "KoSvgTextChunkShape.h"
+#include "KoSvgTextShape.h"
 
 #include <QPainter>
 #include <QTimer>
@@ -50,6 +53,14 @@
 
 #include "kis_painting_tweaks.h"
 
+bool KoShapeManager::Private::shapeUsedInRenderingTree(KoShape *shape)
+{
+    // FIXME: make more general!
+
+    return !dynamic_cast<KoShapeGroup*>(shape) &&
+            !dynamic_cast<KoShapeLayer*>(shape) &&
+            !(dynamic_cast<KoSvgTextChunkShape*>(shape) && !dynamic_cast<KoSvgTextShape*>(shape));
+}
 
 void KoShapeManager::Private::updateTree()
 {
@@ -65,21 +76,22 @@ void KoShapeManager::Private::updateTree()
     }
 
     foreach (KoShape *shape, aggregate4update) {
+        if (!shapeUsedInRenderingTree(shape)) continue;
+
         tree.remove(shape);
         QRectF br(shape->boundingRect());
-        strategy->adapt(shape, br);
         tree.insert(br, shape);
     }
 
     // do it again to see which shapes we intersect with _after_ moving.
-    foreach (KoShape *shape, aggregate4update)
+    foreach (KoShape *shape, aggregate4update) {
         detector.detect(tree, shape, shapeIndexesBeforeUpdate[shape]);
+    }
     aggregate4update.clear();
     shapeIndexesBeforeUpdate.clear();
 
     detector.fireSignals();
     if (selectionModified) {
-        selection->updateSizeAndPosition();
         emit q->selectionContentChanged();
     }
     if (anyModified) {
@@ -90,17 +102,17 @@ void KoShapeManager::Private::updateTree()
 void KoShapeManager::Private::paintGroup(KoShapeGroup *group, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
     QList<KoShape*> shapes = group->shapes();
-    qSort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
+    std::sort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
     Q_FOREACH (KoShape *child, shapes) {
         // we paint recursively here, so we do not have to check recursively for visibility
-        if (!child->isVisible())
+        if (!child->isVisible(false))
             continue;
         KoShapeGroup *childGroup = dynamic_cast<KoShapeGroup*>(child);
         if (childGroup) {
             paintGroup(childGroup, painter, converter, paintContext);
         } else {
             painter.save();
-            strategy->paint(child, painter, converter, paintContext);
+            KoShapeManager::renderSingleShape(child, painter, converter, paintContext);
             painter.restore();
         }
     }
@@ -121,28 +133,35 @@ KoShapeManager::KoShapeManager(KoCanvasBase *canvas)
     connect(d->selection, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
 }
 
-KoShapeManager::~KoShapeManager()
+void KoShapeManager::Private::unlinkFromShapesRecursively(const QList<KoShape*> &shapes)
 {
-    Q_FOREACH (KoShape *shape, d->shapes) {
-        shape->priv()->removeShapeManager(this);
+    Q_FOREACH (KoShape *shape, shapes) {
+        shape->priv()->removeShapeManager(q);
+
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
+        if (container) {
+            unlinkFromShapesRecursively(container->shapes());
+        }
     }
-    Q_FOREACH (KoShape *shape, d->additionalShapes) {
-        shape->priv()->removeShapeManager(this);
-    }
-    delete d;
 }
 
+KoShapeManager::~KoShapeManager()
+{
+    d->unlinkFromShapesRecursively(d->shapes);
+    d->shapes.clear();
+
+    delete d;
+}
 
 void KoShapeManager::setShapes(const QList<KoShape *> &shapes, Repaint repaint)
 {
     //clear selection
     d->selection->deselectAll();
-    Q_FOREACH (KoShape *shape, d->shapes) {
-        shape->priv()->removeShapeManager(this);
-    }
+    d->unlinkFromShapesRecursively(d->shapes);
     d->aggregate4update.clear();
     d->tree.clear();
     d->shapes.clear();
+
     Q_FOREACH (KoShape *shape, shapes) {
         addShape(shape, repaint);
     }
@@ -154,10 +173,12 @@ void KoShapeManager::addShape(KoShape *shape, Repaint repaint)
         return;
     shape->priv()->addShapeManager(this);
     d->shapes.append(shape);
-    if (! dynamic_cast<KoShapeGroup*>(shape) && ! dynamic_cast<KoShapeLayer*>(shape)) {
+
+    if (d->shapeUsedInRenderingTree(shape)) {
         QRectF br(shape->boundingRect());
         d->tree.insert(br, shape);
     }
+
     if (repaint == PaintShapeOnAdd) {
         shape->update();
     }
@@ -176,17 +197,6 @@ void KoShapeManager::addShape(KoShape *shape, Repaint repaint)
     detector.fireSignals();
 }
 
-void KoShapeManager::addAdditional(KoShape *shape)
-{
-    if (shape) {
-        if (d->additionalShapes.contains(shape)) {
-            return;
-        }
-        shape->priv()->addShapeManager(this);
-        d->additionalShapes.append(shape);
-    }
-}
-
 void KoShapeManager::remove(KoShape *shape)
 {
     Private::DetectCollision detector;
@@ -197,7 +207,10 @@ void KoShapeManager::remove(KoShape *shape)
     shape->priv()->removeShapeManager(this);
     d->selection->deselect(shape);
     d->aggregate4update.remove(shape);
-    d->tree.remove(shape);
+
+    if (d->shapeUsedInRenderingTree(shape)) {
+        d->tree.remove(shape);
+    }
     d->shapes.removeAll(shape);
 
     // remove the children of a KoShapeContainer
@@ -207,20 +220,34 @@ void KoShapeManager::remove(KoShape *shape)
             remove(containerShape);
         }
     }
-
-    // This signal is used in the annotation shape.
-    // FIXME: Is this really what we want?  (and shouldn't it be called shapeDeleted()?)
-    shapeRemoved(shape);
 }
 
-void KoShapeManager::removeAdditional(KoShape *shape)
+KoShapeManager::ShapeInterface::ShapeInterface(KoShapeManager *_q)
+    : q(_q)
 {
-    if (shape) {
-        shape->priv()->removeShapeManager(this);
-        d->additionalShapes.removeAll(shape);
-    }
 }
 
+void KoShapeManager::ShapeInterface::notifyShapeDestructed(KoShape *shape)
+{
+    q->d->selection->deselect(shape);
+    q->d->aggregate4update.remove(shape);
+
+    // we cannot access RTTI of the semi-destructed shape, so just
+    // unlink it lazily
+    if (q->d->tree.contains(shape)) {
+        q->d->tree.remove(shape);
+    }
+
+    q->d->shapes.removeAll(shape);
+}
+
+
+KoShapeManager::ShapeInterface *KoShapeManager::shapeInterface()
+{
+    return &d->shapeInterface;
+}
+
+
 void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter, bool forPrint)
 {
     d->updateTree();
@@ -240,7 +267,7 @@ void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter,
     // also filter shapes with a parent which has filter effects applied
     QList<KoShape*> sortedShapes;
     foreach (KoShape *shape, unsortedShapes) {
-        if (!shape->isVisible(true))
+        if (!shape->isVisible())
             continue;
         bool addShapeToList = true;
         // check if one of the shapes ancestors have filter effects
@@ -262,22 +289,12 @@ void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter,
         }
     }
 
-    qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+    std::sort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+
+    KoShapePaintingContext paintContext(d->canvas, forPrint); //FIXME
 
     foreach (KoShape *shape, sortedShapes) {
-        if (shape->parent() != 0 && shape->parent()->isClipped(shape))
-            continue;
-
-        painter.save();
-
-        // apply shape clipping
-        KoClipPath::applyClipping(shape, painter, converter);
-
-        // let the painting strategy paint the shape
-        KoShapePaintingContext paintContext(d->canvas, forPrint); //FIXME
-        d->strategy->paint(shape, painter, converter, paintContext);
-
-        painter.restore();
+        renderSingleShape(shape, painter, converter, paintContext);
     }
 
 #ifdef CALLIGRA_RTREE_DEBUG
@@ -297,6 +314,20 @@ void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter,
     }
 }
 
+void KoShapeManager::renderSingleShape(KoShape *shape, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+{
+    KisQPainterStateSaver saver(&painter);
+
+    // apply shape clipping
+    KoClipPath::applyClipping(shape, painter, converter);
+
+    // apply transformation
+    painter.setTransform(shape->absoluteTransformation(&converter) * painter.transform());
+
+    // paint the shape
+    paintShape(shape, painter, converter, paintContext);
+}
+
 void KoShapeManager::paintShape(KoShape *shape, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
     qreal transparency = shape->transparency(true);
@@ -310,15 +341,38 @@ void KoShapeManager::paintShape(KoShape *shape, QPainter &painter, const KoViewC
         painter.restore();
     }
     if (!shape->filterEffectStack() || shape->filterEffectStack()->isEmpty()) {
-        painter.save();
-        shape->paint(painter, converter, paintContext);
-        painter.restore();
-        if (shape->stroke()) {
-            painter.save();
-            shape->stroke()->paint(shape, painter, converter);
-            painter.restore();
+
+        QScopedPointer<KoClipMaskPainter> clipMaskPainter;
+        QPainter *shapePainter = &painter;
+
+        KoClipMask *clipMask = shape->clipMask();
+        if (clipMask) {
+            clipMaskPainter.reset(new KoClipMaskPainter(&painter, shape->boundingRect()));
+            shapePainter = clipMaskPainter->shapePainter();
         }
+
+        /**
+         * We expect the shape to save/restore the painter's state itself. Such design was not
+         * not always here, so we need a period of sanity checks to ensure all the shapes are
+         * ported correctly.
+         */
+        const QTransform sanityCheckTransformSaved = shapePainter->transform();
+
+        shape->paint(*shapePainter, converter, paintContext);
+        shape->paintStroke(*shapePainter, converter, paintContext);
+
+        KIS_SAFE_ASSERT_RECOVER(shapePainter->transform() == sanityCheckTransformSaved) {
+            shapePainter->setTransform(sanityCheckTransformSaved);
+        }
+
+        if (clipMask) {
+            shape->clipMask()->drawMask(clipMaskPainter->maskPainter(), shape);
+            clipMaskPainter->renderOnGlobalPainter();
+        }
+
     } else {
+        // TODO: clipping mask is not implemented for this case!
+
         // There are filter effects, then we need to prerender the shape on an image, to filter it
         QRectF shapeBound(QPointF(), shape->size());
         // First step, compute the rectangle used for the image
@@ -350,16 +404,12 @@ void KoShapeManager::paintShape(KoShape *shape, QPainter &painter, const KoViewC
                 // the childrens matrix contains the groups matrix as well
                 // so we have to compensate for that before painting the children
                 imagePainter.setTransform(group->absoluteTransformation(&converter).inverted(), true);
-                d->paintGroup(group, imagePainter, converter, paintContext);
+                Private::paintGroup(group, imagePainter, converter, paintContext);
             } else {
                 imagePainter.save();
                 shape->paint(imagePainter, converter, paintContext);
+                shape->paintStroke(imagePainter, converter, paintContext);
                 imagePainter.restore();
-                if (shape->stroke()) {
-                    imagePainter.save();
-                    shape->stroke()->paint(shape, imagePainter, converter);
-                    imagePainter.restore();
-                }
                 imagePainter.end();
             }
         }
@@ -434,11 +484,11 @@ KoShape *KoShapeManager::shapeAt(const QPointF &position, KoFlake::ShapeSelectio
 {
     d->updateTree();
     QList<KoShape*> sortedShapes(d->tree.contains(position));
-    qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+    std::sort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
     KoShape *firstUnselectedShape = 0;
     for (int count = sortedShapes.count() - 1; count >= 0; count--) {
         KoShape *shape = sortedShapes.at(count);
-        if (omitHiddenShapes && ! shape->isVisible(true))
+        if (omitHiddenShapes && ! shape->isVisible())
             continue;
         if (! shape->hitTest(position))
             continue;
@@ -479,28 +529,39 @@ KoShape *KoShapeManager::shapeAt(const QPointF &position, KoFlake::ShapeSelectio
     return 0; // missed everything
 }
 
-QList<KoShape *> KoShapeManager::shapesAt(const QRectF &rect, bool omitHiddenShapes)
+QList<KoShape *> KoShapeManager::shapesAt(const QRectF &rect, bool omitHiddenShapes, bool containedMode)
 { 
     d->updateTree();
-    QList<KoShape*> intersectedShapes(d->tree.intersects(rect));
-    
-    for (int count = intersectedShapes.count() - 1; count >= 0; count--) {
+    QList<KoShape*> shapes(containedMode ? d->tree.contained(rect) : d->tree.intersects(rect));
+
+    for (int count = shapes.count() - 1; count >= 0; count--) {
      
-        KoShape *shape = intersectedShapes.at(count);
+        KoShape *shape = shapes.at(count);
        
-        if (omitHiddenShapes && ! shape->isVisible(true)) {
-            intersectedShapes.removeAt(count);
+        if (omitHiddenShapes && !shape->isVisible()) {
+            shapes.removeAt(count);
         } else {
             const QPainterPath outline = shape->absoluteTransformation(0).map(shape->outline());
-            if (! outline.intersects(rect) && ! outline.contains(rect)) { 
-                intersectedShapes.removeAt(count);
+
+            if (!containedMode && !outline.intersects(rect) && !outline.contains(rect)) {
+                shapes.removeAt(count);
+
+            } else if (containedMode) {
+
+                QPainterPath containingPath;
+                containingPath.addRect(rect);
+
+                if (!containingPath.contains(outline)) {
+                    shapes.removeAt(count);
+                }
             }
         }
     }
-    return intersectedShapes;
+
+    return shapes;
 }
 
-void KoShapeManager::update(QRectF &rect, const KoShape *shape, bool selectionHandles)
+void KoShapeManager::update(const QRectF &rect, const KoShape *shape, bool selectionHandles)
 {
     d->canvas->updateCanvas(rect);
     if (selectionHandles && d->selection->isSelected(shape)) {
@@ -512,7 +573,7 @@ void KoShapeManager::update(QRectF &rect, const KoShape *shape, bool selectionHa
 void KoShapeManager::notifyShapeChanged(KoShape *shape)
 {
     Q_ASSERT(shape);
-    if (d->aggregate4update.contains(shape) || d->additionalShapes.contains(shape)) {
+    if (d->aggregate4update.contains(shape)) {
         return;
     }
     const bool wasEmpty = d->aggregate4update.isEmpty();
@@ -524,9 +585,10 @@ void KoShapeManager::notifyShapeChanged(KoShape *shape)
         Q_FOREACH (KoShape *child, container->shapes())
             notifyShapeChanged(child);
     }
-    if (wasEmpty && !d->aggregate4update.isEmpty())
-        QTimer::singleShot(100, this, SLOT(updateTree()));
-    emit shapeChanged(shape);
+
+    if (wasEmpty && !d->aggregate4update.isEmpty()) {
+        d->updateTreeCompressor.start();
+    }
 }
 
 QList<KoShape*> KoShapeManager::shapes() const
@@ -539,7 +601,7 @@ QList<KoShape*> KoShapeManager::topLevelShapes() const
     QList<KoShape*> shapes;
     // get all toplevel shapes
     Q_FOREACH (KoShape *shape, d->shapes) {
-        if (shape->parent() == 0) {
+        if (!shape->parent() || dynamic_cast<KoShapeLayer*>(shape->parent())) {
             shapes.append(shape);
         }
     }
@@ -551,40 +613,6 @@ KoSelection *KoShapeManager::selection() const
     return d->selection;
 }
 
-void KoShapeManager::suggestChangeTool(KoPointerEvent *event)
-{
-    QList<KoShape*> shapes;
-
-    KoShape *clicked = shapeAt(event->point);
-    if (clicked) {
-        if (! selection()->isSelected(clicked)) {
-            selection()->deselectAll();
-            selection()->select(clicked);
-        }
-        shapes.append(clicked);
-    }
-
-    QList<KoShape*> shapes2;
-    foreach (KoShape *shape, shapes) {
-        QSet<KoShape*> delegates = shape->toolDelegates();
-        if (delegates.isEmpty()) {
-            shapes2.append(shape);
-        } else {
-            foreach (KoShape *delegatedShape, delegates) {
-                shapes2.append(delegatedShape);
-            }
-        }
-    }
-    KoToolManager::instance()->switchToolRequested(
-        KoToolManager::instance()->preferredToolForSelection(shapes2));
-}
-
-void KoShapeManager::setPaintingStrategy(KoShapeManagerPaintingStrategy *strategy)
-{
-    delete d->strategy;
-    d->strategy = strategy;
-}
-
 KoCanvasBase *KoShapeManager::canvas()
 {
     return d->canvas;
