@@ -20,6 +20,7 @@
 #ifndef MOCKSHAPES_H
 #define MOCKSHAPES_H
 
+#include <KoSelectedShapesProxySimple.h>
 #include <KoShapeGroup.h>
 #include <KoCanvasBase.h>
 #include <KoShapeBasedDocumentBase.h>
@@ -30,44 +31,47 @@
 #include "KoSnapData.h"
 #include "KoUnit.h"
 
-class MockShape : public KoShape
+#include "kritaflake_export.h"
+
+
+class KRITAFLAKE_EXPORT MockShape : public KoShape
 {
 public:
     MockShape() : paintedCount(0) {}
-    void paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) {
+    void paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) override {
         Q_UNUSED(painter);
         Q_UNUSED(converter);
         //qDebug() << "Shape" << kBacktrace( 10 );
         paintedCount++;
     }
-    virtual void saveOdf(KoShapeSavingContext &) const {}
-    virtual bool loadOdf(const KoXmlElement &, KoShapeLoadingContext &) {
+    void saveOdf(KoShapeSavingContext &) const override {}
+    bool loadOdf(const KoXmlElement &, KoShapeLoadingContext &) override {
         return true;
     }
     int paintedCount;
 };
 
-class MockContainer : public KoShapeContainer
+class KRITAFLAKE_EXPORT MockContainer : public KoShapeContainer
 {
 public:
     MockContainer(KoShapeContainerModel *model = 0) : KoShapeContainer(model), paintedCount(0) {}
-    void paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) {
+    void paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) override {
         Q_UNUSED(painter);
         Q_UNUSED(converter);
         //qDebug() << "Container:" << kBacktrace( 10 );
         paintedCount++;
     }
 
-    virtual void saveOdf(KoShapeSavingContext &) const {}
-    virtual bool loadOdf(const KoXmlElement &, KoShapeLoadingContext &) {
+    void saveOdf(KoShapeSavingContext &) const override {}
+    bool loadOdf(const KoXmlElement &, KoShapeLoadingContext &) override {
         return true;
     }
     int paintedCount;
 };
 
-class MockGroup : public KoShapeGroup
+class KRITAFLAKE_EXPORT MockGroup : public KoShapeGroup
 {
-    void paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) {
+    void paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &) override {
         Q_UNUSED(painter);
         Q_UNUSED(converter);
     }
@@ -75,72 +79,107 @@ class MockGroup : public KoShapeGroup
 
 class KoToolProxy;
 
-class MockCanvas : public KoCanvasBase
+class KRITAFLAKE_EXPORT MockShapeController : public KoShapeBasedDocumentBase
 {
+public:
+    void addShapes(const QList<KoShape*> shapes) override {
+        Q_FOREACH (KoShape *shape, shapes) {
+            m_shapes.insert(shape);
+            if (m_shapeManager) {
+                m_shapeManager->addShape(shape);
+            }
+        }
+    }
+    void removeShape(KoShape* shape) override {
+        m_shapes.remove(shape);
+        if (m_shapeManager) {
+            m_shapeManager->remove(shape);
+        }
+    }
+    bool contains(KoShape* shape) {
+        return m_shapes.contains(shape);
+    }
+
+    void setShapeManager(KoShapeManager *shapeManager) {
+        m_shapeManager = shapeManager;
+    }
+
+    QRectF documentRectInPixels() const override {
+        return QRectF(0,0,100,100);
+    }
+
+    qreal pixelsPerInch() const override {
+        return 72.0;
+    }
+
+private:
+    QSet<KoShape * > m_shapes;
+    KoShapeManager *m_shapeManager = 0;
+};
+
+class KRITAFLAKE_EXPORT MockCanvas : public KoCanvasBase
+{
+    Q_OBJECT
 public:
     MockCanvas(KoShapeBasedDocumentBase *aKoShapeBasedDocumentBase =0)//made for TestSnapStrategy.cpp
-            : KoCanvasBase(aKoShapeBasedDocumentBase), m_shapeManager(new KoShapeManager(this)) {}
-    ~MockCanvas() {}
+            : KoCanvasBase(aKoShapeBasedDocumentBase),
+              m_shapeManager(new KoShapeManager(this)),
+              m_selectedShapesProxy(new KoSelectedShapesProxySimple(m_shapeManager.data()))
+    {
+        if (MockShapeController *controller = dynamic_cast<MockShapeController*>(aKoShapeBasedDocumentBase)) {
+            controller->setShapeManager(m_shapeManager.data());
+        }
+    }
+
+    ~MockCanvas() override {}
     void setHorz(qreal pHorz){
         m_horz = pHorz;
     }
     void setVert(qreal pVert){
         m_vert = pVert;
     }
-    void gridSize(QPointF *offset, QSizeF *spacing) const {
+    void gridSize(QPointF *offset, QSizeF *spacing) const override {
         Q_UNUSED(offset);
 
         spacing->setWidth(m_horz);
         spacing->setHeight(m_vert);
     }
-    bool snapToGrid() const  {
+    bool snapToGrid() const override  {
         return true;
     }
-    void addCommand(KUndo2Command*) { }
-    KoShapeManager *shapeManager() const  {
-        return m_shapeManager;
+    void addCommand(KUndo2Command*) override { }
+    KoShapeManager *shapeManager() const override  {
+        return m_shapeManager.data();
     }
-    void updateCanvas(const QRectF&)  {}
-    KoToolProxy * toolProxy() const {
+    KoSelectedShapesProxy *selectedShapesProxy() const override {
+        return m_selectedShapesProxy.data();
+    }
+    void updateCanvas(const QRectF&) override  {}
+    KoToolProxy * toolProxy() const override {
         return 0;
     }
-    KoViewConverter *viewConverter() const {
+    KoViewConverter *viewConverter() const override {
         return 0;
     }
-    QWidget* canvasWidget() {
+    QWidget* canvasWidget() override {
         return 0;
     }
-    const QWidget* canvasWidget() const {
+    const QWidget* canvasWidget() const override {
         return 0;
     }
-    KoUnit unit() const {
+    KoUnit unit() const override {
         return KoUnit(KoUnit::Millimeter);
     }
-    void updateInputMethodInfo() {}
-    void setCursor(const QCursor &) {}
+    void updateInputMethodInfo() override {}
+    void setCursor(const QCursor &) override {}
     private:
-        KoShapeManager *m_shapeManager;
+        QScopedPointer<KoShapeManager> m_shapeManager;
+        QScopedPointer<KoSelectedShapesProxy> m_selectedShapesProxy;
         qreal m_horz;
         qreal m_vert;
 };
 
-class MockShapeController : public KoShapeBasedDocumentBase
-{
-public:
-    void addShape(KoShape* shape) {
-        m_shapes.insert(shape);
-    }
-    void removeShape(KoShape* shape) {
-        m_shapes.remove(shape);
-    }
-    bool contains(KoShape* shape) {
-        return m_shapes.contains(shape);
-    }
-private:
-    QSet<KoShape * > m_shapes;
-};
-
-class MockContainerModel : public KoShapeContainerModel
+class KRITAFLAKE_EXPORT MockContainerModel : public KoShapeContainerModel
 {
 public:
     MockContainerModel() {
@@ -148,47 +187,43 @@ public:
     }
 
     /// reimplemented
-    void add(KoShape *child) {
+    void add(KoShape *child) override {
         m_children.append(child); // note that we explicitly do not check for duplicates here!
     }
     /// reimplemented
-    void remove(KoShape *child) {
+    void remove(KoShape *child) override {
         m_children.removeAll(child);
     }
 
     /// reimplemented
-    void setClipped(const KoShape *, bool) { }  // ignored
+    void setClipped(const KoShape *, bool) override { }  // ignored
     /// reimplemented
-    bool isClipped(const KoShape *) const {
+    bool isClipped(const KoShape *) const override {
         return false;
     }// ignored
     /// reimplemented
-    bool isChildLocked(const KoShape *child) const {
-        return child->isGeometryProtected();
-    }
-    /// reimplemented
-    int count() const {
+    int count() const override {
         return m_children.count();
     }
     /// reimplemented
-    QList<KoShape*> shapes() const {
+    QList<KoShape*> shapes() const override {
         return m_children;
     }
     /// reimplemented
-    void containerChanged(KoShapeContainer *, KoShape::ChangeType) {
+    void containerChanged(KoShapeContainer *, KoShape::ChangeType) override {
         m_containerChangedCalled++;
     }
     /// reimplemented
-    void proposeMove(KoShape *, QPointF &) {
+    void proposeMove(KoShape *, QPointF &) override {
         m_proposeMoveCalled++;
     }
     /// reimplemented
-    void childChanged(KoShape *, KoShape::ChangeType) {
+    void childChanged(KoShape *, KoShape::ChangeType) override {
         m_childChangedCalled++;
     }
-    void setInheritsTransform(const KoShape *, bool) {
+    void setInheritsTransform(const KoShape *, bool) override {
     }
-    bool inheritsTransform(const KoShape *) const {
+    bool inheritsTransform(const KoShape *) const override {
         return false;
     }
 
