@@ -34,17 +34,21 @@ struct Q_DECL_HIDDEN KisBaseNode::Private
     KoProperties properties;
     KisBaseNode::Property hack_visible; //HACK
     QUuid id;
+    QMap<QString, KisKeyframeChannel*> keyframeChannels;
+    QScopedPointer<KisScalarKeyframeChannel> opacityChannel;
+
+    bool systemLocked;
     bool collapsed;
     bool supportsLodMoves;
-
-    QMap<QString, KisKeyframeChannel*> keyframeChannels;
     bool animated;
     bool useInTimeline;
 
-    QScopedPointer<KisScalarKeyframeChannel> opacityChannel;
-
     Private()
-        : animated(false)
+        : id(QUuid::createUuid())
+        , systemLocked(false)
+        , collapsed(false)
+        , supportsLodMoves(false)
+        , animated(false)
         , useInTimeline(false)
     {
     }
@@ -52,6 +56,7 @@ struct Q_DECL_HIDDEN KisBaseNode::Private
     Private(const Private &rhs)
         : compositeOp(rhs.compositeOp),
           id(QUuid::createUuid()),
+          systemLocked(false),
           collapsed(rhs.collapsed),
           supportsLodMoves(rhs.supportsLodMoves),
           animated(rhs.animated),
@@ -71,7 +76,7 @@ KisBaseNode::KisBaseNode()
     /**
      * Be cautious! These two calls are vital to warm-up KoProperties.
      * We use it and its QMap in a threaded environment. This is not
-     * officially suported by Qt, but our environment guarantees, that
+     * officially supported by Qt, but our environment guarantees, that
      * there will be the only writer and several readers. Whilst the
      * value of the QMap is boolean and there are no implicit-sharing
      * calls provocated, it is safe to work with it in such an
@@ -83,8 +88,6 @@ KisBaseNode::KisBaseNode()
     setSupportsLodMoves(true);
 
     m_d->compositeOp = COMPOSITE_OVER;
-
-    setUuid(QUuid::createUuid());
 }
 
 
@@ -93,6 +96,10 @@ KisBaseNode::KisBaseNode(const KisBaseNode & rhs)
     , KisShared()
     , m_d(new Private(*rhs.m_d))
 {
+    if (rhs.m_d->opacityChannel) {
+        m_d->opacityChannel.reset(new KisScalarKeyframeChannel(*rhs.m_d->opacityChannel, 0));
+        m_d->keyframeChannels.insert(m_d->opacityChannel->id(), m_d->opacityChannel.data());
+    }
 }
 
 KisBaseNode::~KisBaseNode()
@@ -125,9 +132,8 @@ void KisBaseNode::setOpacity(quint8 val)
 
     if (opacity() == val) return;
 
-    nodeProperties().setProperty("opacity", val);
+    setNodeProperty("opacity", val);
 
-    baseNodeChangedCallback();
     baseNodeInvalidateAllFramesCallback();
 }
 
@@ -170,11 +176,17 @@ void KisBaseNode::setSectionModelProperties(const KisBaseNode::PropertyList &pro
     setUserLocked(properties.at(1).state.toBool());
 }
 
-KoProperties & KisBaseNode::nodeProperties() const
+const KoProperties & KisBaseNode::nodeProperties() const
 {
     return m_d->properties;
 }
 
+void KisBaseNode::setNodeProperty(const QString & name, const QVariant & value)
+{
+    m_d->properties.setProperty(name, value);
+    baseNodeChangedCallback();
+}
+
 void KisBaseNode::mergeNodeProperties(const KoProperties & properties)
 {
     QMapIterator<QString, QVariant> iter = properties.propertyIterator();
@@ -333,9 +345,9 @@ void KisBaseNode::setSupportsLodMoves(bool value)
 }
 
 
-QList<KisKeyframeChannel*> KisBaseNode::keyframeChannels() const
+QMap<QString, KisKeyframeChannel*> KisBaseNode::keyframeChannels() const
 {
-    return m_d->keyframeChannels.values();
+    return m_d->keyframeChannels;
 }
 
 KisKeyframeChannel * KisBaseNode::getKeyframeChannel(const QString &id) const
