@@ -92,7 +92,7 @@ public:
         bool stateInStasis;
 
         bool operator==(const Property &rhs) const {
-            return rhs.name == name;
+            return rhs.name == name && rhs.state == state;
         }
 
         Property(): isMutable( false ) { }
@@ -131,7 +131,7 @@ public:
     /**
      * Delete this node
      */
-    virtual ~KisBaseNode();
+    ~KisBaseNode() override;
 
 
     /**
@@ -235,7 +235,7 @@ public:
      */
     virtual QIcon icon() const {
         return QIcon();
-    };
+    }
 
     /**
      * Return a the properties of this base node (locked, visible etc,
@@ -263,11 +263,18 @@ public:
      * Return all the properties of this layer as a KoProperties-based
      * serializable key-value list.
      */
-    KoProperties & nodeProperties() const;
+    const KoProperties & nodeProperties() const;
+
+    /**
+     * Set a node property.
+     * @param name name of the property to be set.
+     * @param value value to set the property to.
+     */
+    void setNodeProperty(const QString & name, const QVariant & value);
 
     /**
      * Merge the specified properties with the properties of this
-     * layer. Whereever these properties overlap, the value of the
+     * layer. Wherever these properties overlap, the value of the
      * node properties is changed. No properties on the node are
      * deleted. If there are new properties in this list, they will be
      * added on the node.
@@ -321,6 +328,15 @@ public:
      */
     virtual QImage createThumbnail(qint32 w, qint32 h);
 
+    /**
+     * @return a thumbnail in requested size for the defined timestamp.
+     * The thumbnail is a rgba Image and may have transparent parts.
+     * Returns a fully transparent QImage of the requested size if the
+     * current node type cannot generate a thumbnail. If the requested
+     * size is too big, return a null QImage.
+     */
+    virtual QImage createThumbnailForFrame(qint32 w, qint32 h, int time);
+
     /**
      * Ask this node to re-read the pertinent settings from the krita
      * configuration.
@@ -332,6 +348,9 @@ public:
      * @return true if this node is visible (i.e, active (except for
      * selection masks where visible and active properties are
      * different)) in the graph
+     *
+     * @param bool recursive if true, check whether all parents of
+     * this node are visible as well.
      */
     virtual bool visible(bool recursive = false) const;
 
@@ -349,7 +368,7 @@ public:
      * @param visible the new visibility state
      * @param isLoading if true, the property is set during loading.
      */
-    virtual void setVisible(bool visibile, bool loading = false);
+    virtual void setVisible(bool visible, bool loading = false);
 
     /**
      * Return the locked status of this node. Locked nodes cannot be
@@ -361,26 +380,7 @@ public:
      * Set the locked status of this node. Locked nodes cannot be
      * edited.
      */
-    void setUserLocked(bool l);
-
-    /**
-     * Return the locked status of this node. System Locked nodes indicates
-     * that an algorithm is processing them and that an other
-     * algorithm need to wait before accessing it.
-     */
-    bool systemLocked() const;
-
-    /**
-     * Set the locked status of this node. System Locked nodes indicates
-     * that an algorithm is processing them and that an other
-     * algorithm need to wait before accessing it.
-     *
-     * A KisNode will update the layer model when the lock is released.
-     *
-     * @param l lock state
-     * @param update set false if the tools shouldn't be locked
-     */
-    virtual void setSystemLocked(bool l, bool update = true);
+    virtual void setUserLocked(bool l);
 
     /**
      * @return true if the node can be edited:
@@ -475,7 +475,7 @@ public:
      * Return the keyframe channels associated with this node
      * @return list of keyframe channels
      */
-    QList<KisKeyframeChannel *> keyframeChannels() const;
+    QMap<QString, KisKeyframeChannel*> keyframeChannels() const;
 
     /**
      * Get the keyframe channel with given id.
@@ -507,7 +507,7 @@ protected:
      * the node stack
      */
     virtual KisBaseNodeSP parentCallback() const {
-        return 0;
+        return KisBaseNodeSP();
     }
 
     virtual void notifyParentVisibilityChanged(bool value) {
@@ -557,10 +557,6 @@ Q_SIGNALS:
      * This signal is emitted when the node is locked or unlocked with \ref setUserLocked.
      */
     void userLockingChanged(bool);
-    /**
-     * This signal is emitted when the node is locked or unlocked with \ref setSystemLocked.
-     */
-    void systemLockingChanged(bool);
 
     void keyframeChannelAdded(KisKeyframeChannel *channel);
 
