@@ -73,10 +73,10 @@ public:
     **/
     KisLayer(KisImageWSP image, const QString &name, quint8 opacity);
     KisLayer(const KisLayer& rhs);
-    virtual ~KisLayer();
+    ~KisLayer() override;
 
     /// returns the image's colorSpace or null, if there is no image
-    virtual const KoColorSpace * colorSpace() const override;
+    const KoColorSpace * colorSpace() const override;
 
     /// returns the layer's composite op for the colorspace of the layer's parent.
     const KoCompositeOp * compositeOp() const override;
@@ -87,7 +87,7 @@ public:
     /**
      * \see a comment in KisNode::projectionPlane()
      */
-    virtual KisAbstractProjectionPlaneSP projectionPlane() const override;
+    KisAbstractProjectionPlaneSP projectionPlane() const override;
 
     /**
      * The projection plane representing the layer itself without any
@@ -111,7 +111,7 @@ public:
      * Return the layer data before the effect masks have had their go
      * at it.
      */
-    virtual KisPaintDeviceSP original() const override = 0;
+    KisPaintDeviceSP original() const override = 0;
 
     /**
      * @return the selection associated with this layer, if there is
@@ -125,8 +125,8 @@ public:
      */
     virtual KisSelectionSP selection() const;
 
-    virtual KisBaseNode::PropertyList sectionModelProperties() const override;
-    virtual void setSectionModelProperties(const KisBaseNode::PropertyList &properties) override;
+    KisBaseNode::PropertyList sectionModelProperties() const override;
+    void setSectionModelProperties(const KisBaseNode::PropertyList &properties) override;
 
     /**
      * set/unset the channel flag for the alpha channel of this layer
@@ -186,7 +186,7 @@ public:
      * Decendands override this to create specific merged types when possible.
      * The KisLayer one creates a KisPaintLayerSP via a bitBlt, and can work on all layer types.
      *
-     * Decendants that perform there own version do NOT call KisLayer::createMergedLayer
+     * Descendants that perform their own version do NOT call KisLayer::createMergedLayer
      */
     virtual KisLayerSP createMergedLayerTemplate(KisLayerSP prevLayer);
     virtual void fillMergedLayerTemplate(KisLayerSP dstLayer, KisLayerSP prevLayer);
@@ -224,6 +224,11 @@ public:
      */
     void updateClones(const QRect &rect);
 
+    /**
+     * Informs this layers that its masks might have changed.
+     */
+    void notifyChildMaskChanged(KisNodeSP changedChildMask);
+
 public:
     qint32 x() const override;
     qint32 y() const override;
@@ -245,6 +250,8 @@ public:
 
     QImage createThumbnail(qint32 w, qint32 h) override;
 
+    QImage createThumbnailForFrame(qint32 w, qint32 h, int time) override;
+
 public:
     /**
      * Returns true if there are any effect masks present
@@ -254,7 +261,12 @@ public:
     /**
      * @return the list of effect masks
      */
-    QList<KisEffectMaskSP> effectMasks(KisNodeSP lastNode = 0) const;
+    const QList<KisEffectMaskSP> &effectMasks() const;
+
+    /**
+     * @return the list of effect masks up to a certain node
+     */
+    QList<KisEffectMaskSP> effectMasks(KisNodeSP lastNode) const;
 
     /**
      * Get the group layer that contains this layer.
@@ -270,6 +282,8 @@ protected:
     // override from KisNode
     QRect changeRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const override;
 
+    void childNodeChanged(KisNodeSP changedChildNode) override;
+
 protected:
 
     /**
@@ -369,11 +383,21 @@ protected:
                      KisNodeSP filthyNode, KisNodeSP lastNode) const;
 
     bool canMergeAndKeepBlendOptions(KisLayerSP otherLayer);
+
+    void updateSelectionMask();
+
+    void updateEffectMasks();
+
+    QList<KisEffectMaskSP> searchEffectMasks(KisNodeSP lastNode) const;
+
 private:
     friend class KisLayerProjectionPlane;
     friend class KisTransformMask;
     friend class KisLayerTest;
 
+private:
+    QRect layerExtentImpl(bool exactBounds) const;
+
 private:
     struct Private;
     Private * const m_d;
