@@ -50,6 +50,7 @@
 #include "kis_layer_properties_icons.h"
 #include "lazybrush/kis_colorize_mask.h"
 #include "commands/kis_node_property_list_command.h"
+#include <KisDelayedUpdateNodeInterface.h>
 #include "krita_utils.h"
 
 
@@ -58,7 +59,7 @@ namespace KisLayerUtils {
     void fetchSelectionMasks(KisNodeList mergedNodes, QVector<KisSelectionMaskSP> &selectionMasks)
     {
         foreach (KisNodeSP node, mergedNodes) {
-            KisLayerSP layer = dynamic_cast<KisLayer*>(node.data());
+            KisLayerSP layer = qobject_cast<KisLayer*>(node.data());
 
             KisSelectionMaskSP mask;
 
@@ -114,10 +115,6 @@ namespace KisLayerUtils {
             mergedNodes << prevLayer;
             return mergedNodes;
         }
-
-        KisLayerSP dstLayer() {
-            return dynamic_cast<KisLayer*>(dstNode.data());
-        }
     };
 
     struct MergeMultipleInfo : public MergeDownInfoBase {
@@ -281,6 +278,19 @@ namespace KisLayerUtils {
         MergeDownInfoBaseSP m_info;
     };
 
+    struct RefreshDelayedUpdateLayers : public KUndo2Command {
+        RefreshDelayedUpdateLayers(MergeDownInfoBaseSP info) : m_info(info) {}
+
+        void redo() override {
+            foreach (KisNodeSP node, m_info->allSrcNodes()) {
+                forceAllDelayedNodesUpdate(node);
+            }
+        }
+
+    private:
+        MergeDownInfoBaseSP m_info;
+    };
+
     struct KeepMergedNodesSelected : public KisCommandUtils::AggregateCommand {
         KeepMergedNodesSelected(MergeDownInfoSP info, bool finalizing)
             : m_singleInfo(info),
@@ -331,7 +341,7 @@ namespace KisLayerUtils {
         CreateMergedLayer(MergeDownInfoSP info) : m_info(info) {}
 
         void populateChildCommands() override {
-            // actual merging done by KisLayer::createMergedLayer (or specialized decendant)
+            // actual merging done by KisLayer::createMergedLayer (or specialized descendant)
             m_info->dstNode = m_info->currLayer->createMergedLayerTemplate(m_info->prevLayer);
 
             if (m_info->frames.size() > 0) {
@@ -383,7 +393,7 @@ namespace KisLayerUtils {
                     break;
                 }
 
-                KisLayerSP layer = dynamic_cast<KisLayer*>(node.data());
+                KisLayerSP layer = qobject_cast<KisLayer*>(node.data());
                 if (layer && layer->layerStyle()) {
                     compositionVaries = true;
                     break;
@@ -409,7 +419,7 @@ namespace KisLayerUtils {
         MergeLayers(MergeDownInfoSP info) : m_info(info) {}
 
         void populateChildCommands() override {
-            // actual merging done by KisLayer::createMergedLayer (or specialized decendant)
+            // actual merging done by KisLayer::createMergedLayer (or specialized descendant)
             m_info->currLayer->fillMergedLayerTemplate(m_info->dstLayer(), m_info->prevLayer);
         }
 
@@ -545,7 +555,7 @@ namespace KisLayerUtils {
     bool RemoveNodeHelper::scanForLastLayer(KisImageWSP image, KisNodeList nodesToRemove) {
         bool removeLayers = false;
         Q_FOREACH(KisNodeSP nodeToRemove, nodesToRemove) {
-            if (dynamic_cast<KisLayer*>(nodeToRemove.data())) {
+            if (qobject_cast<KisLayer*>(nodeToRemove.data())) {
                 removeLayers = true;
                 break;
             }
@@ -556,7 +566,7 @@ namespace KisLayerUtils {
         KisNodeSP node = image->root()->firstChild();
         while (node) {
             if (!nodesToRemove.contains(node) &&
-                dynamic_cast<KisLayer*>(node.data())) {
+                qobject_cast<KisLayer*>(node.data())) {
 
                 lastLayer = false;
                 break;
@@ -632,8 +642,7 @@ namespace KisLayerUtils {
 
             if (!parent) {
                 KisNodeSP oldRoot = m_info->image->root();
-                KisNodeSP newRoot =
-                    new KisGroupLayer(m_info->image, "root", OPACITY_OPAQUE_U8);
+                KisNodeSP newRoot(new KisGroupLayer(m_info->image, "root", OPACITY_OPAQUE_U8));
 
                 addCommand(new KisImageLayerAddCommand(m_info->image,
                                                        m_info->dstNode,
@@ -697,10 +706,8 @@ namespace KisLayerUtils {
             KIS_SAFE_ASSERT_RECOVER_RETURN(newLayer);
 
             foreach (KisSelectionMaskSP mask, selectionMasks) {
-                if (mask) {
-                    addCommand(new KisImageLayerMoveCommand(image, mask, newLayer, newLayer->lastChild()));
-                    addCommand(new KisActivateSelectionMaskCommand(mask, false));
-                }
+                addCommand(new KisImageLayerMoveCommand(image, mask, newLayer, newLayer->lastChild()));
+                addCommand(new KisActivateSelectionMaskCommand(mask, false));
             }
         }
     private:
@@ -804,7 +811,7 @@ namespace KisLayerUtils {
         if (!layer->prevSibling()) return;
 
         // XXX: this breaks if we allow free mixing of masks and layers
-        KisLayerSP prevLayer = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+        KisLayerSP prevLayer = qobject_cast<KisLayer*>(layer->prevSibling().data());
         if (!prevLayer) return;
 
         if (!layer->visible() && !prevLayer->visible()) {
@@ -832,6 +839,11 @@ namespace KisLayerUtils {
             applicator.applyCommand(new FillSelectionMasks(info));
             applicator.applyCommand(new CreateMergedLayer(info), KisStrokeJobData::BARRIER);
 
+            // NOTE: shape layer may have emitted spontaneous jobs during layer creation,
+            //       wait for them to complete!
+            applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
+            applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
+
             // in two-layer mode we disable pass trhough only when the destination layer
             // is not a group layer
             applicator.applyCommand(new DisablePassThroughForHeadsOnly(info, true));
@@ -843,12 +855,14 @@ namespace KisLayerUtils {
 
                     applicator.applyCommand(new AddNewFrame(info, frame));
                     applicator.applyCommand(new RefreshHiddenAreas(info));
+                    applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
                     applicator.applyCommand(new MergeLayers(info), KisStrokeJobData::BARRIER);
 
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, true, info->storage));
                 }
             } else {
                 applicator.applyCommand(new RefreshHiddenAreas(info));
+                applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
                 applicator.applyCommand(new MergeLayers(info), KisStrokeJobData::BARRIER);
             }
 
@@ -943,7 +957,7 @@ namespace KisLayerUtils {
         KisNodeList::iterator it = nodes.begin();
 
         while (it != nodes.end()) {
-            if ((!allowMasks && !dynamic_cast<KisLayer*>(it->data())) ||
+            if ((!allowMasks && !qobject_cast<KisLayer*>(it->data())) ||
                 checkIsChildOf(*it, nodes)) {
 
                 qDebug() << "Skipping node" << ppVar((*it)->name());
@@ -1084,10 +1098,14 @@ namespace KisLayerUtils {
                                            bool flattenSingleLayer, const KUndo2MagicString &actionName,
                                            bool cleanupNodes = true, const QString layerName = QString())
     {
+        if (!putAfter) {
+            putAfter = mergedNodes.first();
+        }
+
         filterMergableNodes(mergedNodes);
         {
             KisNodeList tempNodes;
-            qSwap(mergedNodes, tempNodes);
+            std::swap(mergedNodes, tempNodes);
             sortMergableNodes(image->root(), tempNodes, mergedNodes);
         }
 
@@ -1136,12 +1154,14 @@ namespace KisLayerUtils {
 
                     applicator.applyCommand(new AddNewFrame(info, frame));
                     applicator.applyCommand(new RefreshHiddenAreas(info));
+                    applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
                     applicator.applyCommand(new MergeLayersMultiple(info), KisStrokeJobData::BARRIER);
 
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, true, info->storage));
                 }
             } else {
                 applicator.applyCommand(new RefreshHiddenAreas(info));
+                applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
                 applicator.applyCommand(new MergeLayersMultiple(info), KisStrokeJobData::BARRIER);
             }
 
@@ -1186,7 +1206,7 @@ namespace KisLayerUtils {
 
             KisLayerSP parentLayer;
             do {
-                parentLayer = dynamic_cast<KisLayer*>(parent.data());
+                parentLayer = qobject_cast<KisLayer*>(parent.data());
 
                 parent = parent->parent();
             } while(!parentLayer && parent);
@@ -1242,7 +1262,7 @@ namespace KisLayerUtils {
 
         if (mergedNodes.isEmpty()) return false;
 
-        KisLayerSP parentLayer = dynamic_cast<KisLayer*>(selectionMasks.first()->parent().data());
+        KisLayerSP parentLayer = qobject_cast<KisLayer*>(selectionMasks.first()->parent().data());
         KIS_ASSERT_RECOVER(parentLayer) { return 0; }
 
         KisImageSignalVector emitSignals;
@@ -1335,7 +1355,19 @@ namespace KisLayerUtils {
         return recursiveFindNode(root,
             [uuid] (KisNodeSP node) {
                 return node->uuid() == uuid;
-            });
+        });
+    }
+
+    void forceAllDelayedNodesUpdate(KisNodeSP root)
+    {
+        KisLayerUtils::recursiveApplyNodes(root,
+        [] (KisNodeSP node) {
+            KisDelayedUpdateNodeInterface *delayedUpdate =
+                    dynamic_cast<KisDelayedUpdateNodeInterface*>(node.data());
+            if (delayedUpdate) {
+                delayedUpdate->forceUpdateTimedNode();
+            }
+        });
     }
 
 }
