@@ -46,7 +46,7 @@ typedef KisSafeReadList<KisNodeSP> KisSafeReadNodeList;
 #include "kis_keyframe_channel.h"
 
 /**
- *The link between KisProjection ans KisImageUpdater
+ *The link between KisProjection and KisImageUpdater
  *uses queued signals with an argument of KisNodeSP type,
  *so we should register it beforehand
  */
@@ -160,7 +160,7 @@ void KisNode::Private::processDuplicatedClones(const KisNode *srcDuplicationRoot
                                                         clone->copyFrom());
 
         if (newCopyFrom) {
-            KisLayer *newCopyFromLayer = dynamic_cast<KisLayer*>(const_cast<KisNode*>(newCopyFrom));
+            KisLayer *newCopyFromLayer = qobject_cast<KisLayer*>(const_cast<KisNode*>(newCopyFrom));
             KIS_ASSERT_RECOVER_RETURN(newCopyFromLayer);
 
             clone->setCopyFrom(newCopyFromLayer);
@@ -190,6 +190,13 @@ KisNode::KisNode(const KisNode & rhs)
     m_d->graphListener = 0;
     moveToThread(qApp->thread());
 
+    // HACK ALERT: we create opacity channel in KisBaseNode, but we cannot
+    //             initialize its node from there! So workaround it here!
+    QMap<QString, KisKeyframeChannel*> channels = rhs.keyframeChannels();
+    for (auto it = channels.begin(); it != channels.end(); ++it) {
+        it.value()->setNode(this);
+    }
+
     // NOTE: the nodes are not supposed to be added/removed while
     // creation of another node, so we do *no* locking here!
 
@@ -242,6 +249,10 @@ QRect KisNode::accessRect(const QRect &rect, PositionToFilthy pos) const
     return rect;
 }
 
+void KisNode::childNodeChanged(KisNodeSP /*changedChildNode*/)
+{
+}
+
 KisAbstractProjectionPlaneSP KisNode::projectionPlane() const
 {
     KIS_ASSERT_RECOVER_NOOP(0 && "KisNode::projectionPlane() is not defined!");
@@ -263,7 +274,7 @@ bool KisNode::accept(KisNodeVisitor &v)
 
 void KisNode::accept(KisProcessingVisitor &visitor, KisUndoAdapter *undoAdapter)
 {
-    return visitor.visit(this, undoAdapter);
+    visitor.visit(this, undoAdapter);
 }
 
 int KisNode::graphSequenceNumber() const
@@ -297,7 +308,7 @@ void KisNode::setParent(KisNodeWSP parent)
 KisNodeSP KisNode::parent() const
 {
     QReadLocker l(&m_d->nodeSubgraphLock);
-    return m_d->parent.isValid() ? KisNodeSP(m_d->parent) : 0;
+    return m_d->parent.isValid() ? KisNodeSP(m_d->parent) : KisNodeSP();
 }
 
 KisBaseNodeSP KisNode::parentCallback() const
@@ -494,11 +505,12 @@ bool KisNode::add(KisNodeSP newNode, KisNodeSP aboveThis)
         newNode->setGraphListener(m_d->graphListener);
     }
 
+    childNodeChanged(newNode);
+
     if (m_d->graphListener) {
         m_d->graphListener->nodeHasBeenAdded(this, idx);
     }
 
-
     return true;
 }
 
@@ -521,6 +533,8 @@ bool KisNode::remove(quint32 index)
             m_d->nodes.removeAt(index);
         }
 
+        childNodeChanged(removedNode);
+
         if (m_d->graphListener) {
             m_d->graphListener->nodeHasBeenRemoved(this, index);
         }
@@ -573,8 +587,8 @@ void KisNode::setDirty()
 
 void KisNode::setDirty(const QVector<QRect> &rects)
 {
-    Q_FOREACH (const QRect &rc, rects) {
-        setDirty(rc);
+    if(m_d->graphListener) {
+        m_d->graphListener->requestProjectionUpdate(this, rects, true);
     }
 }
 
@@ -584,9 +598,24 @@ void KisNode::setDirty(const QRegion &region)
 }
 
 void KisNode::setDirty(const QRect & rect)
+{
+    setDirty(QVector<QRect>({rect}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache()
+{
+    setDirtyDontResetAnimationCache(QVector<QRect>({extent()}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache(const QRect &rect)
+{
+    setDirtyDontResetAnimationCache(QVector<QRect>({rect}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache(const QVector<QRect> &rects)
 {
     if(m_d->graphListener) {
-        m_d->graphListener->requestProjectionUpdate(this, rect);
+        m_d->graphListener->requestProjectionUpdate(this, rects, false);
     }
 }
 
