@@ -100,6 +100,8 @@ public:
     KisDefaultBoundsBaseSP defaultBounds;
     QScopedPointer<KisPaintDeviceStrategy> basicStrategy;
     QScopedPointer<KisPaintDeviceWrappedStrategy> wrappedStrategy;
+    QMutex m_wrappedStrategyMutex;
+
     QScopedPointer<KisPaintDeviceFramesInterface> framesInterface;
     bool isProjectionDevice;
 
@@ -140,6 +142,10 @@ public:
         return currentData()->cache();
     }
 
+    inline KisIteratorCompleteListener* cacheInvalidator() {
+        return currentData()->cacheInvalidator();
+    }
+
 
     void cloneAllDataObjects(Private *rhs, bool copyFrames)
     {
@@ -385,7 +391,7 @@ public:
 
     KoColor frameDefaultPixel(int frameId) const
     {
-        DataSP data = m_frames[frameId];  
+        DataSP data = m_frames[frameId];
         return KoColor(data->dataManager()->defaultPixel(),
                        data->colorSpace());
     }
@@ -403,6 +409,38 @@ public:
 
     void tesingFetchLodDevice(KisPaintDeviceSP targetDevice);
 
+
+private:
+    qint64 estimateDataSize(Data *data) const {
+        const QRect &rc = data->dataManager()->extent();
+        return rc.width() * rc.height() * data->colorSpace()->pixelSize();
+    }
+
+public:
+
+    void estimateMemoryStats(qint64 &imageData, qint64 &temporaryData, qint64 &lodData) const {
+        imageData = 0;
+        temporaryData = 0;
+        lodData = 0;
+
+        if (m_data) {
+            imageData += estimateDataSize(m_data.data());
+        }
+
+        if (m_lodData) {
+            lodData += estimateDataSize(m_lodData.data());
+        }
+
+        if (m_externalFrameData) {
+            temporaryData += estimateDataSize(m_externalFrameData.data());
+        }
+
+        Q_FOREACH (DataSP value, m_frames.values()) {
+            imageData += estimateDataSize(value.data());
+        }
+    }
+
+
 private:
 
     QRegion syncWholeDevice(Data *srcData);
@@ -552,9 +590,16 @@ KisPaintDevice::Private::KisPaintDeviceStrategy* KisPaintDevice::Private::curren
         return basicStrategy.data();
     }
 
-    QRect wrapRect = defaultBounds->bounds();
+    const QRect wrapRect = defaultBounds->bounds();
+
     if (!wrappedStrategy || wrappedStrategy->wrapRect() != wrapRect) {
-        wrappedStrategy.reset(new KisPaintDeviceWrappedStrategy(wrapRect, q, this));
+        QMutexLocker locker(&m_wrappedStrategyMutex);
+
+        if (!wrappedStrategy) {
+            wrappedStrategy.reset(new KisPaintDeviceWrappedStrategy(wrapRect, q, this));
+        }  else if (wrappedStrategy->wrapRect() != wrapRect) {
+            wrappedStrategy->setWrapRect(wrapRect);
+        }
     }
 
     return wrappedStrategy.data();
@@ -693,7 +738,7 @@ void KisPaintDevice::Private::updateLodDataStruct(LodDataStruct *_dst, const QRe
     while (rowsRemaining > 0) {
 
         int colsRemaining = srcRect.width();
-        while (colsRemaining > 0) {
+        while (colsRemaining > 0 && srcIntIt.nextPixel()) {
 
             memcpy(blendDataPtr, srcIntIt.rawDataConst(), pixelSize);
             blendDataPtr += pixelSize;
@@ -704,7 +749,6 @@ void KisPaintDevice::Private::updateLodDataStruct(LodDataStruct *_dst, const QRe
                 columnsAccumulated = 0;
             }
 
-            srcIntIt.nextPixel();
             colsRemaining--;
         }
 
@@ -714,11 +758,13 @@ void KisPaintDevice::Private::updateLodDataStruct(LodDataStruct *_dst, const QRe
 
             // blend and write the final data
             blendDataPtr = blendData.data();
-            for (int i = 0; i < dstRect.width(); i++) {
-                mixOp->mixColors(blendDataPtr, weights.data(), srcCellSize, dstIntIt.rawData());
 
+            int colsRemaining = dstRect.width();
+            while (colsRemaining > 0 && dstIntIt.nextPixel()) {
+                mixOp->mixColors(blendDataPtr, weights.data(), srcCellSize, dstIntIt.rawData());
                 blendDataPtr += srcCellStride;
-                dstIntIt.nextPixel();
+
+                colsRemaining--;
             }
 
             // reset counters
@@ -804,6 +850,8 @@ void KisPaintDevice::Private::uploadFrameData(DataSP srcData, DataSP dstData)
 
     const QRect rect = srcData->dataManager()->extent();
     dstData->dataManager()->bitBltRough(srcData->dataManager(), rect);
+    dstData->setX(srcData->x());
+    dstData->setY(srcData->y());
 }
 
 void KisPaintDevice::Private::tesingFetchLodDevice(KisPaintDeviceSP targetDevice)
@@ -1049,6 +1097,11 @@ int KisPaintDevice::sequenceNumber() const
     return m_d->cache()->sequenceNumber();
 }
 
+void KisPaintDevice::estimateMemoryStats(qint64 &imageData, qint64 &temporaryData, qint64 &lodData) const
+{
+    m_d->estimateMemoryStats(imageData, temporaryData, lodData);
+}
+
 void KisPaintDevice::setParentNode(KisNodeWSP parent)
 {
     m_d->parent = parent;
@@ -1386,7 +1439,7 @@ void KisPaintDevice::clear(const QRect & rc)
 
 void KisPaintDevice::fill(const QRect & rc, const KoColor &color)
 {
-    Q_ASSERT(*color.colorSpace() == *colorSpace());
+    KIS_ASSERT_RECOVER_RETURN(*color.colorSpace() == *colorSpace());
     m_d->currentStrategy()->fill(rc, color.data());
 }
 
@@ -1493,7 +1546,7 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *dstProfile,
                            renderingIntent, conversionFlags);
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags) const
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags) const
 {
 
     if (w < 0)
@@ -1553,25 +1606,40 @@ static KisPaintDeviceSP createThumbnailDeviceInternal(const KisPaintDevice* srcD
     return thumbnail;
 }
 
+QSize fixThumbnailSize(QSize size)
+{
+    if (!size.width() && size.height()) {
+        size.setWidth(1);
+    }
+
+    if (size.width() && !size.height()) {
+        size.setHeight(1);
+    }
+
+    return size;
+}
+
 KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, QRect rect, QRect outputRect) const
 {
     QSize thumbnailSize(w, h);
 
-    int srcWidth, srcHeight;
-    int srcX0, srcY0;
     QRect imageRect = rect.isValid() ? rect : extent();
 
-    //can't create thumbnail for an empty device, e.g. layer thumbnail for empty image
-    if (imageRect.isEmpty() || !imageRect.isValid()) {
-        return new KisPaintDevice(colorSpace());
-    }
-
-    imageRect.getRect(&srcX0, &srcY0, &srcWidth, &srcHeight);
-
     if ((thumbnailSize.width() > imageRect.width()) || (thumbnailSize.height() > imageRect.height())) {
         thumbnailSize.scale(imageRect.size(), Qt::KeepAspectRatio);
     }
 
+    thumbnailSize = fixThumbnailSize(thumbnailSize);
+
+    //can't create thumbnail for an empty device, e.g. layer thumbnail for empty image
+    if (imageRect.isEmpty() || thumbnailSize.isEmpty()) {
+        return new KisPaintDevice(colorSpace());
+    }
+
+    int srcWidth, srcHeight;
+    int srcX0, srcY0;
+    imageRect.getRect(&srcX0, &srcY0, &srcWidth, &srcHeight);
+
     if (!outputRect.isValid()) {
         outputRect = QRect(0, 0, w, h);
     }
@@ -1589,12 +1657,7 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDeviceOversampled(qint32 w, qint
     QSize thumbnailOversampledSize = oversampleAdjusted * thumbnailSize;
 
     QRect outputRect;
-    QRect imageRect = (rect.isValid() && !rect.isNull()) ? rect : extent();
-
-    //can't create thumbnail for an empty device, e.g. layer thumbnail for empty image
-    if (imageRect.isEmpty() || !imageRect.isValid()) {
-        return new KisPaintDevice(colorSpace());
-    }
+    QRect imageRect = rect.isValid() ? rect : extent();
 
     qint32 hstart = thumbnailOversampledSize.height();
 
@@ -1602,6 +1665,13 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDeviceOversampled(qint32 w, qint
         thumbnailOversampledSize.scale(imageRect.size(), Qt::KeepAspectRatio);
     }
 
+    thumbnailOversampledSize = fixThumbnailSize(thumbnailOversampledSize);
+
+    //can't create thumbnail for an empty device, e.g. layer thumbnail for empty image
+    if (imageRect.isEmpty() || thumbnailSize.isEmpty() || thumbnailOversampledSize.isEmpty()) {
+        return new KisPaintDevice(colorSpace());
+    }
+
     oversampleAdjusted *= (hstart > 0) ? ((qreal)thumbnailOversampledSize.height() / hstart) : 1.; //readjusting oversample ratio, given that we had to adjust thumbnail size
 
     outputRect = QRect(0, 0, thumbnailOversampledSize.width(), thumbnailOversampledSize.height());
@@ -1626,14 +1696,18 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDeviceOversampled(qint32 w, qint
 
 QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, QRect rect, qreal oversample, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags)
 {
-    KisPaintDeviceSP dev = createThumbnailDeviceOversampled(w, h, oversample, rect);
+    QSize size = fixThumbnailSize(QSize(w, h));
+
+    KisPaintDeviceSP dev = createThumbnailDeviceOversampled(size.width(), size.height(), oversample, rect);
     QImage thumbnail = dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile(), 0, 0, w, h, renderingIntent, conversionFlags);
     return thumbnail;
 }
 
 QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, qreal oversample, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags)
 {
-    return m_d->cache()->createThumbnail(w, h, oversample, renderingIntent, conversionFlags);
+    QSize size = fixThumbnailSize(QSize(w, h));
+
+    return m_d->cache()->createThumbnail(size.width(), size.height(), oversample, renderingIntent, conversionFlags);
 }
 
 KisHLineIteratorSP KisPaintDevice::createHLineIteratorNG(qint32 x, qint32 y, qint32 w)
@@ -1660,12 +1734,12 @@ KisVLineConstIteratorSP KisPaintDevice::createVLineConstIteratorNG(qint32 x, qin
 
 KisRepeatHLineConstIteratorSP KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth) const
 {
-    return new KisRepeatHLineConstIteratorNG(m_d->dataManager().data(), x, y, w, m_d->x(), m_d->y(), _dataWidth);
+    return new KisRepeatHLineConstIteratorNG(m_d->dataManager().data(), x, y, w, m_d->x(), m_d->y(), _dataWidth, m_d->cacheInvalidator());
 }
 
 KisRepeatVLineConstIteratorSP KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth) const
 {
-    return new KisRepeatVLineConstIteratorNG(m_d->dataManager().data(), x, y, h, m_d->x(), m_d->y(), _dataWidth);
+    return new KisRepeatVLineConstIteratorNG(m_d->dataManager().data(), x, y, h, m_d->x(), m_d->y(), _dataWidth, m_d->cacheInvalidator());
 }
 
 KisRandomAccessorSP KisPaintDevice::createRandomAccessorNG(qint32 x, qint32 y)
@@ -1905,7 +1979,7 @@ QVector<qint32> KisPaintDevice::channelSizes() const
 {
     QVector<qint32> sizes;
     QList<KoChannelInfo*> channels = colorSpace()->channels();
-    qSort(channels);
+    std::sort(channels.begin(), channels.end());
 
     Q_FOREACH (KoChannelInfo * channelInfo, channels) {
         sizes.append(channelInfo->size());
