@@ -25,6 +25,7 @@
 #include <QColor>
 #include <QFile>
 #include <QDomDocument>
+#include <QDomElement>
 #include <QBuffer>
 
 #include <klocalizedstring.h>
@@ -127,6 +128,10 @@ QGradient* KoStopGradient::toQGradient() const
         i->second.toQColor(&color);
         gradient->setColorAt(i->first , color);
     }
+
+    gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
+    gradient->setSpread(this->spread());
+
     return gradient;
 }
 
@@ -205,32 +210,32 @@ void KoStopGradient::colorAt(KoColor& dst, qreal t) const
     }
 }
 
-KoStopGradient * KoStopGradient::fromQGradient(QGradient * gradient)
+KoStopGradient * KoStopGradient::fromQGradient(const QGradient * gradient)
 {
     if (! gradient)
         return 0;
 
-    KoStopGradient * newGradient = new KoStopGradient("");
+    KoStopGradient * newGradient = new KoStopGradient(QString());
     newGradient->setType(gradient->type());
     newGradient->setSpread(gradient->spread());
 
     switch (gradient->type()) {
     case QGradient::LinearGradient: {
-        QLinearGradient * g = static_cast<QLinearGradient*>(gradient);
+        const QLinearGradient * g = static_cast<const QLinearGradient*>(gradient);
         newGradient->m_start = g->start();
         newGradient->m_stop = g->finalStop();
         newGradient->m_focalPoint = g->start();
         break;
     }
     case QGradient::RadialGradient: {
-        QRadialGradient * g = static_cast<QRadialGradient*>(gradient);
+        const QRadialGradient * g = static_cast<const QRadialGradient*>(gradient);
         newGradient->m_start = g->center();
         newGradient->m_stop = g->center() + QPointF(g->radius(), 0);
         newGradient->m_focalPoint = g->focalPoint();
         break;
     }
     case QGradient::ConicalGradient: {
-        QConicalGradient * g = static_cast<QConicalGradient*>(gradient);
+        const QConicalGradient * g = static_cast<const QConicalGradient*>(gradient);
         qreal radian = g->angle() * M_PI / 180.0;
         newGradient->m_start = g->center();
         newGradient->m_stop = QPointF(100.0 * cos(radian), 100.0 * sin(radian));
@@ -248,6 +253,8 @@ KoStopGradient * KoStopGradient::fromQGradient(QGradient * gradient)
         newGradient->m_stops.append(KoGradientStop(stop.first, color));
     }
 
+    newGradient->setValid(true);
+
     return newGradient;
 }
 
@@ -639,6 +646,37 @@ QString KoStopGradient::defaultFileExtension() const
     return QString(".svg");
 }
 
+void KoStopGradient::toXML(QDomDocument &doc, QDomElement &gradientElt) const
+{
+    gradientElt.setAttribute("type", "stop");
+    for (int s = 0; s<m_stops.size(); s++) {
+        KoGradientStop stop = m_stops.at(s);
+        QDomElement stopElt = doc.createElement("stop");
+        stopElt.setAttribute("offset", stop.first);
+        stopElt.setAttribute("bitdepth", stop.second.colorSpace()->colorDepthId().id());
+        stopElt.setAttribute("alpha", stop.second.opacityF());
+        stop.second.toXML(doc, stopElt);
+        gradientElt.appendChild(stopElt);
+    }
+}
+
+KoStopGradient KoStopGradient::fromXML(const QDomElement &elt)
+{
+    KoStopGradient gradient;
+    QList<KoGradientStop> stops;
+    QDomElement stopElt = elt.firstChildElement("stop");
+    while (!stopElt.isNull()) {
+        qreal offset = stopElt.attribute("offset", "0").toDouble();
+        QString bitDepth = stopElt.attribute("bitdepth", Integer8BitsColorDepthID.id());
+        KoColor color = KoColor::fromXML(stopElt.firstChildElement(), bitDepth);
+        color.setOpacity(stopElt.attribute("alpha", "1.0").toDouble());
+        stops.append(KoGradientStop(offset, color));
+        stopElt = stopElt.nextSiblingElement("stop");
+    }
+    gradient.setStops(stops);
+    return gradient;
+}
+
 bool KoStopGradient::saveToDevice(QIODevice *dev) const
 {
     QTextStream stream(dev);
