@@ -1,23 +1,22 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright (C) 2009 Thomas Zander <zander@kde.org>
-   Copyright (C) 2012 Boudewijn Rempt <boud@valdyas.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
-*/
+/*
+ * Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
+ * Copyright (C) 2012 Boudewijn Rempt <boud@valdyas.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 
 #include "KisApplication.h"
 
@@ -31,7 +30,7 @@
 #include "osx.h"
 #endif
 
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QDesktopWidget>
 #include <QDir>
 #include <QFile>
@@ -85,10 +84,12 @@
 #include <kis_debug.h>
 #include "kis_action_registry.h"
 #include <kis_brush_server.h>
-#include <kis_resource_server_provider.h>
+#include <KisResourceServerProvider.h>
 #include <KoResourceServerProvider.h>
 #include "kis_image_barrier_locker.h"
 #include "opengl/kis_opengl.h"
+#include "kis_spin_box_unit_manager.h"
+#include "kis_document_aware_spin_box_unit_manager.h"
 #include "KisViewManager.h"
 #include "kis_workspace_resource.h"
 
@@ -146,7 +147,6 @@ public:
 };
 
 
-
 KisApplication::KisApplication(const QString &key, int &argc, char **argv)
     : QtSingleApplication(key, argc, argv)
     , d(new KisApplicationPrivate)
@@ -219,7 +219,7 @@ BOOL isWow64()
 }
 #endif
 
-void initializeGlobals(const KisApplicationArguments &args)
+void KisApplication::initializeGlobals(const KisApplicationArguments &args)
 {
     int dpiX = args.dpiX();
     int dpiY = args.dpiY();
@@ -228,8 +228,9 @@ void initializeGlobals(const KisApplicationArguments &args)
     }
 }
 
-void addResourceTypes()
+void KisApplication::addResourceTypes()
 {
+//    qDebug() << "addResourceTypes();";
     // All Krita's resource types
     KoResourcePaths::addResourceType("kis_pics", "data", "/pics/");
     KoResourcePaths::addResourceType("kis_images", "data", "/images/");
@@ -254,6 +255,9 @@ void addResourceTypes()
     KoResourcePaths::addResourceType("ko_effects", "data", "/effects/");
     KoResourcePaths::addResourceType("tags", "data", "/tags/");
     KoResourcePaths::addResourceType("templates", "data", "/templates");
+    KoResourcePaths::addResourceType("pythonscripts", "data", "/pykrita");
+    KoResourcePaths::addResourceType("symbols", "data", "/symbols");
+    KoResourcePaths::addResourceType("preset_icons", "data", "/preset_icons");
 
     //    // Extra directories to look for create resources. (Does anyone actually use that anymore?)
     //    KoResourcePaths::addResourceDir("ko_gradients", "/usr/share/create/gradients/gimp");
@@ -270,6 +274,7 @@ void addResourceTypes()
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/tags/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/asl/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/bundles/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/brushes/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/gradients/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/paintoppresets/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/palettes/");
@@ -277,40 +282,59 @@ void addResourceTypes()
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/taskset/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/workspaces/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/input/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/pykrita/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/symbols/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/color-schemes/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/tool_icons/");
+    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/emblem_icons/");
+
+    // Indicate that it is now safe for users of KoResourcePaths to load resources
+    KoResourcePaths::setReady();
 }
 
 void KisApplication::loadResources()
 {
-    setSplashScreenLoadingText(i18n("Loading Gradients..."));
-    processEvents();
-    KoResourceServerProvider::instance()->gradientServer(true);
+//    qDebug() << "loadResources();";
 
-
-    // Load base resources
-    setSplashScreenLoadingText(i18n("Loading Patterns..."));
+    setSplashScreenLoadingText(i18n("Loading Resources..."));
     processEvents();
-    KoResourceServerProvider::instance()->patternServer(true);
+    KoResourceServerProvider::instance();
 
-    setSplashScreenLoadingText(i18n("Loading Palettes..."));
+    setSplashScreenLoadingText(i18n("Loading Brush Presets..."));
     processEvents();
-    KoResourceServerProvider::instance()->paletteServer(false);
+    KisResourceServerProvider::instance();
 
     setSplashScreenLoadingText(i18n("Loading Brushes..."));
     processEvents();
-    KisBrushServer::instance()->brushServer(true);
+    KisBrushServer::instance()->brushServer();
 
-    // load paintop presets
-    setSplashScreenLoadingText(i18n("Loading Paint Operations..."));
+    setSplashScreenLoadingText(i18n("Loading Bundles..."));
     processEvents();
-    KisResourceServerProvider::instance()->paintOpPresetServer(true);
+    KisResourceBundleServerProvider::instance();
+}
 
-    setSplashScreenLoadingText(i18n("Loading Resource Bundles..."));
-    processEvents();
-    KisResourceServerProvider::instance()->resourceBundleServer();
+void KisApplication::loadResourceTags()
+{
+//    qDebug() << "loadResourceTags()";
+
+    KoResourceServerProvider::instance()->patternServer()->loadTags();
+    KoResourceServerProvider::instance()->gradientServer()->loadTags();
+    KoResourceServerProvider::instance()->paletteServer()->loadTags();
+    KoResourceServerProvider::instance()->svgSymbolCollectionServer()->loadTags();
+    KisBrushServer::instance()->brushServer()->loadTags();
+    KisResourceServerProvider::instance()->workspaceServer()->loadTags();
+    KisResourceServerProvider::instance()->layerStyleCollectionServer()->loadTags();
+    KisResourceBundleServerProvider::instance()->resourceBundleServer()->loadTags();
+    KisResourceServerProvider::instance()->paintOpPresetServer()->loadTags();
+
+    KisResourceServerProvider::instance()->paintOpPresetServer()->clearOldSystemTags();
 }
 
 void KisApplication::loadPlugins()
 {
+//    qDebug() << "loadPlugins();";
+
     KoShapeRegistry* r = KoShapeRegistry::instance();
     r->add(new KisShapeSelectionFactory());
 
@@ -319,10 +343,15 @@ void KisApplication::loadPlugins()
     KisGeneratorRegistry::instance();
     KisPaintOpRegistry::instance();
     KoColorSpaceRegistry::instance();
+}
 
+void KisApplication::loadGuiPlugins()
+{
+//    qDebug() << "loadGuiPlugins();";
     // Load the krita-specific tools
     setSplashScreenLoadingText(i18n("Loading Plugins for Krita/Tool..."));
     processEvents();
+//    qDebug() << "loading tools";
     KoPluginLoader::instance()->load(QString::fromLatin1("Krita/Tool"),
                                      QString::fromLatin1("[X-Krita-Version] == 28"));
 
@@ -330,16 +359,17 @@ void KisApplication::loadPlugins()
     // Load dockers
     setSplashScreenLoadingText(i18n("Loading Plugins for Krita/Dock..."));
     processEvents();
+//    qDebug() << "loading dockers";
     KoPluginLoader::instance()->load(QString::fromLatin1("Krita/Dock"),
                                      QString::fromLatin1("[X-Krita-Version] == 28"));
 
     // XXX_EXIV: make the exiv io backends real plugins
     setSplashScreenLoadingText(i18n("Loading Plugins Exiv/IO..."));
     processEvents();
+//    qDebug() << "loading exiv2";
     KisExiv2::initialize();
 }
 
-
 bool KisApplication::start(const KisApplicationArguments &args)
 {
     KisConfig cfg;
@@ -371,6 +401,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
     processEvents();
     initializeGlobals(args);
 
+    const bool doNewImage = args.doNewImage();
     const bool doTemplate = args.doTemplate();
     const bool print = args.print();
     const bool exportAs = args.exportAs();
@@ -384,7 +415,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
     // TODO: fix print & exportAsPdf to work without mainwindow shown
     const bool showmainWindow = !exportAs; // would be !batchRun;
 
-    const bool showSplashScreen = !m_batchRun && qEnvironmentVariableIsEmpty("NOSPLASH");// &&  qgetenv("XDG_CURRENT_DESKTOP") != "GNOME";
+    const bool showSplashScreen = !m_batchRun && qEnvironmentVariableIsEmpty("NOSPLASH");
     if (showSplashScreen && d->splashScreen) {
         d->splashScreen->show();
         d->splashScreen->repaint();
@@ -408,12 +439,18 @@ bool KisApplication::start(const KisApplicationArguments &args)
     processEvents();
     addResourceTypes();
 
-    // Load all resources and tags before the plugins do that
-    loadResources();
-
     // Load the plugins
     loadPlugins();
 
+    // Load all resources
+    loadResources();
+
+    // Load all the tags
+    loadResourceTags();
+
+    // Load the gui plugins
+    loadGuiPlugins();
+
     if (needsMainWindow) {
         // show a mainWindow asap, if we want that
         setSplashScreenLoadingText(i18n("Loading Main Window..."));
@@ -435,7 +472,12 @@ bool KisApplication::start(const KisApplicationArguments &args)
                 m_mainWindow->viewManager()->switchCanvasOnly(true);
             }
 
-            m_mainWindow->show();
+            if (args.fullScreen()) {
+                m_mainWindow->showFullScreen();
+            }
+            else {
+                m_mainWindow->show();
+            }
         }
     }
     short int numberOfOpenDocuments = 0; // number of documents open
@@ -448,6 +490,21 @@ bool KisApplication::start(const KisApplicationArguments &args)
     setSplashScreenLoadingText(QString()); // done loading, so clear out label
     processEvents();
 
+    //configure the unit manager
+    KisSpinBoxUnitManagerFactory::setDefaultUnitManagerBuilder(new KisDocumentAwareSpinBoxUnitManagerBuilder());
+    connect(this, &KisApplication::aboutToQuit, &KisSpinBoxUnitManagerFactory::clearUnitManagerBuilder); //ensure the builder is destroyed when the application leave.
+    //the new syntax slot syntax allow to connect to a non q_object static method.
+
+
+    // Create a new image, if needed
+    if (doNewImage) {
+        KisDocument *doc = args.image();
+        if (doc) {
+            KisPart::instance()->addDocument(doc);
+            m_mainWindow->addViewAndNotifyLoadingCompleted(doc);
+        }
+    }
+
     // Get the command line arguments which we have to parse
     int argsCount = args.filenames().count();
     if (argsCount > 0) {
@@ -467,9 +524,8 @@ bool KisApplication::start(const KisApplicationArguments &args)
                 // now try to load
             }
             else {
-
                 if (exportAs) {
-                    QString outputMimetype = KisMimeDatabase::mimeTypeForFile(exportFileName);
+                    QString outputMimetype = KisMimeDatabase::mimeTypeForFile(exportFileName, false);
                     if (outputMimetype == "application/octetstream") {
                         dbgKrita << i18n("Mimetype not found, try using the -mimetype option") << endl;
                         return 1;
@@ -482,17 +538,15 @@ bool KisApplication::start(const KisApplicationArguments &args)
                     qApp->processEvents(); // For vector layers to be updated
 
                     doc->setFileBatchMode(true);
-                    doc->setOutputMimeType(outputMimetype.toLatin1());
-                    if (!doc->exportDocument(QUrl::fromLocalFile(exportFileName))) {
+                    if (!doc->exportDocumentSync(QUrl::fromLocalFile(exportFileName), outputMimetype.toLatin1())) {
                         dbgKrita << "Could not export " << fileName << "to" << exportFileName << ":" << doc->errorMessage();
                     }
                     nPrinted++;
                     QTimer::singleShot(0, this, SLOT(quit()));
                 }
                 else if (m_mainWindow) {
-                    KisDocument *doc = KisPart::instance()->createDocument();
-                    doc->setFileBatchMode(m_batchRun);
-                    if (m_mainWindow->openDocumentInternal(QUrl::fromLocalFile(fileName), doc)) {
+                    KisMainWindow::OpenFlags flags = m_batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
+                    if (m_mainWindow->openDocument(QUrl::fromLocalFile(fileName), flags)) {
                         if (print) {
                             m_mainWindow->slotFilePrint();
                             nPrinted++;
@@ -524,8 +578,8 @@ bool KisApplication::start(const KisApplicationArguments &args)
     // fixes BUG:369308  - Krita crashing on splash screen when loading.
     // trying to open a file before Krita has loaded can cause it to hang and crash
     if (d->splashScreen) {
-        d->splashScreen->displayLinks();
-        d->splashScreen->displayRecentFiles();
+        d->splashScreen->displayLinks(true);
+        d->splashScreen->displayRecentFiles(true);
     }
 
 
@@ -546,7 +600,8 @@ void KisApplication::setSplashScreen(QWidget *splashScreen)
 void KisApplication::setSplashScreenLoadingText(QString textToLoad)
 {
     if (d->splashScreen) {
-        d->splashScreen->loadingLabel->setText(textToLoad);
+       //d->splashScreen->loadingLabel->setText(textToLoad);
+        d->splashScreen->setLoadingText(textToLoad);
         d->splashScreen->repaint();
     }
 }
@@ -601,9 +656,8 @@ void KisApplication::remoteArguments(QByteArray message, QObject *socket)
                 createNewDocFromTemplate(filename, mw);
             }
             else if (QFile(filename).exists()) {
-                KisDocument *doc = KisPart::instance()->createDocument();
-                doc->setFileBatchMode(m_batchRun);
-                mw->openDocumentInternal(QUrl::fromLocalFile(filename), doc);
+                KisMainWindow::OpenFlags flags = m_batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
+                mw->openDocument(QUrl::fromLocalFile(filename), flags);
             }
         }
     }
@@ -613,9 +667,8 @@ void KisApplication::fileOpenRequested(const QString &url)
 {
     KisMainWindow *mainWindow = KisPart::instance()->mainWindows().first();
     if (mainWindow) {
-        KisDocument *doc = KisPart::instance()->createDocument();
-        doc->setFileBatchMode(m_batchRun);
-        mainWindow->openDocumentInternal(QUrl::fromLocalFile(url), doc);
+        KisMainWindow::OpenFlags flags = m_batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
+        mainWindow->openDocument(QUrl::fromLocalFile(url), flags);
     }
 }
 
@@ -637,40 +690,39 @@ void KisApplication::checkAutosaveFiles()
 #endif
 
     // all autosave files for our application
-    m_autosaveFiles = dir.entryList(filters, QDir::Files | QDir::Hidden);
+    QStringList autosaveFiles = dir.entryList(filters, QDir::Files | QDir::Hidden);
 
     // Allow the user to make their selection
-    if (m_autosaveFiles.size() > 0) {
+    if (autosaveFiles.size() > 0) {
         if (d->splashScreen) {
             // hide the splashscreen to see the dialog
             d->splashScreen->hide();
         }
-        m_autosaveDialog = new KisAutoSaveRecoveryDialog(m_autosaveFiles, activeWindow());
+        m_autosaveDialog = new KisAutoSaveRecoveryDialog(autosaveFiles, activeWindow());
         QDialog::DialogCode result = (QDialog::DialogCode) m_autosaveDialog->exec();
 
         if (result == QDialog::Accepted) {
             QStringList filesToRecover = m_autosaveDialog->recoverableFiles();
-            Q_FOREACH (const QString &autosaveFile, m_autosaveFiles) {
+            Q_FOREACH (const QString &autosaveFile, autosaveFiles) {
                 if (!filesToRecover.contains(autosaveFile)) {
                     QFile::remove(dir.absolutePath() + "/" + autosaveFile);
                 }
             }
-            m_autosaveFiles = filesToRecover;
+            autosaveFiles = filesToRecover;
         } else {
-            m_autosaveFiles.clear();
+            autosaveFiles.clear();
         }
 
-        if (m_autosaveFiles.size() > 0) {
+        if (autosaveFiles.size() > 0) {
             QList<QUrl> autosaveUrls;
-            Q_FOREACH (const QString &autoSaveFile, m_autosaveFiles) {
+            Q_FOREACH (const QString &autoSaveFile, autosaveFiles) {
                 const QUrl url = QUrl::fromLocalFile(dir.absolutePath() + QLatin1Char('/') + autoSaveFile);
                 autosaveUrls << url;
             }
             if (m_mainWindow) {
                 Q_FOREACH (const QUrl &url, autosaveUrls) {
-                    KisDocument *doc = KisPart::instance()->createDocument();
-                    doc->setFileBatchMode(m_batchRun);
-                    m_mainWindow->openDocumentInternal(url, doc);
+                    KisMainWindow::OpenFlags flags = m_batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
+                    m_mainWindow->openDocument(url, flags | KisMainWindow::RecoveryFile);
                 }
             }
         }
@@ -718,12 +770,8 @@ bool KisApplication::createNewDocFromTemplate(const QString &fileName, KisMainWi
         QUrl templateURL;
         templateURL.setPath(templateBase.adjusted(QUrl::RemoveFilename|QUrl::StripTrailingSlash).path() + '/' + templateName);
 
-        KisDocument *doc = KisPart::instance()->createDocument();
-        doc->setFileBatchMode(m_batchRun);
-        if (mainWindow->openDocumentInternal(templateURL, doc)) {
-            doc->resetURL();
-            doc->setEmpty();
-            doc->setTitleModified();
+        KisMainWindow::OpenFlags batchFlags = m_batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
+        if (mainWindow->openDocument(templateURL, KisMainWindow::Import | batchFlags)) {
             dbgUI << "Template loaded...";
             return true;
         }
