@@ -30,7 +30,6 @@
 #include <QUrl>
 #include <KoCanvasObserverBase.h>
 #include <KoCanvasSupervisor.h>
-
 #include "KisView.h"
 
 class QCloseEvent;
@@ -60,6 +59,15 @@ class KRITAUI_EXPORT KisMainWindow : public KXmlGuiWindow, public KoCanvasSuperv
 {
     Q_OBJECT
 
+public:
+    enum OpenFlag {
+        None = 0,
+        Import = 0x1,
+        BatchMode = 0x2,
+        RecoveryFile = 0x4
+    };
+    Q_DECLARE_FLAGS(OpenFlags, OpenFlag)
+
 public:
 
     /**
@@ -74,12 +82,6 @@ public:
      */
     ~KisMainWindow() override;
 
-    /**
-     * Update caption from document info - call when document info
-     * (title in the about page) changes.
-     */
-    void updateCaption();
-
 
     // If noCleanup is set, KisMainWindow will not delete the root document
     // or part manager on destruction.
@@ -113,11 +115,22 @@ public:
      *
      * @return TRUE on success.
      */
-    bool openDocument(const QUrl &url);
+    bool openDocument(const QUrl &url, OpenFlags flags);
+
+    /**
+     * Saves the document, asking for a filename if necessary.
+     *
+     * @param saveas if set to TRUE the user is always prompted for a filename
+     * @param silent if set to TRUE rootDocument()->setTitleModified will not be called.
+     *
+     * @return TRUE on success, false on error or cancel
+     *         (don't display anything in this case, the error dialog box is also implemented here
+     *         but restore the original URL in slotFileSaveAs)
+     */
+    bool saveDocument(KisDocument *document, bool saveas, bool isExporting);
 
     void setReadWrite(bool readwrite);
 
-
     /// Return the list of dock widgets belonging to this main window.
     QList<QDockWidget*> dockWidgets() const;
 
@@ -138,9 +151,9 @@ public:
 
     KisViewManager *viewManager() const;
 
-    void addViewAndNotifyLoadingCompleted(KisDocument *document);
+    KisView *addViewAndNotifyLoadingCompleted(KisDocument *document);
 
-    QStringList showOpenFileDialog();
+    QStringList showOpenFileDialog(bool isImporting);
 
     /**
      * Shows if the main window is saving anything right now. If the
@@ -177,9 +190,6 @@ Q_SIGNALS:
 
     void guiLoadingFinished();
 
-    /// This signal is emitted when the user clicked on the progressbar cancel
-    void sigProgressCanceled();
-
 public Q_SLOTS:
 
     /**
@@ -196,7 +206,7 @@ public Q_SLOTS:
      *  If the current document is empty, the opened document replaces it.
      *  If not a new mainwindow will be opened for showing the opened file.
      */
-    void slotFileOpen();
+    void slotFileOpen(bool isImporting = false);
 
     /**
      *  Slot for opening a file among the recently opened files.
@@ -211,36 +221,26 @@ public Q_SLOTS:
      */
     void slotPreferences();
 
+    /**
+     * Update caption from document info - call when document info
+     * (title in the about page) changes.
+     */
+    void updateCaption();
+
     /**
      *  Saves the current document with the current name.
      */
     void slotFileSave();
 
     // XXX: disabled
-    KisPrintJob* exportToPdf(const QString &pdfFileName = QString());
-
-    void slotProgress(int value);
-
-    /**
-     * Saves the document, asking for a filename if necessary.
-     *
-     * @param saveas if set to TRUE the user is always prompted for a filename
-     *
-     * @param silent if set to TRUE rootDocument()->setTitleModified will not be called.
-     *
-     * @param specialOutputFlag set to enums defined in KisDocument if save to special output format
-     *
-     * @return TRUE on success, false on error or cancel
-     *         (don't display anything in this case, the error dialog box is also implemented here
-     *         but restore the original URL in slotFileSaveAs)
-     */
-    bool saveDocument(KisDocument *document, bool saveas = false, bool silent = false, int specialOutputFlag = 0);
+    KisPrintJob* exportToPdf(QString pdfFileName = QString());
 
     /**
      * Update the option widgets to the argument ones, removing the currently set widgets.
      */
     void newOptionWidgets(KoCanvasController *controller, const QList<QPointer<QWidget> > & optionWidgetList);
 
+    KisView *newView(QObject *document);
 
     void notifyChildViewDestroyed(KisView *view);
 
@@ -256,12 +256,10 @@ private Q_SLOTS:
     void slotSaveCanceled(const QString &);
     void forceDockTabFonts();
 
-    void slotProgressCanceled();
-
     /**
      * @internal
      */
-    void slotDocumentTitleModified(const QString &caption, bool mod);
+    void slotDocumentTitleModified();
 
     /**
      *  Prints the actual document.
@@ -275,8 +273,6 @@ private Q_SLOTS:
 
     void slotFilePrintPreview();
 
-    KisPrintJob* exportToPdf(KoPageLayout pageLayout, QString pdfFileName = QString());
-
     void importAnimation();
 
     /**
@@ -333,16 +329,14 @@ private Q_SLOTS:
     /**
      * File --> Import
      *
-     * This will call slotFileOpen().  To differentiate this from an ordinary
-     * call to slotFileOpen() call @ref isImporting().
+     * This will call slotFileOpen().
      */
     void slotImportFile();
 
     /**
      * File --> Export
      *
-     * This will call slotFileSaveAs().  To differentiate this from an ordinary
-     * call to slotFileSaveAs() call @ref isExporting().
+     * This will call slotFileSaveAs().
      */
     void slotExportFile();
 
@@ -358,11 +352,10 @@ private Q_SLOTS:
 
     void undo();
     void redo();
-    void subWindowActivated();
     void updateWindowMenu();
     void setActiveSubWindow(QWidget *window);
     void configChanged();
-    void newView(QObject *document);
+
     void newWindow();
     void closeCurrentWindow();
     void checkSanity();
@@ -374,17 +367,17 @@ protected:
     void closeEvent(QCloseEvent * e) override;
     void resizeEvent(QResizeEvent * e) override;
 
-    /// Set the active view, this will update the undo/redo actions
-    virtual void setActiveView(KisView *view);
-
     // QWidget overrides
     void dragEnterEvent(QDragEnterEvent * event) override;
     void dropEvent(QDropEvent * event) override;
     void dragMoveEvent(QDragMoveEvent * event) override;
     void dragLeaveEvent(QDragLeaveEvent * event) override;
 
+    void mouseReleaseEvent(QMouseEvent *event) override;
+
 
 private:
+
     /**
      * Add a the given view to the list of views of this mainwindow.
      * This is a private implementation. For public usage please use
@@ -392,6 +385,15 @@ private:
      */
     void addView(KisView *view);
 
+public Q_SLOTS:
+
+    /// Set the active view, this will update the undo/redo actions
+    void setActiveView(KisView *view);
+
+    void subWindowActivated();
+
+
+private:
 
     friend class KisApplication;
 
@@ -404,27 +406,7 @@ private:
      */
     QDockWidget* createDockWidget(KoDockFactoryBase* factory);
 
-    bool openDocumentInternal(const QUrl &url, KisDocument *newdoc = 0);
-
-    /**
-     * Returns whether or not the current slotFileSave[As]() or saveDocument()
-     * call is actually an export operation (like File --> Export).
-     *
-     * If this is true, you must call KisDocument::export() instead of
-     * KisDocument::save() or KisDocument::saveAs(), in any reimplementation of
-     * saveDocument().
-     */
-    bool isExporting() const;
-
-    /**
-     * Returns whether or not the current slotFileOpen() or openDocument()
-     * call is actually an import operation (like File --> Import).
-     *
-     * If this is true, you must call KisDocument::import() instead of
-     * KisDocument::openUrl(), in any reimplementation of openDocument() or
-     * openDocumentInternal().
-     */
-    bool isImporting() const;
+    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags flags = 0);
 
     /**
      * Reloads the recent documents list.
@@ -443,8 +425,6 @@ private:
 
     void applyDefaultSettings(QPrinter &printer);
 
-    bool exportConfirmation(const QByteArray &outputFormat);
-
     void createActions();
 
     void applyToolBarLayout();
@@ -481,4 +461,6 @@ private:
 
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(KisMainWindow::OpenFlags)
+
 #endif
