@@ -81,12 +81,11 @@
 #include "KisUndoStackAction.h"
 #include "KisViewManager.h"
 #include "kis_zoom_manager.h"
-#include "kis_composite_progress_proxy.h"
 #include "kis_statusbar.h"
 #include "kis_painting_assistants_decoration.h"
-#include "kis_progress_widget.h"
 #include "kis_signal_compressor.h"
 #include "kis_filter_manager.h"
+#include "kis_file_layer.h"
 #include "krita_utils.h"
 #include "input/kis_input_manager.h"
 #include "KisRemoteFileFetcher.h"
@@ -225,8 +224,8 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
 
     QStatusBar * sb = statusBar();
     if (sb) { // No statusbar in e.g. konqueror
-        connect(d->document, SIGNAL(statusBarMessage(const QString&)),
-                this, SLOT(slotActionStatusText(const QString&)));
+        connect(d->document, SIGNAL(statusBarMessage(const QString&, int)),
+                this, SLOT(slotSavingStatusMessage(const QString&, int)));
         connect(d->document, SIGNAL(clearStatusBarMessage()),
                 this, SLOT(slotClearStatusText()));
     }
@@ -260,17 +259,17 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
 KisView::~KisView()
 {
     if (d->viewManager) {
-        KoProgressProxy *proxy = d->viewManager->statusBar()->progress()->progressProxy();
-        KIS_ASSERT_RECOVER_NOOP(proxy);
-        image()->compositeProgressProxy()->removeProxy(proxy);
-
         if (d->viewManager->filterManager()->isStrokeRunning()) {
             d->viewManager->filterManager()->cancel();
         }
+
+        d->viewManager->mainWindow()->notifyChildViewDestroyed(this);
     }
 
-    KisPart::instance()->removeView(this);
     KoToolManager::instance()->removeCanvasController(&d->canvasController);
+    d->canvasController.setCanvas(0);
+
+    KisPart::instance()->removeView(this);
     delete d;
 }
 
@@ -357,15 +356,6 @@ void KisView::setViewManager(KisViewManager *view)
             SLOT(slotContinueRemoveNode(KisNodeSP)),
             Qt::AutoConnection);
 
-    /*
-     * WARNING: Currently we access the global progress bar in two ways:
-     * connecting to composite progress proxy (strokes) and creating
-     * progress updaters. The latter way should be deprecated in favour
-     * of displaying the status of the global strokes queue
-     */
-    image()->compositeProgressProxy()->addProxy(d->viewManager->statusBar()->progress()->progressProxy());
-    connect(d->viewManager->statusBar()->progress(), SIGNAL(sigCancellationRequested()), image(), SLOT(requestStrokeCancellation()));
-
     d->viewManager->updateGUI();
 
     KoToolManager::instance()->switchToolRequested("KritaShape/KisToolBrush");
@@ -535,19 +525,28 @@ void KisView::dropEvent(QDropEvent *event)
             QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
             QAction *insertManyLayers = new QAction(i18n("Insert Many Layers"), &popup);
 
+            QAction *insertAsNewFileLayer = new QAction(i18n("Insert as New File Layer"), &popup);
+            QAction *insertManyFileLayers = new QAction(i18n("Insert Many File Layers"), &popup);
+
             QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
             QAction *openManyDocuments = new QAction(i18n("Open Many Documents"), &popup);
 
             QAction *cancel = new QAction(i18n("Cancel"), &popup);
 
             popup.addAction(insertAsNewLayer);
+            popup.addAction(insertAsNewFileLayer);
             popup.addAction(openInNewDocument);
+
             popup.addAction(insertManyLayers);
+            popup.addAction(insertManyFileLayers);
             popup.addAction(openManyDocuments);
 
             insertAsNewLayer->setEnabled(image() && urls.count() == 1);
+            insertAsNewFileLayer->setEnabled(image() && urls.count() == 1);
             openInNewDocument->setEnabled(urls.count() == 1);
+
             insertManyLayers->setEnabled(image() && urls.count() > 1);
+            insertManyFileLayers->setEnabled(image() && urls.count() > 1);
             openManyDocuments->setEnabled(urls.count() > 1);
 
             popup.addSeparator();
@@ -556,7 +555,7 @@ void KisView::dropEvent(QDropEvent *event)
             QAction *action = popup.exec(QCursor::pos());
             if (action != 0 && action != cancel) {
                 QTemporaryFile *tmp = 0;
-                Q_FOREACH (QUrl url, urls) {
+                for (QUrl url : urls) {
 
                     if (!url.isLocalFile()) {
                         // download the file and substitute the url
@@ -574,10 +573,16 @@ void KisView::dropEvent(QDropEvent *event)
                             d->viewManager->imageManager()->importImage(url);
                             activateWindow();
                         }
+                        else if (action == insertAsNewFileLayer || action == insertManyFileLayers) {
+                            KisNodeCommandsAdapter adapter(viewManager());
+                            KisFileLayer *fileLayer = new KisFileLayer(image(), "", url.toLocalFile(),
+                                                                       KisFileLayer::None, image()->nextLayerName(), OPACITY_OPAQUE_U8);
+                            adapter.addNode(fileLayer, viewManager()->activeNode()->parent(), viewManager()->activeNode());
+                        }
                         else {
                             Q_ASSERT(action == openInNewDocument || action == openManyDocuments);
                             if (mainWindow()) {
-                                mainWindow()->openDocument(url);
+                                mainWindow()->openDocument(url, KisMainWindow::None);
                             }
                         }
                     }
@@ -601,8 +606,8 @@ void KisView::setDocument(KisDocument *document)
     d->document = document;
     QStatusBar *sb = statusBar();
     if (sb) { // No statusbar in e.g. konqueror
-        connect(d->document, SIGNAL(statusBarMessage(const QString&)),
-                this, SLOT(slotActionStatusText(const QString&)));
+        connect(d->document, SIGNAL(statusBarMessage(const QString&, int)),
+                this, SLOT(slotSavingStatusMessage(const QString&, int)));
         connect(d->document, SIGNAL(clearStatusBarMessage()),
                 this, SLOT(slotClearStatusText()));
     }
@@ -613,11 +618,6 @@ void KisView::setDocumentDeleted()
     d->documentDeleted = true;
 }
 
-KoPageLayout KisView::pageLayout() const
-{
-    return document()->pageLayout();
-}
-
 QPrintDialog *KisView::createPrintDialog(KisPrintJob *printJob, QWidget *parent)
 {
     Q_UNUSED(parent);
@@ -639,11 +639,20 @@ QStatusBar * KisView::statusBar() const
     return mw ? mw->statusBar() : 0;
 }
 
-void KisView::slotActionStatusText(const QString &text)
+void KisView::slotSavingStatusMessage(const QString &text, int timeout, bool isAutoSaving)
 {
     QStatusBar *sb = statusBar();
     if (sb)
-        sb->showMessage(text);
+        sb->showMessage(text, timeout);
+
+    KisConfig cfg;
+
+    if (sb->isHidden() ||
+        (!isAutoSaving && cfg.forceShowSaveMessages()) ||
+        (cfg.forceShowAutosaveMessages() && isAutoSaving)) {
+
+        viewManager()->showFloatingMessage(text, QIcon());
+    }
 }
 
 void KisView::slotClearStatusText()
@@ -670,7 +679,7 @@ void KisView::closeEvent(QCloseEvent *event)
     }
 
     if (queryClose()) {
-        d->viewManager->removeStatusBarItem(zoomManager()->zoomActionWidget());
+        d->viewManager->statusBar()->setView(0);
         event->accept();
         return;
     }
@@ -684,6 +693,8 @@ bool KisView::queryClose()
     if (!document())
         return true;
 
+    document()->waitForSavingToComplete();
+
     if (document()->isModified()) {
         QString name;
         if (document()->documentInfo()) {
@@ -702,15 +713,20 @@ bool KisView::queryClose()
 
         switch (res) {
         case QMessageBox::Yes : {
-            bool isNative = (document()->outputMimeType() == document()->nativeFormatMimeType());
-            if (!viewManager()->mainWindow()->saveDocument(document(), !isNative))
+            bool isNative = (document()->mimeType() == document()->nativeFormatMimeType());
+            if (!viewManager()->mainWindow()->saveDocument(document(), !isNative, false))
                 return false;
             break;
         }
-        case QMessageBox::No :
+        case QMessageBox::No : {
+            KisImageSP image = document()->image();
+            image->requestStrokeCancellation();
+            viewManager()->blockUntilOperationsFinishedForced(image);
+
             document()->removeAutoSaveFiles();
             document()->setModified(false);   // Now when queryClose() is called by closeEvent it won't do anything.
             break;
+        }
         default : // case QMessageBox::Cancel :
             return false;
         }
@@ -773,9 +789,20 @@ void KisView::resetImageSizeAndScroll(bool changeCentering,
     d->canvasController.setPreferredCenter(oldPreferredCenter - oldStillPoint + newStillPoint);
 }
 
+void KisView::syncLastActiveNodeToDocument()
+{
+    KisDocument *doc = document();
+    if (doc) {
+        doc->setPreActivatedNode(d->currentNode);
+    }
+}
+
 void KisView::setCurrentNode(KisNodeSP node)
 {
     d->currentNode = node;
+    d->canvas.slotTrySwitchShapeManager();
+
+    syncLastActiveNodeToDocument();
 }
 
 KisNodeSP KisView::currentNode() const
@@ -793,7 +820,7 @@ KisLayerSP KisView::currentLayer() const
     else {
         node = d->currentNode;
     }
-    return dynamic_cast<KisLayer*>(node.data());
+    return qobject_cast<KisLayer*>(node.data());
 }
 
 KisMaskSP KisView::currentMask() const
@@ -896,7 +923,6 @@ void KisView::slotLoadingFinished()
     connect(image(), SIGNAL(sigSizeChanged(QPointF,QPointF)), this, SIGNAL(sigSizeChanged(QPointF,QPointF)));
 
     KisNodeSP activeNode = document()->preActivatedNode();
-    document()->setPreActivatedNode(0); // to make sure that we don't keep a reference to a layer the user can later delete.
 
     if (!activeNode) {
         activeNode = image()->rootLayer()->lastChild();
