@@ -8,6 +8,7 @@
  *                2003-2011 Boudewijn Rempt <boud@valdyas.org>
  *                2004 Clarence Dang <dang@kde.org>
  *                2011 José Luis Vergara <pentalis@gmail.com>
+ *                2017 L. E. Segovia <leo.segovia@siggraph.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,8 +34,9 @@
 #include <QApplication>
 #include <QBuffer>
 #include <QByteArray>
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QDesktopWidget>
+#include <QDesktopServices>
 #include <QGridLayout>
 #include <QMainWindow>
 #include <QMenu>
@@ -83,6 +85,7 @@
 #include "kis_canvas_controls_manager.h"
 #include "kis_canvas_resource_provider.h"
 #include "kis_composite_progress_proxy.h"
+#include <KoProgressUpdater.h>
 #include "kis_config.h"
 #include "kis_config_notifier.h"
 #include "kis_control_frame.h"
@@ -92,6 +95,7 @@
 #include "kis_filter_manager.h"
 #include "kis_group_layer.h"
 #include <kis_image.h>
+#include <kis_image_barrier_locker.h>
 #include "kis_image_manager.h"
 #include <kis_layer.h>
 #include "kis_mainwindow_observer.h"
@@ -107,8 +111,8 @@
 #include <brushengine/kis_paintop_preset.h>
 #include "KisPart.h"
 #include "KisPrintJob.h"
-#include "kis_progress_widget.h"
-#include "kis_resource_server_provider.h"
+#include <KoUpdater.h>
+#include "KisResourceServerProvider.h"
 #include "kis_selection.h"
 #include "kis_selection_manager.h"
 #include "kis_shape_controller.h"
@@ -121,13 +125,15 @@
 #include <kis_undo_adapter.h>
 #include "KisView.h"
 #include "kis_zoom_manager.h"
-#include "kra/kis_kra_loader.h"
 #include "widgets/kis_floating_message.h"
 #include "kis_signal_auto_connection.h"
 #include "kis_icon_utils.h"
 #include "kis_guides_manager.h"
 #include "kis_derived_resources.h"
 #include "dialogs/kis_delayed_save_dialog.h"
+#include <kis_image.h>
+#include <KisMainWindow.h>
+#include "kis_signals_blocker.h"
 
 
 class BlockingUserInputEventFilter : public QObject
@@ -186,15 +192,9 @@ public:
         , mirrorManager(_q)
         , inputManager(_q)
         , actionAuthor(0)
+        , showPixelGrid(0)
     {
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisCompositeOpResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisEffectiveCompositeOpResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisOpacityResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisFlowResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisSizeResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisLodAvailabilityResourceConverter));
-        canvasResourceManager.addDerivedResourceConverter(toQShared(new KisEraserModeResourceConverter));
-        canvasResourceManager.addResourceUpdateMediator(toQShared(new KisPresetUpdateMediator));
+        KisViewManager::initializeResourceManager(&canvasResourceManager);
     }
 
 public:
@@ -220,6 +220,10 @@ public:
     KisSelectionManager selectionManager;
     KisGuidesManager guidesManager;
     KisStatusBar statusBar;
+    QPointer<KoUpdater> persistentImageProgressUpdater;
+
+    QScopedPointer<KoProgressUpdater> persistentUnthreadedProgressUpdaterRouter;
+    QPointer<KoUpdater> persistentUnthreadedProgressUpdater;
 
     KisControlFrame controlFrame;
     KisNodeManager nodeManager;
@@ -242,13 +246,16 @@ public:
 
     KisSignalAutoConnectionsStore viewConnections;
     KSelectAction *actionAuthor; // Select action for author profile.
+    KisAction *showPixelGrid;
 
     QByteArray canvasState;
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
+    QFlags<Qt::WindowState> windowFlags;
+#endif
 
-    bool blockUntillOperationsFinishedImpl(KisImageSP image, bool force);
+    bool blockUntilOperationsFinishedImpl(KisImageSP image, bool force);
 };
 
-
 KisViewManager::KisViewManager(QWidget *parent, KActionCollection *_actionCollection)
     : d(new KisViewManagerPrivate(this, _actionCollection, parent))
 {
@@ -262,6 +269,23 @@ KisViewManager::KisViewManager(QWidget *parent, KActionCollection *_actionCollec
 
     // These initialization functions must wait until KisViewManager ctor is complete.
     d->statusBar.setup();
+    d->persistentImageProgressUpdater =
+        d->statusBar.progressUpdater()->startSubtask(1, "", true);
+    // reset state to "completed"
+    d->persistentImageProgressUpdater->setRange(0,100);
+    d->persistentImageProgressUpdater->setValue(100);
+
+    d->persistentUnthreadedProgressUpdater =
+        d->statusBar.progressUpdater()->startSubtask(1, "", true);
+        // reset state to "completed"
+    d->persistentUnthreadedProgressUpdater->setRange(0,100);
+    d->persistentUnthreadedProgressUpdater->setValue(100);
+
+    d->persistentUnthreadedProgressUpdaterRouter.reset(
+        new KoProgressUpdater(d->persistentUnthreadedProgressUpdater,
+                              KoProgressUpdater::Unthreaded));
+    d->persistentUnthreadedProgressUpdaterRouter->setAutoNestNames(true);
+
     d->controlFrame.setup(parent);
 
     //Check to draw scrollbars after "Canvas only mode" toggle is created.
@@ -288,6 +312,7 @@ KisViewManager::KisViewManager(QWidget *parent, KActionCollection *_actionCollec
     connect(KisPart::instance(), SIGNAL(sigViewRemoved(KisView*)), SLOT(slotViewRemoved(KisView*)));
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(slotUpdateAuthorProfileActions()));
+    connect(KisConfigNotifier::instance(), SIGNAL(pixelGridModeChanged()), SLOT(slotUpdatePixelGridAction()));
 
     KisInputProfileManager::instance()->loadProfiles();
 
@@ -312,6 +337,20 @@ KisViewManager::~KisViewManager()
     delete d;
 }
 
+void KisViewManager::initializeResourceManager(KoCanvasResourceManager *resourceManager)
+{
+    resourceManager->addDerivedResourceConverter(toQShared(new KisCompositeOpResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisEffectiveCompositeOpResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisOpacityResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisFlowResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisSizeResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisLodAvailabilityResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisLodSizeThresholdResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisLodSizeThresholdSupportedResourceConverter));
+    resourceManager->addDerivedResourceConverter(toQShared(new KisEraserModeResourceConverter));
+    resourceManager->addResourceUpdateMediator(toQShared(new KisPresetUpdateMediator));
+}
+
 KActionCollection *KisViewManager::actionCollection() const
 {
     return d->actionCollection;
@@ -349,6 +388,7 @@ void KisViewManager::setCurrentView(KisView *view)
         first = false;
         KisDocument* doc = d->currentImageView->document();
         if (doc) {
+            doc->image()->compositeProgressProxy()->removeProxy(d->persistentImageProgressUpdater);
             doc->disconnect(this);
         }
         d->currentImageView->canvasController()->proxyObject->disconnect(&d->statusBar);
@@ -371,10 +411,23 @@ void KisViewManager::setCurrentView(KisView *view)
         if (first) {
             KisConfig cfg;
             KisPaintOpPresetResourceServer * rserver = KisResourceServerProvider::instance()->paintOpPresetServer();
-            QString lastPreset = cfg.readEntry("LastPreset", QString("Basic_tip_default"));
+            QString defaultPresetName = "basic_tip_default";
+            bool foundTip = false;
+            for (int i=0; i<rserver->resourceCount(); i++) {
+                KisPaintOpPresetSP resource = rserver->resources().at(i);
+                if (resource->name().toLower().contains("basic_tip_default")) {
+                    defaultPresetName = resource->name();
+                    foundTip = true;
+                } else if (foundTip == false && (resource->name().toLower().contains("default") ||
+                                                 resource->filename().toLower().contains("default"))) {
+                    defaultPresetName = resource->name();
+                    foundTip = true;
+                }
+            }
+            QString lastPreset = cfg.readEntry("LastPreset", defaultPresetName);
             KisPaintOpPresetSP preset = rserver->resourceByName(lastPreset);
             if (!preset) {
-                preset = rserver->resourceByName("Basic_tip_default");
+                preset = rserver->resourceByName(defaultPresetName);
             }
 
             if (!preset) {
@@ -415,6 +468,12 @@ void KisViewManager::setCurrentView(KisView *view)
         d->viewConnections.addUniqueConnection(d->softProof, SIGNAL(toggled(bool)), view, SLOT(slotSoftProofing(bool)) );
         d->viewConnections.addUniqueConnection(d->gamutCheck, SIGNAL(toggled(bool)), view, SLOT(slotGamutCheck(bool)) );
 
+        // set up progrress reporting
+        doc->image()->compositeProgressProxy()->addProxy(d->persistentImageProgressUpdater);
+        d->viewConnections.addUniqueConnection(&d->statusBar, SIGNAL(sigCancellationRequested()), doc->image(), SLOT(requestStrokeCancellation()));
+
+        d->viewConnections.addUniqueConnection(d->showPixelGrid, SIGNAL(toggled(bool)), canvasController, SLOT(slotTogglePixelGrid(bool)));
+
         imageView->zoomManager()->setShowRulers(d->showRulersAction->isChecked());
         imageView->zoomManager()->setRulersTrackMouse(d->rulersTrackMouseAction->isChecked());
 
@@ -523,9 +582,14 @@ KisPaintopBox* KisViewManager::paintOpBox() const
     return d->controlFrame.paintopBox();
 }
 
-KoProgressUpdater* KisViewManager::createProgressUpdater(KoProgressUpdater::Mode mode)
+QPointer<KoUpdater> KisViewManager::createUnthreadedUpdater(const QString &name)
 {
-    return new KisProgressUpdater(d->statusBar.progress(), document()->progressProxy(), mode);
+    return d->persistentUnthreadedProgressUpdaterRouter->startSubtask(1, name, false);
+}
+
+QPointer<KoUpdater> KisViewManager::createThreadedUpdater(const QString &name)
+{
+    return d->statusBar.progressUpdater()->startSubtask(1, name, false);
 }
 
 KisSelectionManager * KisViewManager::selectionManager()
@@ -675,6 +739,8 @@ void KisViewManager::createActions()
     actionCollection()->addAction("settings_active_author", d->actionAuthor);
     slotUpdateAuthorProfileActions();
 
+    d->showPixelGrid = actionManager()->createAction("view_pixel_grid");
+    slotUpdatePixelGridAction();
 }
 
 void KisViewManager::setupManagers()
@@ -700,7 +766,6 @@ void KisViewManager::setupManagers()
     d->canvasControlsManager.setup(actionManager());
 
     d->mirrorManager.setup(actionCollection());
-
 }
 
 void KisViewManager::updateGUI()
@@ -734,11 +799,6 @@ KisGuidesManager * KisViewManager::guidesManager() const
     return &d->guidesManager;
 }
 
-KisPaintingAssistantsManager* KisViewManager::paintingAssistantsManager() const
-{
-    return &d->paintingAssistantsManager;
-}
-
 KisDocument *KisViewManager::document() const
 {
     if (d->currentImageView && d->currentImageView->document()) {
@@ -756,7 +816,7 @@ int KisViewManager::viewCount() const
     return 0;
 }
 
-bool KisViewManager::KisViewManagerPrivate::blockUntillOperationsFinishedImpl(KisImageSP image, bool force)
+bool KisViewManager::KisViewManagerPrivate::blockUntilOperationsFinishedImpl(KisImageSP image, bool force)
 {
     const int busyWaitDelay = 1000;
     KisDelayedSaveDialog dialog(image, !force ? KisDelayedSaveDialog::GeneralDialog : KisDelayedSaveDialog::ForcedDialog, busyWaitDelay, mainWindow);
@@ -766,14 +826,14 @@ bool KisViewManager::KisViewManagerPrivate::blockUntillOperationsFinishedImpl(Ki
 }
 
 
-bool KisViewManager::blockUntillOperationsFinished(KisImageSP image)
+bool KisViewManager::blockUntilOperationsFinished(KisImageSP image)
 {
-    return d->blockUntillOperationsFinishedImpl(image, false);
+    return d->blockUntilOperationsFinishedImpl(image, false);
 }
 
-void KisViewManager::blockUntillOperationsFinishedForced(KisImageSP image)
+void KisViewManager::blockUntilOperationsFinishedForced(KisImageSP image)
 {
-    d->blockUntillOperationsFinishedImpl(image, true);
+    d->blockUntilOperationsFinishedImpl(image, true);
 }
 
 void KisViewManager::slotCreateTemplate()
@@ -784,19 +844,25 @@ void KisViewManager::slotCreateTemplate()
 
 void KisViewManager::slotCreateCopy()
 {
-    if (!document()) return;
-    KisDocument *doc = KisPart::instance()->createDocument();
+    KisDocument *srcDoc = document();
+    if (!srcDoc) return;
 
-    QString name = document()->documentInfo()->aboutInfo("name");
+    if (!this->blockUntilOperationsFinished(srcDoc->image())) return;
+
+    KisDocument *doc = 0;
+    {
+        KisImageBarrierLocker l(srcDoc->image());
+        doc = srcDoc->clone();
+    }
+    KIS_SAFE_ASSERT_RECOVER_RETURN(doc);
+
+    QString name = srcDoc->documentInfo()->aboutInfo("name");
     if (name.isEmpty()) {
         name = document()->url().toLocalFile();
     }
     name = i18n("%1 (Copy)", name);
     doc->documentInfo()->setAboutInfo("title", name);
-    KisImageWSP image = document()->image();
-    KisImageSP newImage = new KisImage(doc->createUndoStore(), image->width(), image->height(), image->colorSpace(), name);
-    newImage->setRootLayer(dynamic_cast<KisGroupLayer*>(image->rootLayer()->clone().data()));
-    doc->setCurrentImage(newImage);
+
     KisPart::instance()->addDocument(doc);
     KisMainWindow *mw  = qobject_cast<KisMainWindow*>(d->mainWindow);
     mw->addViewAndNotifyLoadingCompleted(doc);
@@ -821,7 +887,6 @@ void KisViewManager::setQtMainWindow(QMainWindow* newMainWindow)
     d->mainWindow = newMainWindow;
 }
 
-
 void KisViewManager::slotDocumentSaved()
 {
     d->saveIncremental->setEnabled(true);
@@ -832,6 +897,12 @@ void KisViewManager::slotSaveIncremental()
 {
     if (!document()) return;
 
+    if (document()->url().isEmpty()) {
+        KisMainWindow *mw = qobject_cast<KisMainWindow*>(d->mainWindow);
+        mw->saveDocument(document(), true, false);
+        return;
+    }
+
     bool foundVersion;
     bool fileAlreadyExists;
     bool isBackup;
@@ -868,6 +939,7 @@ void KisViewManager::slotSaveIncremental()
         }
         version.remove(0, 1);            //  Trim "_"
     } else {
+        // TODO: this will not work with files extensions like jp2
         // ...else, simply add a version to it so the next loop works
         QRegExp regex2("[.][a-z]{2,4}$");  //  Heuristic to find file extension
         regex2.indexIn(fileName);
@@ -914,7 +986,7 @@ void KisViewManager::slotSaveIncremental()
         return;
     }
     document()->setFileBatchMode(true);
-    document()->saveAs(QUrl::fromUserInput(fileName));
+    document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
     document()->setFileBatchMode(false);
 
     if (mainWindow()) {
@@ -927,6 +999,12 @@ void KisViewManager::slotSaveIncrementalBackup()
 {
     if (!document()) return;
 
+    if (document()->url().isEmpty()) {
+        KisMainWindow *mw = qobject_cast<KisMainWindow*>(d->mainWindow);
+        mw->saveDocument(document(), true, false);
+        return;
+    }
+
     bool workingOnBackup;
     bool fileAlreadyExists;
     QString version = "000";
@@ -985,7 +1063,7 @@ void KisViewManager::slotSaveIncrementalBackup()
             return;
         }
         QFile::copy(fileName, backupFileName);
-        document()->saveAs(QUrl::fromUserInput(fileName));
+        document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
 
         if (mainWindow()) mainWindow()->updateCaption();
     }
@@ -1023,7 +1101,7 @@ void KisViewManager::slotSaveIncrementalBackup()
         // Save both as backup and on current file for interapplication workflow
         document()->setFileBatchMode(true);
         QFile::copy(fileName, backupFileName);
-        document()->saveAs(QUrl::fromUserInput(fileName));
+        document()->saveAs(QUrl::fromUserInput(fileName), document()->mimeType(), true);
         document()->setFileBatchMode(false);
 
         if (mainWindow()) mainWindow()->updateCaption();
@@ -1072,6 +1150,9 @@ void KisViewManager::switchCanvasOnly(bool toggled)
 
     if (toggled) {
         d->canvasState = qtMainWindow()->saveState();
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
+        d->windowFlags = main->windowState();
+#endif
     }
 
     if (cfg.hideStatusbarFullscreen()) {
@@ -1107,11 +1188,19 @@ void KisViewManager::switchCanvasOnly(bool toggled)
         }
     }
 
+    // QT in windows does not return to maximized upon 4th tab in a row
+    // https://bugreports.qt.io/browse/QTBUG-57882, https://bugreports.qt.io/browse/QTBUG-52555, https://codereview.qt-project.org/#/c/185016/
     if (cfg.hideTitlebarFullscreen() && !cfg.fullscreenMode()) {
         if(toggled) {
             main->setWindowState( main->windowState() | Qt::WindowFullScreen);
         } else {
             main->setWindowState( main->windowState() & ~Qt::WindowFullScreen);
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
+            // If window was maximized prior to fullscreen, restore that
+            if (d->windowFlags & Qt::WindowMaximized) {
+                main->setWindowState( main->windowState() | Qt::WindowMaximized);
+            }
+#endif
         }
     }
 
@@ -1187,7 +1276,6 @@ void KisViewManager::updateIcons()
             while (!objects.isEmpty()) {
                 QObject* object = objects.takeFirst();
                 objects.append(object->children());
-
                 KisIconUtils::updateIconCommon(object);
             }
         }
@@ -1262,10 +1350,8 @@ void KisViewManager::setShowFloatingMessage(bool show)
 void KisViewManager::changeAuthorProfile(const QString &profileName)
 {
     KConfigGroup appAuthorGroup(KoGlobal::calligraConfig(), "Author");
-    if (profileName.isEmpty()) {
+    if (profileName.isEmpty() || profileName == i18nc("choice for author profile", "Anonymous")) {
         appAuthorGroup.writeEntry("active-profile", "");
-    } else if (profileName == i18nc("choice for author profile", "Anonymous")) {
-        appAuthorGroup.writeEntry("active-profile", "anonymous");
     } else {
         appAuthorGroup.writeEntry("active-profile", profileName);
     }
@@ -1282,11 +1368,20 @@ void KisViewManager::slotUpdateAuthorProfileActions()
         return;
     }
     d->actionAuthor->clear();
-    d->actionAuthor->addAction(i18n("Default Author Profile"));
     d->actionAuthor->addAction(i18nc("choice for author profile", "Anonymous"));
 
     KConfigGroup authorGroup(KoGlobal::calligraConfig(), "Author");
     QStringList profiles = authorGroup.readEntry("profile-names", QStringList());
+    QString authorInfo = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/authorinfo/";
+    QStringList filters = QStringList() << "*.authorinfo";
+    QDir dir(authorInfo);
+    Q_FOREACH(QString entry, dir.entryList(filters)) {
+        int ln = QString(".authorinfo").size();
+        entry.chop(ln);
+        if (!profiles.contains(entry)) {
+            profiles.append(entry);
+        }
+    }
     Q_FOREACH (const QString &profile , profiles) {
         d->actionAuthor->addAction(profile);
     }
@@ -1294,11 +1389,19 @@ void KisViewManager::slotUpdateAuthorProfileActions()
     KConfigGroup appAuthorGroup(KoGlobal::calligraConfig(), "Author");
     QString profileName = appAuthorGroup.readEntry("active-profile", "");
 
-    if (profileName == "anonymous") {
-        d->actionAuthor->setCurrentItem(1);
+    if (profileName == "anonymous" || profileName.isEmpty()) {
+        d->actionAuthor->setCurrentItem(0);
     } else if (profiles.contains(profileName)) {
         d->actionAuthor->setCurrentAction(profileName);
-    } else {
-        d->actionAuthor->setCurrentItem(0);
     }
 }
+
+void KisViewManager::slotUpdatePixelGridAction()
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(d->showPixelGrid);
+
+    KisSignalsBlocker b(d->showPixelGrid);
+
+    KisConfig cfg;
+    d->showPixelGrid->setChecked(cfg.pixelGridEnabled());
+}
