@@ -33,6 +33,7 @@
 
 #include <KoUnit.h>
 #include <KoShapeManager.h>
+#include <KisSelectedShapesProxy.h>
 #include <KoColorProfile.h>
 #include <KoCanvasControllerWidget.h>
 #include <KisDocument.h>
@@ -45,6 +46,7 @@
 #include "kis_prescaled_projection.h"
 #include "kis_image.h"
 #include "kis_image_barrier_locker.h"
+#include "kis_undo_adapter.h"
 #include "KisDocument.h"
 #include "flake/kis_shape_layer.h"
 #include "kis_canvas_resource_provider.h"
@@ -84,6 +86,10 @@
 #include "kis_painting_assistants_decoration.h"
 
 #include "kis_canvas_updates_compressor.h"
+#include "KoZoomController.h"
+
+#include <KisStrokeSpeedMonitor.h>
+#include "opengl/kis_opengl_canvas_debugger.h"
 
 class Q_DECL_HIDDEN KisCanvas2::KisCanvas2Private
 {
@@ -94,6 +100,7 @@ public:
         : coordinatesConverter(coordConverter)
         , view(view)
         , shapeManager(parent)
+        , selectedShapesProxy(&shapeManager)
         , toolProxy(parent)
         , displayColorConverter(resourceManager, view)
     {
@@ -103,6 +110,7 @@ public:
     QPointer<KisView>view;
     KisAbstractCanvasWidget *canvasWidget = 0;
     KoShapeManager shapeManager;
+    KisSelectedShapesProxy selectedShapesProxy;
     bool currentCanvasIsOpenGL;
     int openGLFilterMode;
     KisToolProxy toolProxy;
@@ -126,12 +134,41 @@ public:
     KisAnimationFrameCacheSP frameCache;
     bool lodAllowedInCanvas;
     bool bootstrapLodBlocked;
+    QPointer<KoShapeManager> currentlyActiveShapeManager;
+    KisInputActionGroupsMask inputActionGroupsMask = AllActionGroup;
 
     bool effectiveLodAllowedInCanvas() {
         return lodAllowedInCanvas && !bootstrapLodBlocked;
     }
 };
 
+namespace {
+KoShapeManager* fetchShapeManagerFromNode(KisNodeSP node)
+{
+    KoShapeManager *shapeManager = 0;
+
+    KisLayer *layer = dynamic_cast<KisLayer*>(node.data());
+
+    if (layer) {
+        KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(layer);
+        if (shapeLayer) {
+            shapeManager = shapeLayer->shapeManager();
+
+        } else {
+            KisSelectionSP selection = layer->selection();
+            if (selection && selection->hasShapeSelection()) {
+                KisShapeSelection *shapeSelection = dynamic_cast<KisShapeSelection*>(selection->shapeSelection());
+                KIS_ASSERT_RECOVER_RETURN_VALUE(shapeSelection, 0);
+
+                shapeManager = shapeSelection->shapeManager();
+            }
+        }
+    }
+
+    return shapeManager;
+}
+}
+
 KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceManager *resourceManager, KisView *view, KoShapeBasedDocumentBase *sc)
     : KoCanvasBase(sc, resourceManager)
     , m_d(new KisCanvas2Private(this, coordConverter, view, resourceManager))
@@ -144,10 +181,10 @@ KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResource
     m_d->bootstrapLodBlocked = true;
     connect(view->mainWindow(), SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
 
-    m_d->updateSignalCompressor.setDelay(10);
+    KisImageConfig config;
+
+    m_d->updateSignalCompressor.setDelay(1000 / config.fpsLimit());
     m_d->updateSignalCompressor.setMode(KisSignalCompressor::FIRST_ACTIVE);
-
-
 }
 
 void KisCanvas2::setup()
@@ -188,6 +225,28 @@ void KisCanvas2::setup()
             globalShapeManager()->selection(), SIGNAL(currentLayerChanged(const KoShapeLayer*)));
 
     connect(&m_d->updateSignalCompressor, SIGNAL(timeout()), SLOT(slotDoCanvasUpdate()));
+
+    initializeFpsDecoration();
+}
+
+void KisCanvas2::initializeFpsDecoration()
+{
+    KisConfig cfg;
+
+    const bool shouldShowDebugOverlay =
+        (canvasIsOpenGL() && cfg.enableOpenGLFramerateLogging()) ||
+        cfg.enableBrushSpeedLogging();
+
+    if (shouldShowDebugOverlay && !decoration(KisFpsDecoration::idTag)) {
+        addDecoration(new KisFpsDecoration(imageView()));
+
+        if (cfg.enableBrushSpeedLogging()) {
+            connect(KisStrokeSpeedMonitor::instance(), SIGNAL(sigStatsUpdated()), this, SLOT(updateCanvas()));
+        }
+    } else if (!shouldShowDebugOverlay && decoration(KisFpsDecoration::idTag)) {
+        m_d->canvasWidget->removeDecoration(KisFpsDecoration::idTag);
+        disconnect(KisStrokeSpeedMonitor::instance(), SIGNAL(sigStatsUpdated()), this, SLOT(updateCanvas()));
+    }
 }
 
 KisCanvas2::~KisCanvas2()
@@ -286,15 +345,15 @@ bool KisCanvas2::yAxisMirrored() const
 
 void KisCanvas2::channelSelectionChanged()
 {
-    KisImageWSP image = this->image();
+    KisImageSP image = this->image();
     m_d->channelFlags = image->rootLayer()->channelFlags();
 
+    m_d->view->viewManager()->blockUntilOperationsFinishedForced(image);
+
     image->barrierLock();
     m_d->canvasWidget->channelSelectionChanged(m_d->channelFlags);
     startUpdateInPatches(image->bounds());
-
     image->unlock();
-
 }
 
 void KisCanvas2::addCommand(KUndo2Command *command)
@@ -305,28 +364,23 @@ void KisCanvas2::addCommand(KUndo2Command *command)
 
 KoShapeManager* KisCanvas2::shapeManager() const
 {
-    if (!viewManager()) return globalShapeManager();
-    if (!viewManager()->nodeManager()) return globalShapeManager();
+    KisNodeSP node = m_d->view->currentNode();
+    KoShapeManager *localShapeManager = fetchShapeManagerFromNode(node);
 
-    KisLayerSP activeLayer = viewManager()->nodeManager()->activeLayer();
-    if (activeLayer && activeLayer->isEditable()) {
-        KisShapeLayer * shapeLayer = dynamic_cast<KisShapeLayer*>(activeLayer.data());
-        if (shapeLayer) {
-            return shapeLayer->shapeManager();
-        }
-        KisSelectionSP selection = activeLayer->selection();
-        if (selection && !selection.isNull()) {
-            if (selection->hasShapeSelection()) {
-                KoShapeManager* m = dynamic_cast<KisShapeSelection*>(selection->shapeSelection())->shapeManager();
-                return m;
-            }
-
-        }
+    // sanity check for consistency of the local shape manager
+    KIS_SAFE_ASSERT_RECOVER (localShapeManager == m_d->currentlyActiveShapeManager) {
+        localShapeManager = globalShapeManager();
     }
-    return globalShapeManager();
+
+    return localShapeManager ? localShapeManager : globalShapeManager();
 }
 
-KoShapeManager * KisCanvas2::globalShapeManager() const
+KoSelectedShapesProxy* KisCanvas2::selectedShapesProxy() const
+{
+    return &m_d->selectedShapesProxy;
+}
+
+KoShapeManager* KisCanvas2::globalShapeManager() const
 {
     return &m_d->shapeManager;
 }
@@ -351,6 +405,16 @@ KisInputManager* KisCanvas2::globalInputManager() const
     return m_d->view->globalInputManager();
 }
 
+KisInputActionGroupsMask KisCanvas2::inputActionGroupsMask() const
+{
+    return m_d->inputActionGroupsMask;
+}
+
+void KisCanvas2::setInputActionGroupsMask(KisInputActionGroupsMask mask)
+{
+    m_d->inputActionGroupsMask = mask;
+}
+
 QWidget* KisCanvas2::canvasWidget()
 {
     return m_d->canvasWidget->widget();
@@ -411,10 +475,6 @@ void KisCanvas2::createOpenGLCanvas()
     m_d->frameCache = KisAnimationFrameCache::getFrameCache(canvasWidget->openGLImageTextures());
 
     setCanvasWidget(canvasWidget);
-
-    if (canvasWidget->needsFpsDebugging() && !decoration(KisFpsDecoration::idTag)) {
-        addDecoration(new KisFpsDecoration(imageView()));
-    }
 }
 
 void KisCanvas2::createCanvas(bool useOpenGL)
@@ -453,15 +513,17 @@ void KisCanvas2::createCanvas(bool useOpenGL)
 
 void KisCanvas2::initializeImage()
 {
-    KisImageWSP image = m_d->view->image();
+    KisImageSP image = m_d->view->image();
 
     m_d->coordinatesConverter->setImage(image);
+    m_d->toolProxy.initializeImage(image);
 
     connect(image, SIGNAL(sigImageUpdated(QRect)), SLOT(startUpdateCanvasProjection(QRect)), Qt::DirectConnection);
     connect(this, SIGNAL(sigCanvasCacheUpdated()), SLOT(updateCanvasProjection()));
     connect(image, SIGNAL(sigProofingConfigChanged()), SLOT(slotChangeProofingConfig()));
     connect(image, SIGNAL(sigSizeChanged(const QPointF&, const QPointF&)), SLOT(startResizingImage()), Qt::DirectConnection);
     connect(this, SIGNAL(sigContinueResizeImage(qint32,qint32)), SLOT(finishResizingImage(qint32,qint32)));
+    connect(image->undoAdapter(), SIGNAL(selectionChanged()), SLOT(slotTrySwitchShapeManager()));
 
     connectCurrentCanvas();
 }
@@ -522,12 +584,12 @@ void KisCanvas2::startUpdateInPatches(const QRect &imageRect)
 void KisCanvas2::setDisplayFilter(QSharedPointer<KisDisplayFilter> displayFilter)
 {
     m_d->displayColorConverter.setDisplayFilter(displayFilter);
-    KisImageWSP image = this->image();
+    KisImageSP image = this->image();
+
+    m_d->view->viewManager()->blockUntilOperationsFinishedForced(image);
 
     image->barrierLock();
-
     m_d->canvasWidget->setDisplayFilter(displayFilter);
-
     image->unlock();
 }
 
@@ -559,17 +621,26 @@ void KisCanvas2::setProofingOptions(bool softProof, bool gamutCheck)
         m_d->proofingConfig = cfg.defaultProofingconfiguration();
     }
     KoColorConversionTransformation::ConversionFlags conversionFlags = m_d->proofingConfig->conversionFlags;
+#if QT_VERSION >= 0x050700
 
-    if (softProof && this->image()->colorSpace()->colorDepthId().id().contains("U")) {
+    if (this->image()->colorSpace()->colorDepthId().id().contains("U")) {
+        conversionFlags.setFlag(KoColorConversionTransformation::SoftProofing, softProof);
+        if (softProof) {
+            conversionFlags.setFlag(KoColorConversionTransformation::GamutCheck, gamutCheck);
+        }
+    }
+#else
+    if (this->image()->colorSpace()->colorDepthId().id().contains("U")) {
         conversionFlags |= KoColorConversionTransformation::SoftProofing;
     } else {
-        conversionFlags = conversionFlags &  ~KoColorConversionTransformation::SoftProofing;
+        conversionFlags = conversionFlags & ~KoColorConversionTransformation::SoftProofing;
     }
     if (gamutCheck && softProof && this->image()->colorSpace()->colorDepthId().id().contains("U")) {
         conversionFlags |= KoColorConversionTransformation::GamutCheck;
     } else {
         conversionFlags = conversionFlags & ~KoColorConversionTransformation::GamutCheck;
     }
+#endif
     m_d->proofingConfig->conversionFlags = conversionFlags;
 
     m_d->proofingConfigUpdated = true;
@@ -609,9 +680,7 @@ KisProofingConfigurationSP KisCanvas2::proofingConfiguration() const
     if (!m_d->proofingConfig) {
         m_d->proofingConfig = this->image()->proofingConfiguration();
         if (!m_d->proofingConfig) {
-            qDebug()<<"Canvas: No proofing config found, generating one.";
-            KisImageConfig cfg;
-            m_d->proofingConfig = cfg.defaultProofingconfiguration();
+            m_d->proofingConfig = KisImageConfig().defaultProofingconfiguration();
         }
     }
     return m_d->proofingConfig;
@@ -723,6 +792,21 @@ void KisCanvas2::notifyZoomChanged()
     updateCanvas(); // update the canvas, because that isn't done when zooming using KoZoomAction
 }
 
+void KisCanvas2::slotTrySwitchShapeManager()
+{
+    QPointer<KoShapeManager> oldManager = m_d->currentlyActiveShapeManager;
+
+    KisNodeSP node = m_d->view->currentNode();
+
+    QPointer<KoShapeManager> newManager;
+    newManager = fetchShapeManagerFromNode(node);
+
+    if (newManager != oldManager) {
+        m_d->currentlyActiveShapeManager = newManager;
+        m_d->selectedShapesProxy.setShapeManager(newManager);
+    }
+}
+
 void KisCanvas2::notifyLevelOfDetailChange()
 {
     if (!m_d->effectiveLodAllowedInCanvas()) return;
@@ -738,14 +822,6 @@ void KisCanvas2::notifyLevelOfDetailChange()
     image->setDesiredLevelOfDetail(lod);
 }
 
-void KisCanvas2::preScale()
-{
-    if (!m_d->currentCanvasIsOpenGL) {
-        Q_ASSERT(m_d->prescaledProjection);
-        m_d->prescaledProjection->preScale();
-    }
-}
-
 const KoColorProfile *  KisCanvas2::monitorProfile()
 {
     return m_d->displayColorConverter.monitorProfile();
@@ -799,6 +875,7 @@ void KisCanvas2::slotConfigChanged()
     resetCanvas(cfg.useOpenGL());
     slotSetDisplayProfile(cfg.displayProfile(QApplication::desktop()->screenNumber(this->canvasWidget())));
 
+    initializeFpsDecoration();
 }
 
 void KisCanvas2::refetchDataFromImage()
@@ -852,10 +929,20 @@ QPoint KisCanvas2::documentOffset() const
 
 void KisCanvas2::setFavoriteResourceManager(KisFavoriteResourceManager* favoriteResourceManager)
 {
-    m_d->popupPalette = new KisPopupPalette(favoriteResourceManager, displayColorConverter()->displayRendererInterface(), m_d->view->resourceProvider(), m_d->canvasWidget->widget());
+    m_d->popupPalette = new KisPopupPalette(viewManager(), m_d->coordinatesConverter, favoriteResourceManager, displayColorConverter()->displayRendererInterface(),
+                                            m_d->view->resourceProvider(), m_d->canvasWidget->widget());
+    connect(m_d->popupPalette, SIGNAL(zoomLevelChanged(int)), this, SLOT(slotZoomChanged(int)));
+    connect(m_d->popupPalette, SIGNAL(sigUpdateCanvas()), this, SLOT(updateCanvas()));
+    connect(m_d->view->mainWindow(), SIGNAL(themeChanged()), m_d->popupPalette, SLOT(slotUpdateIcons()));
+
     m_d->popupPalette->showPopupPalette(false);
 }
 
+void KisCanvas2::slotZoomChanged(int zoom ) {
+    m_d->view->viewManager()->zoomController()->setZoom(KoZoomMode::ZOOM_CONSTANT, (qreal)(zoom/100.0)); // 1.0 is 100% zoom
+    notifyZoomChanged();
+}
+
 void KisCanvas2::setCursor(const QCursor &cursor)
 {
     canvasWidget()->setCursor(cursor);
