@@ -38,6 +38,7 @@
 #include "kis_coordinates_converter.h"
 #include "kis_canvas_decoration.h"
 #include "kis_painting_assistants_decoration.h"
+#include "input/KisInputActionGroup.h"
 
 class KoToolProxy;
 class KoColorProfile;
@@ -60,7 +61,7 @@ class KoViewConverter;
  * the widget it contains, which may be either a QPainter based
  * canvas, or an OpenGL based canvas: that are the real widgets.
  */
-class KRITAUI_EXPORT KisCanvas2 : public QObject, public KoCanvasBase
+class KRITAUI_EXPORT KisCanvas2 : public KoCanvasBase, public KisInputActionGroupsMaskInterface
 {
 
     Q_OBJECT
@@ -76,26 +77,26 @@ public:
      */
     KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceManager *resourceManager, KisView *view, KoShapeBasedDocumentBase *sc);
 
-    virtual ~KisCanvas2();
+    ~KisCanvas2() override;
 
     void notifyZoomChanged();
 
-    virtual void disconnectCanvasObserver(QObject *object);
+    void disconnectCanvasObserver(QObject *object) override;
 
 public: // KoCanvasBase implementation
 
-    bool canvasIsOpenGL() const;
+    bool canvasIsOpenGL() const override;
 
     KisOpenGL::FilterMode openGLFilterMode() const;
 
-    void gridSize(QPointF *offset, QSizeF *spacing) const;
+    void gridSize(QPointF *offset, QSizeF *spacing) const override;
 
-    bool snapToGrid() const;
+    bool snapToGrid() const override;
 
-    // XXX: Why?
-    void addCommand(KUndo2Command *command);
+    // This method only exists to support flake-related operations
+    void addCommand(KUndo2Command *command) override;
 
-    virtual QPoint documentOrigin() const;
+    QPoint documentOrigin() const override;
     QPoint documentOffset() const;
 
     /**
@@ -104,36 +105,36 @@ public: // KoCanvasBase implementation
      * layer's canvas' shapemanager, else the shapemanager associated
      * with the global krita canvas.
      */
-    KoShapeManager * shapeManager() const;
+    KoShapeManager * shapeManager() const override;
+
+    /**
+     * Since shapeManager() may change, we need a persistent object where we can
+     * connect to and thack the selection. See more comments in KoCanvasBase.
+     */
+    KoSelectedShapesProxy *selectedShapesProxy() const override;
 
     /**
      * Return the shape manager associated with this canvas
      */
-    KoShapeManager * globalShapeManager() const;
+    KoShapeManager *globalShapeManager() const;
 
-    void updateCanvas(const QRectF& rc);
+    void updateCanvas(const QRectF& rc) override;
 
-    virtual void updateInputMethodInfo();
+    void updateInputMethodInfo() override;
 
     const KisCoordinatesConverter* coordinatesConverter() const;
-    virtual KoViewConverter *viewConverter() const;
+    KoViewConverter *viewConverter() const override;
 
-    virtual QWidget* canvasWidget();
+    QWidget* canvasWidget() override;
 
-    virtual const QWidget* canvasWidget() const;
+    const QWidget* canvasWidget() const override;
 
-    virtual KoUnit unit() const;
+    KoUnit unit() const override;
 
-    virtual KoToolProxy* toolProxy() const;
+    KoToolProxy* toolProxy() const override;
 
     const KoColorProfile* monitorProfile();
 
-    /**
-     * Prescale the canvas represention of the image (if necessary, it
-     * is for QPainter, not for OpenGL).
-     */
-    void preScale();
-
     // FIXME:
     // Temporary! Either get the current layer and image from the
     // resource provider, or use this, which gets them from the
@@ -148,6 +149,18 @@ public: // KoCanvasBase implementation
      */
     KisInputManager* globalInputManager() const;
 
+    /**
+     * Return the mask of currently available input action groups
+     * Note: Override from KisInputActionGroupsMaskInterface
+     */
+    KisInputActionGroupsMask inputActionGroupsMask() const override;
+
+    /**
+     * Set the mask of currently available action groups
+     * Note: Override from KisInputActionGroupsMaskInterface
+     */
+    void setInputActionGroupsMask(KisInputActionGroupsMask mask) override;
+
     KisPaintingAssistantsDecorationSP paintingAssistantsDecoration() const;
 
 
@@ -166,25 +179,29 @@ public: // KisCanvas2 methods
 
     KisDisplayColorConverter *displayColorConverter() const;
     KisExposureGammaCorrectionInterface* exposureGammaCorrectionInterface() const;
+
     /**
      * @brief setProofingOptions
      * set the options for softproofing, without affecting the proofing options as stored inside the image.
      */
     void setProofingOptions(bool softProof, bool gamutCheck);
     KisProofingConfigurationSP proofingConfiguration() const;
+
     /**
      * @brief setProofingConfigUpdated This function is to set whether the proofing config is updated,
      * this is needed for determining whether or not to generate a new proofing transform.
      * @param updated whether it's updated. Just set it to false in normal usage.
      */
     void setProofingConfigUpdated(bool updated);
+
     /**
      * @brief proofingConfigUpdated ask the canvas whether or not it updated the proofing config.
      * @return whether or not the proofing config is updated, if so, a new proofing transform needs to be made
      * in KisOpenGL canvas.
-     */bool proofingConfigUpdated();
+     */
+    bool proofingConfigUpdated();
 
-    void setCursor(const QCursor &cursor);
+    void setCursor(const QCursor &cursor) override;
     KisAnimationFrameCacheSP frameCache() const;
     KisAnimationPlayer *animationPlayer() const;
     void refetchDataFromImage();
@@ -216,6 +233,7 @@ public Q_SLOTS:
     void slotSoftProofing(bool softProofing);
     void slotGamutCheck(bool gamutCheck);
     void slotChangeProofingConfig();
+    void slotZoomChanged(int zoom);
 
     void channelSelectionChanged();
 
@@ -225,6 +243,14 @@ public Q_SLOTS:
     void slotSetDisplayProfile(const KoColorProfile *profile);
     void startUpdateInPatches(const QRect &imageRect);
 
+    void slotTrySwitchShapeManager();
+
+    /**
+     * Called whenever the configuration settings change.
+     */
+    void slotConfigChanged();
+
+
 private Q_SLOTS:
 
     /// The image projection has changed, now start an update
@@ -241,11 +267,6 @@ private Q_SLOTS:
      */
     void documentOffsetMoved(const QPoint &documentOffset);
 
-    /**
-     * Called whenever the configuration settings change.
-     */
-    void slotConfigChanged();
-
     void slotSelectionChanged();
 
     void slotDoCanvasUpdate();
@@ -287,6 +308,8 @@ private:
     // TODO: see to avoid that
     void setup();
 
+    void initializeFpsDecoration();
+
 private:
     friend class KisView; // calls setup()
     class KisCanvas2Private;
