@@ -27,7 +27,7 @@
 #include <kpagedialog.h>
 
 #include "kis_global.h"
-#include "widgets/squeezedcombobox.h"
+#include <squeezedcombobox.h>
 
 #include "ui_wdggeneralsettings.h"
 #include "ui_wdgdisplaysettings.h"
@@ -79,8 +79,14 @@ public:
     bool showCanvasMessages();
     bool compressKra();
     bool toolOptionsInDocker();
+    int kineticScrollingGesture();
+    int kineticScrollingSensitivity();
+    bool kineticScrollingScrollbar();
     bool switchSelectionCtrlAlt();
     bool convertToImageColorspaceOnImport();
+
+    bool calculateAnimationCacheInBackground();
+
 private Q_SLOTS:
     void getBackgroundImage();
     void clearBackgroundImage();
@@ -114,7 +120,7 @@ class ShortcutSettingsTab : public QWidget
 public:
 
     ShortcutSettingsTab(QWidget *parent = 0, const char *name = 0);
-    ~ShortcutSettingsTab();
+    ~ShortcutSettingsTab() override;
 
 public:
     void setDefault();
@@ -215,7 +221,7 @@ class PerformanceTab : public WdgPerformanceSettings
 public:
     PerformanceTab(QWidget *parent = 0, const char *name = 0);
 
-    ~PerformanceTab();
+    ~PerformanceTab() override;
 
     void load(bool requestDefault);
     void save();
@@ -224,11 +230,16 @@ private Q_SLOTS:
 
     void selectSwapDir();
 
+    void slotThreadsLimitChanged(int value);
+    void slotFrameClonesLimitChanged(int value);
+
 private:
     int realTilesRAM();
 
 private:
     QVector<SliderAndSpinBoxSync*> m_syncs;
+    int m_lastUsedThreadsLimit;
+    int m_lastUsedClonesLimit;
 };
 
 //=======================
@@ -307,7 +318,7 @@ public:
 protected:
 
     KisDlgPreferences(QWidget *parent = 0, const char *name = 0);
-    ~KisDlgPreferences();
+    ~KisDlgPreferences() override;
 
 protected:
 
