@@ -412,10 +412,6 @@ void _flush_fn(png_structp png_ptr)
 KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 {
     dbgFile << "Start decoding PNG File";
-    if (!iod->open(QIODevice::ReadOnly)) {
-        dbgFile << "Failed to open PNG File";
-        return (KisImageBuilder_RESULT_FAILURE);
-    }
 
     png_byte signature[8];
     iod->peek((char*)signature, 8);
@@ -538,21 +534,21 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     qDebug() << "sRGB" << sRGBIntent;
 #endif
 
-bool fromBlender = false;
+    bool fromBlender = false;
 
-png_text* text_ptr;
+    png_text* text_ptr;
     int num_comments;
-png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
+    png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
 
-        for (int i = 0; i < num_comments; i++) {
-            QString key = QString(text_ptr[i].key).toLower();
-            if (key == "file") {
-                QString relatedFile = text_ptr[i].text;
-                if (relatedFile.contains(".blend", Qt::CaseInsensitive)){
-                    fromBlender=true;
-                }
+    for (int i = 0; i < num_comments; i++) {
+        QString key = QString(text_ptr[i].key).toLower();
+        if (key == "file") {
+            QString relatedFile = text_ptr[i].text;
+            if (relatedFile.contains(".blend", Qt::CaseInsensitive)){
+                fromBlender=true;
             }
         }
+    }
 
     const KoColorProfile* profile = 0;
     if (png_get_iCCP(png_ptr, info_ptr, &profile_name, &compression_type, &profile_data, &proflen)) {
@@ -579,19 +575,7 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
                 QApplication::restoreOverrideCursor();
                 dlg.exec();
                 if (!dlg.profile().isEmpty()) {
-
-                    QString s = KoColorSpaceRegistry::instance()->colorSpaceId(csName.first, csName.second);
-
-                    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s);
-                    if (csf) {
-                        QList<const KoColorProfile *>  profileList = KoColorSpaceRegistry::instance()->profilesFor(csf);
-                        Q_FOREACH (const KoColorProfile *p, profileList) {
-                            if (p->name() == dlg.profile()) {
-                                profile = p;
-                                break;
-                            }
-                        }
-                    }
+                    profile = KoColorSpaceRegistry::instance()->profileByName(dlg.profile());
                 }
                 QApplication::setOverrideCursor(Qt::WaitCursor);
 
@@ -600,10 +584,11 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
         dbgFile << "no embedded profile, will use the default profile";
     }
 
+    const QString colorSpaceId =
+        KoColorSpaceRegistry::instance()->colorSpaceId(csName.first, csName.second);
+
     // Check that the profile is used by the color space
-    if (profile && !KoColorSpaceRegistry::instance()->colorSpaceFactory(
-                KoColorSpaceRegistry::instance()->colorSpaceId(
-                    csName.first, csName.second))->profileIsCompatible(profile)) {
+    if (profile && !KoColorSpaceRegistry::instance()->profileIsCompatible(profile, colorSpaceId)) {
         warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << csName.first << " " << csName.second;
         profile = 0;
     }
@@ -634,7 +619,7 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
         transform = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile)->createColorConverter(cs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
     }
 
-    // Creating the KisImageWSP
+    // Creating the KisImageSP
     if (m_image == 0) {
         m_image = new KisImage(m_doc->createUndoStore(), width, height, cs, "built image");
         Q_CHECK_PTR(m_image);
@@ -659,19 +644,20 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
         dbgFile << "There are " << num_comments << " comments in the text";
         for (int i = 0; i < num_comments; i++) {
             QString key = QString(text_ptr[i].key).toLower();
-            dbgFile << "key is |" << text_ptr[i].key << "| containing " << text_ptr[i].text << " " << (key ==  "Raw profile type exif ");
+            dbgFile << "key: " << text_ptr[i].key
+                    << ", containing: " << text_ptr[i].text
+                    << ": " << (key == "raw profile type exif " ? "isExif" : "something else");
             if (key == "title") {
                 info->setAboutInfo("title", text_ptr[i].text);
             } else if (key == "description") {
                 info->setAboutInfo("comment", text_ptr[i].text);
             } else if (key == "author") {
-        qDebug()<<"Author:"<<text_ptr[i].text;
                 info->setAuthorInfo("creator", text_ptr[i].text);
-            } else if (key.contains("Raw profile type exif")) {
+            } else if (key.contains("raw profile type exif")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "exif", 6);
-            } else if (key.contains("Raw profile type iptc")) {
+            } else if (key.contains("raw profile type iptc")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "iptc", 14);
-            } else if (key.contains("Raw profile type xmp")) {
+            } else if (key.contains("raw profile type xmp")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "xmp", 0);
             } else if (key == "version") {
                 m_image->addAnnotation(new KisAnnotation("kpp_version", "version", QByteArray(text_ptr[i].text)));
@@ -821,6 +807,11 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const QString &filename)
 
     QFile fp(filename);
     if (fp.exists()) {
+        if (!fp.open(QIODevice::ReadOnly)) {
+            dbgFile << "Failed to open PNG File";
+            return (KisImageBuilder_RESULT_FAILURE);
+        }
+
         return buildImage(&fp);
     }
     return (KisImageBuilder_RESULT_NOT_EXIST);
@@ -828,15 +819,13 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const QString &filename)
 }
 
 
-KisImageWSP KisPNGConverter::image()
+KisImageSP KisPNGConverter::image()
 {
     return m_image;
 }
 
 bool KisPNGConverter::saveDeviceToStore(const QString &filename, const QRect &imageRect, const qreal xRes, const qreal yRes, KisPaintDeviceSP dev, KoStore *store, KisMetaData::Store* metaData)
 {
-
-
     if (store->open(filename)) {
         KoStoreDevice io(store);
         if (!io.open(QIODevice::WriteOnly)) {
@@ -862,7 +851,6 @@ bool KisPNGConverter::saveDeviceToStore(const QString &filename, const QRect &im
             delete cmd;
         }
 
-
         bool success = pngconv.buildFile(&io, imageRect, xRes, yRes, dev, annotIt, annotIt, options, metaDataStore);
         if (success != KisImageBuilder_RESULT_OK) {
             dbgFile << "Saving PNG failed:" << filename;
@@ -888,6 +876,11 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const QString &filename, const
     dbgFile << "Start writing PNG File " << filename;
     // Open a QIODevice for writing
     QFile fp (filename);
+    if (!fp.open(QIODevice::WriteOnly)) {
+        dbgFile << "Failed to open PNG File for writing";
+        return (KisImageBuilder_RESULT_FAILURE);
+    }
+
     KisImageBuilder_Result result = buildFile(&fp, imageRect, xRes, yRes, device, annotationsStart, annotationsEnd, options, metaData);
 
     return result;
@@ -895,11 +888,6 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const QString &filename, const
 
 KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRect &imageRect, const qreal xRes, const qreal yRes, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
-    if (!iodevice->open(QIODevice::WriteOnly)) {
-        dbgFile << "Failed to open PNG File for writing";
-        return (KisImageBuilder_RESULT_FAILURE);
-    }
-
     if (!device)
         return KisImageBuilder_RESULT_INVALID_ARG;
 
@@ -913,10 +901,22 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
         device = tmp;
     }
 
+    if (device->colorSpace()->colorDepthId() == Float16BitsColorDepthID
+            || device->colorSpace()->colorDepthId() == Float32BitsColorDepthID
+            || device->colorSpace()->colorDepthId() == Float64BitsColorDepthID) {
+        const KoColorSpace *dstcs = KoColorSpaceRegistry::instance()->colorSpace(device->colorSpace()->colorModelId().id(), Integer16BitsColorDepthID.id(), device->colorSpace()->profile());
+        KisPaintDeviceSP tmp = new KisPaintDevice(dstcs);
+        KisPainter gc(tmp);
+        gc.bitBlt(imageRect.topLeft(), device, imageRect);
+        gc.end();
+        device = tmp;
+    }
+
     if (options.forceSRGB) {
         const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace(RGBAColorModelID.id(), device->colorSpace()->colorDepthId().id(), "sRGB built-in - (lcms internal)");
         device = new KisPaintDevice(*device);
-        device->convertTo(cs);
+        KUndo2Command *cmd = device->convertTo(cs);
+        delete cmd;
     }
 
     // Initialize structures
@@ -977,7 +977,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
         KisSequentialIterator it(device, imageRect);
 
         bool toomuchcolor = false;
-        do {
+        while (it.nextPixel()) {
             const quint8* c = it.oldRawData();
             bool findit = false;
             for (int i = 0; i < num_palette; i++) {
@@ -998,7 +998,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
                 palette[num_palette].blue = c[0];
                 num_palette++;
             }
-        }  while (it.nextPixel());
+        }
 
         if (!toomuchcolor) {
             dbgFile << "Found a palette of " << num_palette << " colors";
@@ -1083,24 +1083,44 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
 #endif
     }
 
-    // read comments from the document information
-    // warning: according to the official png spec, the keys need to be capitalised!
+    // save comments from the document information
+    // warning: according to the official png spec, the keys need to be capitalized!
     if (m_doc) {
-        png_text texts[3];
+        png_text texts[4];
         int nbtexts = 0;
         KoDocumentInfo * info = m_doc->documentInfo();
         QString title = info->aboutInfo("title");
-        if (!title.isEmpty()) {
+        if (!title.isEmpty() && options.storeMetaData) {
             fillText(texts + nbtexts, "Title", title);
             nbtexts++;
         }
-        QString abstract = info->aboutInfo("comment");
-        if (!abstract.isEmpty()) {
+        QString abstract = info->aboutInfo("subject");
+        if (abstract.isEmpty()) {
+            abstract = info->aboutInfo("abstract");
+        }
+        if (!abstract.isEmpty() && options.storeMetaData) {
+            QString keywords = info->aboutInfo("keyword");
+            if (!keywords.isEmpty()) {
+                abstract = abstract + " keywords: " + keywords;
+            }
             fillText(texts + nbtexts, "Description", abstract);
             nbtexts++;
         }
+
+        QString license = info->aboutInfo("license");
+        if (!license.isEmpty() && options.storeMetaData) {
+            fillText(texts + nbtexts, "Copyright", license);
+            nbtexts++;
+        }
+
         QString author = info->authorInfo("creator");
-        if (!author.isEmpty()) {
+        if (!author.isEmpty() && options.storeAuthor) {
+            if (!info->authorContactInfo().isEmpty()) {
+                QString contact = info->authorContactInfo().at(0);
+                if (!contact.isEmpty()) {
+                    author = author+"("+contact+")";
+                }
+            }
             fillText(texts + nbtexts, "Author", author);
             nbtexts++;
         }
@@ -1254,7 +1274,6 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
     if (color_type == PNG_COLOR_TYPE_PALETTE) {
         delete [] palette;
     }
-    iodevice->close();
     return KisImageBuilder_RESULT_OK;
 }
 
