@@ -29,6 +29,8 @@
 #include <QApplication>
 #include <QDesktopWidget>
 
+#include <Eigen/Core>
+
 #include <kis_icon.h>
 #include <KoPointerEvent.h>
 #include <KoViewConverter.h>
@@ -71,7 +73,7 @@ KisToolFreehand::KisToolFreehand(KoCanvasBase * canvas, const QCursor & cursor,
     m_only_one_assistant = true;
 
     setSupportOutline(true);
-    setMaskSyntheticEvents(true); // Disallow mouse events from finger presses.
+    setMaskSyntheticEvents(KisConfig().disableTouchOnCanvas()); // Disallow mouse events from finger presses unless enabled
 
     m_infoBuilder = new KisToolFreehandPaintingInformationBuilder(this);
     m_recordingAdapter = new KisRecordingAdapter();
@@ -88,6 +90,12 @@ KisToolFreehand::~KisToolFreehand()
     delete m_infoBuilder;
 }
 
+void KisToolFreehand::mouseMoveEvent(KoPointerEvent *event)
+{
+    KisToolPaint::mouseMoveEvent(event);
+    m_helper->cursorMoved(convertToPixelCoord(event));
+}
+
 KisSmoothingOptionsSP KisToolFreehand::smoothingOptions() const
 {
     return m_helper->smoothingOptions();
@@ -147,7 +155,8 @@ void KisToolFreehand::resetHelper(KisToolFreehandHelper *helper)
 
 int KisToolFreehand::flags() const
 {
-    return KisTool::FLAG_USES_CUSTOM_COMPOSITEOP|KisTool::FLAG_USES_CUSTOM_PRESET;
+    return KisTool::FLAG_USES_CUSTOM_COMPOSITEOP|KisTool::FLAG_USES_CUSTOM_PRESET
+           |KisTool::FLAG_USES_CUSTOM_SIZE;
 }
 
 void KisToolFreehand::activate(ToolActivation activation, const QSet<KoShape*> &shapes)
@@ -166,7 +175,9 @@ void KisToolFreehand::deactivate()
 
 void KisToolFreehand::initStroke(KoPointerEvent *event)
 {
-    m_helper->initPaint(event, canvas()->resourceManager(),
+    m_helper->initPaint(event,
+                        convertToPixelCoord(event),
+                        canvas()->resourceManager(),
                         image(),
                         currentNode(),
                         image().data());
@@ -180,7 +191,7 @@ void KisToolFreehand::doStroke(KoPointerEvent *event)
         m_helper->setCanvasHorizontalMirrorState(canvas2->xAxisMirrored());
         m_helper->setCanvasRotation(canvas2->rotationAngle());
     }
-    m_helper->paint(event);
+    m_helper->paintEvent(event);
 }
 
 void KisToolFreehand::endStroke()
@@ -201,8 +212,9 @@ void KisToolFreehand::beginPrimaryAction(KoPointerEvent *event)
     requestUpdateOutline(event->point, event);
 
     NodePaintAbility paintability = nodePaintAbility();
+    // XXX: move this to KisTool and make it work properly for clone layers: for clone layers, the shape paint tools don't work either
     if (!nodeEditable() || paintability != PAINT) {
-        if(paintability == KisToolPaint::VECTOR){
+        if (paintability == KisToolPaint::VECTOR || paintability == KisToolPaint::CLONE){
             KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
             QString message = i18n("The brush tool cannot paint on this layer.  Please select a paint layer or mask.");
             kiscanvas->viewManager()->showFloatingMessage(message, koIcon("object-locked"));
@@ -348,8 +360,7 @@ void KisToolFreehand::continueAlternateAction(KoPointerEvent *event, AlternateAc
     qreal scaleY = 0;
     canvas2->coordinatesConverter()->imageScale(&scaleX, &scaleY);
 
-    // we have no centralized knowledge of the maximum brush size!
-    const qreal maxBrushSize = 1000.0;
+    const qreal maxBrushSize = KisConfig().readEntry("maximumBrushSize", 1000);
     const qreal effectiveMaxDragSize = 0.5 * screenRect.width();
     const qreal effectiveMaxBrushSize = qMin(maxBrushSize, effectiveMaxDragSize / scaleX);
 
@@ -442,7 +453,7 @@ QPainterPath KisToolFreehand::getOutlinePath(const QPointF &documentPos,
                                              const KoPointerEvent *event,
                                              KisPaintOpSettings::OutlineMode outlineMode)
 {
-    QPointF imagePos = currentImage()->documentToPixel(documentPos);
+    QPointF imagePos = convertToPixelCoord(documentPos);
 
     if (currentPaintOpPreset())
         return m_helper->paintOpOutline(imagePos,
