@@ -75,6 +75,7 @@
 #include <kis_action_manager.h>
 #include <kis_action.h>
 #include "strokes/kis_color_picker_stroke_strategy.h"
+#include <kis_canvas_resource_provider.h>
 
 
 KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
@@ -94,7 +95,7 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
     m_supportOutline = false;
 
     {
-        const int maxSize = 1000;
+        int maxSize = KisConfig().readEntry("maximumBrushSize", 1000);
 
         int brushSize = 1;
         do {
@@ -169,16 +170,33 @@ void KisToolPaint::canvasResourceChanged(int key, const QVariant& v)
 
 void KisToolPaint::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
-    if (currentPaintOpPreset()) emit statusTextChanged(currentPaintOpPreset()->name());
+    if (currentPaintOpPreset()) {
+        QString formattedBrushName = currentPaintOpPreset()->name().replace("_", " ");
+        emit statusTextChanged(formattedBrushName);
+    }
+
     KisTool::activate(toolActivation, shapes);
-    connect(actions().value("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()), Qt::UniqueConnection);
-    connect(actions().value("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
+    if (flags() & KisTool::FLAG_USES_CUSTOM_SIZE) {
+        connect(action("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()), Qt::UniqueConnection);
+        connect(action("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
+    }
+
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    m_oldOpacity = provider->opacity();
+    provider->setOpacity(m_localOpacity);
 }
 
 void KisToolPaint::deactivate()
 {
-    disconnect(actions().value("increase_brush_size"), 0, this, 0);
-    disconnect(actions().value("decrease_brush_size"), 0, this, 0);
+    if (flags() & KisTool::FLAG_USES_CUSTOM_SIZE) {
+        disconnect(action("increase_brush_size"), 0, this, 0);
+        disconnect(action("decrease_brush_size"), 0, this, 0);
+    }
+
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    m_localOpacity = provider->opacity();
+    provider->setOpacity(m_oldOpacity);
+
     KisTool::deactivate();
 }
 
@@ -358,7 +376,7 @@ void KisToolPaint::addPickerJob(const PickingJob &pickingJob)
 
     KIS_ASSERT_RECOVER_RETURN(isPickingAction(pickingJob.action));
 
-    const QPoint imagePoint = image()->documentToIntPixel(pickingJob.documentPixel);
+    const QPoint imagePoint = image()->documentToImagePixelFloored(pickingJob.documentPixel);
     const bool fromCurrentNode = pickingJob.action == PickFgNode || pickingJob.action == PickBgNode;
     m_pickingResource = colorPreviewResourceId(pickingJob.action);
 
@@ -468,7 +486,7 @@ QWidget * KisToolPaint::createOptionWidget()
     QVBoxLayout* verticalLayout = new QVBoxLayout(optionWidget);
     verticalLayout->setObjectName("KisToolPaint::OptionWidget::VerticalLayout");
     verticalLayout->setContentsMargins(0,0,0,0);
-    verticalLayout->setSpacing(1);
+    verticalLayout->setSpacing(5);
 
     // See https://bugs.kde.org/show_bug.cgi?id=316896
     QWidget *specialSpacer = new QWidget(optionWidget);
@@ -482,7 +500,7 @@ QWidget * KisToolPaint::createOptionWidget()
 
     verticalLayout->addLayout(m_optionsWidgetLayout);
     m_optionsWidgetLayout->setContentsMargins(0,0,0,0);
-    m_optionsWidgetLayout->setSpacing(1);
+    m_optionsWidgetLayout->setSpacing(5);
 
     if (!quickHelp().isEmpty()) {
         QPushButton* push = new QPushButton(KisIconUtils::loadIcon("help-contents"), QString(), optionWidget);
@@ -604,12 +622,15 @@ void KisToolPaint::setupPaintAction(KisRecordedPaintAction* action)
 KisToolPaint::NodePaintAbility KisToolPaint::nodePaintAbility()
 {
     KisNodeSP node = currentNode();
-    if (!node || node->systemLocked()) {
+    if (!node) {
         return NONE;
     }
     if (node->inherits("KisShapeLayer")) {
         return VECTOR;
     }
+    if (node->inherits("KisCloneLayer")) {
+        return CLONE;
+    }
     if (node->paintDevice()) {
         return PAINT;
     }
@@ -736,20 +757,30 @@ void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const Ko
         colorPreviewDocUpdateRect.adjust(-xoffset,-yoffset,xoffset,yoffset);
     }
 
-    if (!m_oldColorPreviewUpdateRect.isEmpty()) {
-        canvas()->updateCanvas(m_oldColorPreviewUpdateRect);
-    }
+    // DIRTY HACK ALERT: we should fetch the assistant's dirty rect when requesting
+    //                   the update, instead of just dumbly update the entire canvas!
 
-    if (!m_oldOutlineRect.isEmpty()) {
-        canvas()->updateCanvas(m_oldOutlineRect);
-    }
+    KisCanvas2 * kiscanvas = dynamic_cast<KisCanvas2*>(canvas());
+    KisPaintingAssistantsDecorationSP decoration = kiscanvas->paintingAssistantsDecoration();
+    if (decoration && decoration->visible()) {
+        kiscanvas->updateCanvas();
+    } else {
+        // TODO: only this branch should be present!
+        if (!m_oldColorPreviewUpdateRect.isEmpty()) {
+            canvas()->updateCanvas(m_oldColorPreviewUpdateRect);
+        }
 
-    if (!outlineDocRect.isEmpty()) {
-        canvas()->updateCanvas(outlineDocRect);
-    }
+        if (!m_oldOutlineRect.isEmpty()) {
+            canvas()->updateCanvas(m_oldOutlineRect);
+        }
 
-    if (!colorPreviewDocUpdateRect.isEmpty()) {
-        canvas()->updateCanvas(colorPreviewDocUpdateRect);
+        if (!outlineDocRect.isEmpty()) {
+            canvas()->updateCanvas(outlineDocRect);
+        }
+
+        if (!colorPreviewDocUpdateRect.isEmpty()) {
+            canvas()->updateCanvas(colorPreviewDocUpdateRect);
+        }
     }
 
     m_oldOutlineRect = outlineDocRect;
