@@ -24,12 +24,14 @@
 #include <QLabel>
 #include <QGridLayout>
 
+#include <KoUnit.h>
 #include <KoShape.h>
 #include <KoGradientBackground.h>
 #include <KoCanvasBase.h>
 #include <KoShapeController.h>
 #include <KoColorBackground.h>
 #include <KoPatternBackground.h>
+#include <KoShapeStroke.h>
 #include <KoDocumentResourceManager.h>
 #include <KoPathShape.h>
 
@@ -47,6 +49,11 @@
 #include <recorder/kis_node_query_path.h>
 #include <recorder/kis_action_recorder.h>
 
+#include <KoSelectedShapesProxy.h>
+#include <KoSelection.h>
+#include <commands/KoKeepShapesSelectedCommand.h>
+
+
 KisToolShape::KisToolShape(KoCanvasBase * canvas, const QCursor & cursor)
         : KisToolPaint(canvas, cursor)
 {
@@ -125,6 +132,14 @@ KisPainter::StrokeStyle KisToolShape::strokeStyle(void)
     }
 }
 
+qreal KisToolShape::currentStrokeWidth() const
+{
+    const qreal sizeInPx =
+        canvas()->resourceManager()->resource(KisCanvasResourceProvider::Size).toReal();
+
+    return canvas()->unit().fromUserValue(sizeInPx);
+}
+
 void KisToolShape::setupPaintAction(KisRecordedPaintAction* action)
 {
     KisToolPaint::setupPaintAction(action);
@@ -170,21 +185,46 @@ void KisToolShape::addShape(KoShape* shape)
             shape->setBackground(QSharedPointer<KoShapeBackground>(0));
             break;
     }
-    KUndo2Command * cmd = canvas()->shapeController()->addShape(shape);
-    canvas()->addCommand(cmd);
+
+    switch (strokeStyle()) {
+    case KisPainter::StrokeStyleNone:
+        shape->setStroke(KoShapeStrokeModelSP());
+        break;
+    case KisPainter::StrokeStyleBrush: {
+        KoShapeStrokeSP stroke(new KoShapeStroke());
+        stroke->setLineWidth(currentStrokeWidth());
+        stroke->setColor(canvas()->resourceManager()->foregroundColor().toQColor());
+        shape->setStroke(stroke);
+        break;
+    }
+    }
+
+    KUndo2Command *parentCommand = new KUndo2Command();
+
+    KoSelection *selection = canvas()->selectedShapesProxy()->selection();
+    const QList<KoShape*> oldSelectedShapes = selection->selectedShapes();
+
+    // reset selection on the newly added shape :)
+    // TODO: think about moving this into controller->addShape?
+    new KoKeepShapesSelectedCommand(oldSelectedShapes, {shape}, selection, false, parentCommand);
+    KUndo2Command *cmd = canvas()->shapeController()->addShape(shape, 0, parentCommand);
+    parentCommand->setText(cmd->text());
+    new KoKeepShapesSelectedCommand(oldSelectedShapes, {shape}, selection, true, parentCommand);
+
+    canvas()->addCommand(parentCommand);
 }
 
 void KisToolShape::addPathShape(KoPathShape* pathShape, const KUndo2MagicString& name)
 {
     KisNodeSP node = currentNode();
-    if (!node || !blockUntillOperationsFinished()) {
+    if (!node || !blockUntilOperationsFinished()) {
         return;
     }
     // Get painting options
     KisPaintOpPresetSP preset = currentPaintOpPreset();
 
     // Compute the outline
-    KisImageWSP image = this->image();
+    KisImageSP image = this->image();
     QTransform matrix;
     matrix.scale(image->xRes(), image->yRes());
     matrix.translate(pathShape->position().x(), pathShape->position().y());
@@ -193,7 +233,8 @@ void KisToolShape::addPathShape(KoPathShape* pathShape, const KUndo2MagicString&
     // Recorde the paint action
     KisRecordedPathPaintAction bezierCurvePaintAction(
             KisNodeQueryPath::absolutePath(node),
-            preset );
+            preset,
+            KisDistanceInitInfo());
     bezierCurvePaintAction.setPaintColor(currentFgColor());
     QPointF lastPoint, nextPoint;
     int elementCount = mapedOutline.elementCount();
