@@ -59,7 +59,7 @@
 
 #include <KoFileDialog.h>
 #include <KoIconToolTip.h>
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QWidgetAction>
 
 #include "config-qtmultimedia.h"
@@ -183,13 +183,6 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     m_d->existingLayersMenu = m_d->layerEditingMenu->addMenu(KisAnimationUtils::addExistingLayerActionName);
     m_d->layerEditingMenu->addSeparator();
 
-    m_d->showHideLayerAction = new KisAction(KisAnimationUtils::showLayerActionName, this);
-    m_d->showHideLayerAction->setActivationFlags(KisAction::ACTIVE_LAYER);
-    connect(m_d->showHideLayerAction, SIGNAL(triggered()), SLOT(slotHideLayerFromTimeline()));
-    m_d->showHideLayerAction->setCheckable(true);
-    m_d->globalActions.insert("show_in_timeline", m_d->showHideLayerAction);
-    m_d->layerEditingMenu->addAction(m_d->showHideLayerAction);
-
     m_d->layerEditingMenu->addAction(KisAnimationUtils::removeLayerActionName, this, SLOT(slotRemoveLayer()));
 
     connect(m_d->existingLayersMenu, SIGNAL(aboutToShow()), SLOT(slotUpdateLayersMenu()));
@@ -210,7 +203,7 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     m_d->audioOptionsMenu = new QMenu(this);
 
 #ifndef HAVE_QT_MULTIMEDIA
-    m_d->audioOptionsMenu->addSection(i18nc("@item:inmenu", "Audio playback is not suported in this build!"));
+    m_d->audioOptionsMenu->addSection(i18nc("@item:inmenu", "Audio playback is not supported in this build!"));
 #endif
 
     m_d->openAudioAction= new QAction("XXX", this);
@@ -233,6 +226,7 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     m_d->volumeSlider->setSingleStep(1);
     m_d->volumeSlider->setPageStep(10);
     m_d->volumeSlider->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
+
     connect(m_d->volumeSlider, SIGNAL(valueChanged(int)), SLOT(slotAudioVolumeChanged(int)));
 
     QWidgetAction *volumeAction = new QWidgetAction(m_d->audioOptionsMenu);
@@ -292,6 +286,12 @@ QMap<QString, KisAction*> TimelineFramesView::globalActions() const
     return m_d->globalActions;
 }
 
+void TimelineFramesView::setShowInTimeline(KisAction* action)
+{
+    m_d->showHideLayerAction = action;
+    m_d->layerEditingMenu->addAction(m_d->showHideLayerAction);
+}
+
 void resizeToMinimalSize(QAbstractButton *w, int minimalSize) {
     QSize buttonSize = w->sizeHint();
     if (buttonSize.height() > minimalSize) {
@@ -349,7 +349,6 @@ void TimelineFramesView::setModel(QAbstractItemModel *model)
             &m_d->selectionChangedCompressor, SLOT(start()));
 
     connect(m_d->model, SIGNAL(sigEnsureRowVisible(int)), SLOT(slotEnsureRowVisible(int)));
-
     slotUpdateAudioActions();
 }
 
@@ -367,13 +366,13 @@ void TimelineFramesView::setFramesPerSecond(int fps)
 void TimelineFramesView::slotZoomButtonPressed(qreal staticPoint)
 {
     m_d->zoomStillPointIndex =
-        qIsNaN(staticPoint) ? currentIndex().column() : staticPoint;
+            qIsNaN(staticPoint) ? currentIndex().column() : staticPoint;
 
     const int w = m_d->horizontalRuler->defaultSectionSize();
 
     m_d->zoomStillPointOriginalOffset =
-        w * m_d->zoomStillPointIndex -
-        horizontalScrollBar()->value();
+            w * m_d->zoomStillPointIndex -
+            horizontalScrollBar()->value();
 }
 
 void TimelineFramesView::slotZoomButtonChanged(qreal zoomLevel)
@@ -402,7 +401,7 @@ void TimelineFramesView::slotSelectAudioChannelFile()
 {
     if (!m_d->model) return;
 
-    QString defaultDir = QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
+    QString defaultDir = QStandardPaths::writableLocation(QStandardPaths::MusicLocation);
 
     const QString currentFile = m_d->model->audioChannelFileName();
     QDir baseDir = QFileInfo(currentFile).absoluteDir();
@@ -427,6 +426,13 @@ void TimelineFramesView::slotAudioChannelMute(bool value)
     }
 }
 
+void TimelineFramesView::slotUpdateIcons()
+{
+    m_d->addLayersButton->setIcon(KisIconUtils::loadIcon("addlayer"));
+    m_d->audioOptionsButton->setIcon(KisIconUtils::loadIcon("audio-none"));
+    m_d->zoomDragButton->setIcon(KisIconUtils::loadIcon("zoom-horizontal"));
+}
+
 void TimelineFramesView::slotAudioChannelRemove()
 {
     if (!m_d->model) return;
@@ -478,8 +484,8 @@ void TimelineFramesView::slotUpdateInfiniteFramesCount()
 
     const int sectionWidth = m_d->horizontalRuler->defaultSectionSize();
     const int calculatedIndex =
-        (horizontalScrollBar()->value() +
-         m_d->horizontalRuler->width() - 1) / sectionWidth;
+            (horizontalScrollBar()->value() +
+             m_d->horizontalRuler->width() - 1) / sectionWidth;
 
     m_d->model->setLastVisibleFrame(calculatedIndex);
 }
@@ -510,27 +516,27 @@ QItemSelectionModel::SelectionFlags TimelineFramesView::selectionCommand(const Q
      */
 
     if (event &&
-        (event->type() == QEvent::MouseButtonPress ||
-         event->type() == QEvent::MouseButtonRelease) &&
-        index.isValid()) {
+            (event->type() == QEvent::MouseButtonPress ||
+             event->type() == QEvent::MouseButtonRelease) &&
+            index.isValid()) {
 
         const QMouseEvent *mevent = static_cast<const QMouseEvent*>(event);
 
         if (mevent->button() == Qt::RightButton &&
-            selectionModel()->selectedIndexes().contains(index)) {
+                selectionModel()->selectedIndexes().contains(index)) {
 
             // Allow calling context menu for multiple layers
             return QItemSelectionModel::NoUpdate;
         }
 
         if (event->type() == QEvent::MouseButtonPress &&
-            (mevent->modifiers() & Qt::ControlModifier)) {
+                (mevent->modifiers() & Qt::ControlModifier)) {
 
             return QItemSelectionModel::NoUpdate;
         }
 
         if (event->type() == QEvent::MouseButtonRelease &&
-            (mevent->modifiers() & Qt::ControlModifier)) {
+                (mevent->modifiers() & Qt::ControlModifier)) {
 
             return QItemSelectionModel::Toggle;
         }
@@ -584,8 +590,8 @@ void TimelineFramesView::slotDataChanged(const QModelIndex &topLeft, const QMode
 
     for (int j = topLeft.column(); j <= bottomRight.column(); j++) {
         QVariant value = m_d->model->data(
-            m_d->model->index(topLeft.row(), j),
-            TimelineFramesModel::ActiveFrameRole);
+                    m_d->model->index(topLeft.row(), j),
+                    TimelineFramesModel::ActiveFrameRole);
 
         if (value.isValid() && value.toBool()) {
             selectedColumn = j;
@@ -621,15 +627,12 @@ void TimelineFramesView::slotHeaderDataChanged(Qt::Orientation orientation, int
         if (newFps != m_d->fps) {
             setFramesPerSecond(newFps);
         }
-    } else /* if (orientation == Qt::Vertical) */ {
-        updateShowInTimeline();
     }
 }
 
 void TimelineFramesView::rowsInserted(const QModelIndex& parent, int start, int end)
 {
     QTableView::rowsInserted(parent, start, end);
-    updateShowInTimeline();
 }
 
 
@@ -794,8 +797,8 @@ void TimelineFramesView::startDrag(Qt::DropActions supportedActions)
         selectionModel()->clearSelection();
         Q_FOREACH (const QModelIndex &idx, selectionBefore) {
             QModelIndex newIndex =
-                model()->index(idx.row() + selectionOffset.y(),
-                               idx.column() + selectionOffset.x());
+                    model()->index(idx.row() + selectionOffset.y(),
+                                   idx.column() + selectionOffset.x());
             selectionModel()->select(newIndex, QItemSelectionModel::Select);
         }
     }
@@ -857,8 +860,8 @@ void TimelineFramesView::mousePressEvent(QMouseEvent *event)
         } else if (event->button() == Qt::LeftButton) {
             m_d->initialDragPanPos = event->pos();
             m_d->initialDragPanValue =
-                QPoint(horizontalScrollBar()->value(),
-                       verticalScrollBar()->value());
+                    QPoint(horizontalScrollBar()->value(),
+                           verticalScrollBar()->value());
         }
         event->accept();
 
@@ -867,15 +870,15 @@ void TimelineFramesView::mousePressEvent(QMouseEvent *event)
         int numSelectedItems = selectionModel()->selectedIndexes().size();
 
         if (index.isValid() &&
-            numSelectedItems <= 1 &&
-            m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
+                numSelectedItems <= 1 &&
+                m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
 
             model()->setData(index, true, TimelineFramesModel::ActiveLayerRole);
             model()->setData(index, true, TimelineFramesModel::ActiveFrameRole);
             setCurrentIndex(index);
 
             if (model()->data(index, TimelineFramesModel::FrameExistsRole).toBool() ||
-                model()->data(index, TimelineFramesModel::SpecialKeyframeExists).toBool()) {
+                    model()->data(index, TimelineFramesModel::SpecialKeyframeExists).toBool()) {
 
                 {
                     KisSignalsBlocker b(m_d->colorSelector);
@@ -931,7 +934,7 @@ void TimelineFramesView::mousePressEvent(QMouseEvent *event)
         }
 
         m_d->lastPressedPosition =
-            QPoint(horizontalOffset(), verticalOffset()) + event->pos();
+                QPoint(horizontalOffset(), verticalOffset()) + event->pos();
         m_d->lastPressedModifier = event->modifiers();
 
         QAbstractItemView::mousePressEvent(event);
@@ -1017,13 +1020,6 @@ void TimelineFramesView::slotLayerContextMenuRequested(const QPoint &globalPos)
     m_d->layerEditingMenu->exec(globalPos);
 }
 
-void TimelineFramesView::updateShowInTimeline()
-{
-    const int row = m_d->model->activeLayerRow();
-    const bool status = m_d->model->headerData(row, Qt::Vertical, TimelineFramesModel::LayerUsedInTimelineRole).toBool();
-    m_d->showHideLayerAction->setChecked(status);
-}
-
 void TimelineFramesView::slotAddNewLayer()
 {
     QModelIndex index = currentIndex();
@@ -1051,18 +1047,11 @@ void TimelineFramesView::slotRemoveLayer()
     model()->removeRow(index.row());
 }
 
-void TimelineFramesView::slotHideLayerFromTimeline()
-{
-    const int row = m_d->model->activeLayerRow();
-    const bool status = m_d->model->headerData(row, Qt::Vertical, TimelineFramesModel::LayerUsedInTimelineRole).toBool();
-    m_d->model->setHeaderData(row, Qt::Vertical, !status, TimelineFramesModel::LayerUsedInTimelineRole);
-}
-
 void TimelineFramesView::slotNewFrame()
 {
     QModelIndex index = currentIndex();
     if (!index.isValid() ||
-        !m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
+            !m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
 
         return;
     }
@@ -1074,7 +1063,7 @@ void TimelineFramesView::slotCopyFrame()
 {
     QModelIndex index = currentIndex();
     if (!index.isValid() ||
-        !m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
+            !m_d->model->data(index, TimelineFramesModel::FrameEditableRole).toBool()) {
 
         return;
     }
