@@ -31,7 +31,9 @@
 
 #include <kis_icon.h>
 #include "kis_action.h"
-#include <kis_resource_server_provider.h>
+#include <KisResourceServerProvider.h>
+#include <kconfiggroup.h>
+#include <ksharedconfig.h>
 
 #define ICON_SIZE 48
 
@@ -50,12 +52,12 @@ DlgBundleManager::DlgBundleManager(ResourceManager *resourceManager, KisActionMa
     setDefaultButton(Ok);
 
     m_ui->listActive->setIconSize(QSize(ICON_SIZE, ICON_SIZE));
-    m_ui->listActive->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    m_ui->listActive->setSelectionMode(QAbstractItemView::SingleSelection);
     connect(m_ui->listActive, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(itemSelected(QListWidgetItem*,QListWidgetItem*)));
     connect(m_ui->listActive, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(itemSelected(QListWidgetItem*)));
 
     m_ui->listInactive->setIconSize(QSize(ICON_SIZE, ICON_SIZE));
-    m_ui->listInactive->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    m_ui->listInactive->setSelectionMode(QAbstractItemView::SingleSelection);
     connect(m_ui->listInactive, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(itemSelected(QListWidgetItem*,QListWidgetItem*)));
     connect(m_ui->listInactive, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(itemSelected(QListWidgetItem*)));
 
@@ -92,12 +94,11 @@ DlgBundleManager::DlgBundleManager(ResourceManager *resourceManager, KisActionMa
 
 void DlgBundleManager::refreshListData()
 {
-    KoResourceServer<KisResourceBundle> *bundleServer = KisResourceServerProvider::instance()->resourceBundleServer();
+    KoResourceServer<KisResourceBundle> *bundleServer = KisResourceBundleServerProvider::instance()->resourceBundleServer();
 
     m_ui->listInactive->clear();
     m_ui->listActive->clear();
 
-
     Q_FOREACH (const QString &f, bundleServer->blackListedFiles()) {
         KisResourceBundle *bundle = new KisResourceBundle(f);
         bundle->load();
@@ -118,7 +119,7 @@ void DlgBundleManager::refreshListData()
 
 void DlgBundleManager::accept()
 {
-    KoResourceServer<KisResourceBundle> *bundleServer = KisResourceServerProvider::instance()->resourceBundleServer();
+    KoResourceServer<KisResourceBundle> *bundleServer = KisResourceBundleServerProvider::instance()->resourceBundleServer();
 
     for (int i = 0; i < m_ui->listActive->count(); ++i) {
         QListWidgetItem *item = m_ui->listActive->item(i);
@@ -139,30 +140,42 @@ void DlgBundleManager::accept()
         }
 
         if (bundle) {
-            if(!bundle->isInstalled()){
-                bundle->install();
-                //this removes the bundle from the blacklist and add it to the server without saving or putting it in front//
-                if(!bundleServer->addResource(bundle, false, false)){
-
-                    feedback = i18n("Couldn't add bundle to resource server");
-                    bundleFeedback.setText(feedback);
-                    bundleFeedback.exec();
-                }
-                if(!bundleServer->removeFromBlacklist(bundle)){
-                    feedback = i18n("Couldn't remove bundle from blacklist");
-                    bundleFeedback.setText(feedback);
-                    bundleFeedback.exec();
-                }
+            bool isKrita3Bundle = false;
+            if (bundle->filename().endsWith("Krita_3_Default_Resources.bundle")) {
+                isKrita3Bundle = true;
+                KConfigGroup group = KSharedConfig::openConfig()->group("BundleHack");
+                group.writeEntry("HideKrita3Bundle", false);
             }
             else {
-            bundleServer->removeFromBlacklist(bundle);
-            //let's asume that bundles who exist and are installed have to be removed from the blacklist, and if they were already this returns false, so that's not a problem.
+                if (!bundle->isInstalled()) {
+                    bundle->install();
+                    //this removes the bundle from the blacklist and add it to the server without saving or putting it in front//
+                    if (!bundleServer->addResource(bundle, false, false)){
+
+                        feedback = i18n("Couldn't add bundle to resource server");
+                        bundleFeedback.setText(feedback);
+                        bundleFeedback.exec();
+                    }
+                    if (!isKrita3Bundle) {
+                        if (!bundleServer->removeFromBlacklist(bundle)) {
+                            feedback = i18n("Couldn't remove bundle from blacklist");
+                            bundleFeedback.setText(feedback);
+                            bundleFeedback.exec();
+                        }
+                    }
+                }
+                else {
+                    if (!isKrita3Bundle) {
+                        bundleServer->removeFromBlacklist(bundle);
+                    }
+                    //let's assume that bundles that exist and are installed have to be removed from the blacklist, and if they were already this returns false, so that's not a problem.
+                }
             }
         }
         else{
-        QString feedback = i18n("Bundle doesn't exist!");
-        bundleFeedback.setText(feedback);
-        bundleFeedback.exec();
+            QString feedback = i18n("Bundle doesn't exist!");
+            bundleFeedback.setText(feedback);
+            bundleFeedback.exec();
 
         }
     }
@@ -171,11 +184,19 @@ void DlgBundleManager::accept()
         QListWidgetItem *item = m_ui->listInactive->item(i);
         QByteArray ba = item->data(Qt::UserRole).toByteArray();
         KisResourceBundle *bundle = bundleServer->resourceByMD5(ba);
-
-
-        if (bundle && bundle->isInstalled()) {
-            bundle->uninstall();
-            bundleServer->removeResourceAndBlacklist(bundle);
+        bool isKrits3Bundle = false;
+        if (bundle) {
+            if (bundle->filename().contains("Krita_3_Default_Resources.bundle")) {
+                isKrits3Bundle = true;
+                KConfigGroup group = KSharedConfig::openConfig()->group("BundleHack");
+                group.writeEntry("HideKrita3Bundle", true);
+            }
+            if (bundle->isInstalled()) {
+                bundle->uninstall();
+                if (!isKrits3Bundle) {
+                    bundleServer->removeResourceAndBlacklist(bundle);
+                }
+            }
         }
     }
 
@@ -185,7 +206,6 @@ void DlgBundleManager::accept()
 
 void DlgBundleManager::addSelected()
 {
-
     Q_FOREACH (QListWidgetItem *item, m_ui->listActive->selectedItems()) {
         m_ui->listInactive->addItem(m_ui->listActive->takeItem(m_ui->listActive->row(item)));
     }
@@ -218,7 +238,7 @@ void DlgBundleManager::itemSelected(QListWidgetItem *current, QListWidgetItem *)
     else {
 
         QByteArray ba = current->data(Qt::UserRole).toByteArray();
-        KoResourceServer<KisResourceBundle> *bundleServer = KisResourceServerProvider::instance()->resourceBundleServer();
+        KoResourceServer<KisResourceBundle> *bundleServer = KisResourceBundleServerProvider::instance()->resourceBundleServer();
         KisResourceBundle *bundle = bundleServer->resourceByMD5(ba);
 
         if (!bundle) {
@@ -231,16 +251,22 @@ void DlgBundleManager::itemSelected(QListWidgetItem *current, QListWidgetItem *)
             }
         }
 
+
         if (bundle) {
+            QFontMetrics metrics(this->font());
 
             m_currentBundle = bundle;
             m_ui->bnEditBundle->setEnabled(true);
 
             m_ui->lblName->setText(bundle->name());
-            m_ui->lblAuthor->setText(bundle->getMeta("author"));
-            m_ui->lblEmail->setText(bundle->getMeta("email"));
-            m_ui->lblLicense->setText(bundle->getMeta("license"));
-            m_ui->lblWebsite->setText(bundle->getMeta("website"));
+            m_ui->lblAuthor->setText(metrics.elidedText(bundle->getMeta("author"), Qt::ElideRight, m_ui->lblAuthor->width()));
+            m_ui->lblAuthor->setToolTip(bundle->getMeta("author"));
+            m_ui->lblEmail->setText(metrics.elidedText(bundle->getMeta("email"), Qt::ElideRight, m_ui->lblEmail->width()));
+            m_ui->lblEmail->setToolTip(bundle->getMeta("email"));
+            m_ui->lblLicense->setText(metrics.elidedText(bundle->getMeta("license"), Qt::ElideRight, m_ui->lblLicense->width()));
+            m_ui->lblLicense->setToolTip(bundle->getMeta("license"));
+            m_ui->lblWebsite->setText(metrics.elidedText(bundle->getMeta("website"), Qt::ElideRight, m_ui->lblWebsite->width()));
+            m_ui->lblWebsite->setToolTip(bundle->getMeta("website"));
             m_ui->lblDescription->setPlainText(bundle->getMeta("description"));
             m_ui->lblCreated->setText(bundle->getMeta("created"));
             m_ui->lblUpdated->setText(bundle->getMeta("updated"));
@@ -296,10 +322,13 @@ void DlgBundleManager::editBundle()
 {
     if (m_currentBundle) {
         DlgCreateBundle dlg(m_currentBundle);
+        m_activeBundles.remove(m_currentBundle->filename());
+        m_currentBundle = 0;
         if (dlg.exec() != QDialog::Accepted) {
             return;
         }
-        m_resourceManager->saveBundle(dlg);
+        m_currentBundle = m_resourceManager->saveBundle(dlg);
+        refreshListData();
     }
 }
 
@@ -310,6 +339,7 @@ void DlgBundleManager::fillListWidget(QList<KisResourceBundle *> bundles, QListW
 
     Q_FOREACH (KisResourceBundle *bundle, bundles) {
         QPixmap pixmap(ICON_SIZE, ICON_SIZE);
+        pixmap.fill(Qt::gray);
         if (!bundle->image().isNull()) {
             QImage scaled = bundle->image().scaled(ICON_SIZE, ICON_SIZE, Qt::KeepAspectRatio, Qt::SmoothTransformation);
             int x = (ICON_SIZE - scaled.width()) / 2;
@@ -318,9 +348,6 @@ void DlgBundleManager::fillListWidget(QList<KisResourceBundle *> bundles, QListW
             gc.drawImage(x, y, scaled);
             gc.end();
         }
-        else {
-            pixmap.fill(Qt::gray);
-        }
 
         QListWidgetItem *item = new QListWidgetItem(pixmap, bundle->name());
         item->setData(Qt::UserRole, bundle->md5());
@@ -371,6 +398,7 @@ void DlgBundleManager::slotCreateBundle() {
     if (m_actionManager) {
         KisAction *action = m_actionManager->actionByName("create_bundle");
         action->trigger();
+        refreshListData();
     }
 }
 
