@@ -38,6 +38,7 @@
 #include <SvgStyleParser.h>
 #include <SvgWriter.h>
 #include <SvgStyleWriter.h>
+#include <KisQPainterStateSaver.h>
 
 #include <klocalizedstring.h>
 
@@ -66,8 +67,30 @@ ArtisticTextShape::~ArtisticTextShape()
     }
 }
 
+KoShape *ArtisticTextShape::cloneShape() const
+{
+    ArtisticTextShape *clone = new ArtisticTextShape();
+    clone->m_ranges = m_ranges;
+    if (m_path) {
+        clone->m_path = static_cast<KoPathShape*>(m_path->cloneShape());
+    }
+    clone->m_charOutlines = m_charOutlines;
+    clone->m_startOffset = m_startOffset;
+    clone->m_outlineOrigin = m_outlineOrigin;
+    clone->m_outline = m_outline;
+    clone->m_baseline = m_baseline;
+    clone->m_textAnchor = m_textAnchor;
+    clone->m_charOffsets = m_charOffsets;
+    clone->m_charPositions = m_charPositions;
+    clone->m_textUpdateCounter = m_textUpdateCounter;
+    clone->m_defaultFont = m_defaultFont;
+    return clone;
+}
+
 void ArtisticTextShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
+    KisQPainterStateSaver saver(&painter);
+
     applyConversion(painter, converter);
     if (background()) {
         background()->paint(painter, converter, paintContext, outline());
@@ -76,14 +99,14 @@ void ArtisticTextShape::paint(QPainter &painter, const KoViewConverter &converte
 
 void ArtisticTextShape::saveOdf(KoShapeSavingContext &context) const
 {
-    SvgWriter svgWriter(QList<KoShape *>() << const_cast<ArtisticTextShape *>(this), size());
+    SvgWriter svgWriter(QList<KoShape *>() << const_cast<ArtisticTextShape *>(this));
     QByteArray fileContent;
     QBuffer fileContentDevice(&fileContent);
     if (!fileContentDevice.open(QIODevice::WriteOnly)) {
         return;
     }
 
-    if (!svgWriter.save(fileContentDevice)) {
+    if (!svgWriter.save(fileContentDevice, size())) {
         qWarning() << "Could not write svg content";
         return;
     }
@@ -578,7 +601,7 @@ bool ArtisticTextShape::putOnPath(KoPathShape *path)
     setTransformation(QTransform());
     updateSizeAndPosition();
     // move to correct position
-    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeftCorner);
+    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeft);
     update();
 
     return true;
@@ -601,7 +624,7 @@ bool ArtisticTextShape::putOnPath(const QPainterPath &path)
     setTransformation(QTransform());
     updateSizeAndPosition();
     // move to correct position
-    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeftCorner);
+    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeft);
     update();
 
     return true;
@@ -1052,7 +1075,7 @@ bool ArtisticTextShape::saveSvg(SvgSavingContext &context)
     if (layout() == ArtisticTextShape::Straight) {
         context.shapeWriter().addAttributePt("x", anchorOffset);
         context.shapeWriter().addAttributePt("y", baselineOffset());
-        context.shapeWriter().addAttribute("transform", SvgUtil::transformToString(transformation()));
+        SvgUtil::writeTransformAttributeLazy("transform", transformation(), context.shapeWriter());
         Q_FOREACH (const ArtisticTextRange &range, formattedText) {
             saveSvgTextRange(range, context, !hasSingleRange, baselineOffset());
         }
@@ -1219,7 +1242,7 @@ bool ArtisticTextShape::loadSvg(const KoXmlElement &textElement, SvgLoadingConte
 
                 path->setSize(newSize);
                 path->setPosition(newPosition);
-                path->applyAbsoluteTransformation(SvgUtil::parseTransform(p.attribute("transform")));
+                path->applyAbsoluteTransformation(context.currentGC()->matrix);
             }
         } else {
             path = dynamic_cast<KoPathShape *>(context.shapeById(href));
@@ -1354,8 +1377,11 @@ ArtisticTextRange ArtisticTextShape::createTextRange(const QString &text, Artist
     }
 
     range.setRotations(context.rotations(textLength));
+
+#if 0
     range.setLetterSpacing(gc->letterSpacing);
     range.setWordSpacing(gc->wordSpacing);
+
     if (gc->baselineShift == "sub") {
         range.setBaselineShift(ArtisticTextRange::Sub);
     } else if (gc->baselineShift == "super") {
@@ -1368,6 +1394,7 @@ ArtisticTextRange ArtisticTextShape::createTextRange(const QString &text, Artist
             range.setBaselineShift(ArtisticTextRange::Length, value);
         }
     }
+#endif
 
     //range.printDebug();
 
