@@ -35,9 +35,9 @@ class PSDLoader : public QObject {
 public:
 
     PSDLoader(KisDocument *doc);
-    virtual ~PSDLoader();
+    ~PSDLoader() override;
 
-    KisImageBuilder_Result buildImage(const QString &filename);
+    KisImageBuilder_Result buildImage(QIODevice *io);
 
     KisImageSP image();
 
@@ -47,7 +47,7 @@ public Q_SLOTS:
 
 private:
 
-    KisImageBuilder_Result decode(const QString &filename);
+    KisImageBuilder_Result decode(QIODevice *io);
 
 private:
 
