@@ -26,12 +26,13 @@ class KisExperimentPaintOpSettings : public KisNoSizePaintOpSettings
 {
 public:
     KisExperimentPaintOpSettings();
-    virtual ~KisExperimentPaintOpSettings();
+    ~KisExperimentPaintOpSettings() override;
 
-    bool paintIncremental();
-    QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode);
+    bool lodSizeThresholdSupported() const override;
+    bool paintIncremental() override;
+    QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode) override;
 
-    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings);
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
 
 private:
 
