@@ -21,27 +21,46 @@
 #include <kis_paintop_option.h>
 #include <kritapaintop_export.h>
 
-const QString AIRBRUSH_ENABLED = "AirbrushOption/isAirbrushing";
-const QString AIRBRUSH_RATE = "AirbrushOption/rate";
-
-class KisAirbrushWidget;
-
 /**
  * Allows the user to activate airbrushing of the brush mask (brush is painted at the same position over and over)
- * Rate is set in miliseconds.
+ * Rate is set in milliseconds.
  */
 class PAINTOP_EXPORT KisAirbrushOption : public KisPaintOpOption
 {
-public:
-    KisAirbrushOption(bool enabled = true);
-    ~KisAirbrushOption();
+    Q_OBJECT
 
-    void writeOptionSetting(KisPropertiesConfigurationSP setting) const;
-    void readOptionSetting(const KisPropertiesConfigurationSP setting);
+public:
+    KisAirbrushOption(bool enabled = true, bool canIgnoreSpacing = true);
+    ~KisAirbrushOption() override;
+
+    void writeOptionSetting(KisPropertiesConfigurationSP setting) const override;
+    void readOptionSetting(const KisPropertiesConfigurationSP setting) override;
+
+    /**
+     * Returns the airbrushing interval, in milliseconds. This value should be ignored if the
+     * KisAirbrushOption is not checked according to isChecked().
+     */
+    qreal airbrushInterval() const;
+
+    /**
+     * Returns true if regular distance-based spacing should be ignored and overridden by time-based
+     * spacing. This value should be ignored if the KisAirbrushOption is not checked according to
+     * isChecked().
+     */
+    bool ignoreSpacing() const;
+
+private Q_SLOTS:
+    void slotIntervalChanged();
+    void slotIgnoreSpacingChanged();
 
 private:
-    KisAirbrushWidget * m_optionWidget;
+    // Reads the airbrush interval from the GUI.
+    void updateInterval();
+    // Reads the "ignore spacing" setting from the GUI.
+    void updateIgnoreSpacing();
 
+    struct Private;
+    Private *const m_d;
 };
 
 #endif
