@@ -31,6 +31,8 @@
 #include <QGroupBox>
 #include <QRadioButton>
 
+#include "kis_canvas2.h"
+
 
 class KoCanvasBase;
 class MoveToolOptionsWidget;
@@ -45,6 +47,18 @@ public:
     KisToolMove(KoCanvasBase * canvas);
     ~KisToolMove() override;
 
+    /**
+     * @brief wantsAutoScroll
+     * reimplemented from KoToolBase
+     * there's an issue where autoscrolling with this tool never makes the
+     * stroke end, so we return false here so that users don't get stuck with
+     * the tool. See bug 362659
+     * @return false
+     */
+    bool wantsAutoScroll() const override {
+        return false;
+    }
+
 public Q_SLOTS:
     void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes) override;
     void deactivate() override;
@@ -83,6 +97,7 @@ public:
     void endAction(KoPointerEvent *event);
 
     void paint(QPainter& gc, const KoViewConverter &converter) override;
+    void initHandles(const KisNodeList &nodes);
 
     QWidget* createOptionWidget() override;
     void updateUIUnit(int newUnit);
@@ -131,6 +146,12 @@ private:
     int m_resolution;
 
     QAction *m_showCoordinatesAction;
+
+    KisCanvas2* m_canvas;
+
+    QPoint m_pos;
+    QRect m_handlesRect;
+    bool m_dragInProgress = false;
 };
 
 
